/* cuaderno19.c
 *  *
 *  * Copyright (C) 2001, 2002 Igalia, S.L..
 *  *
 *  * This program is free software; you can redistribute it and/or
 *  * modify it under the terms of version 2 of the GNU General Public
 *  * License as published by the Free Software Foundation.
 *  *
 *  * This program is distributed in the hope that it will be useful,
 *  * but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  * General Public License for more details.
 *  *
 *  * You should have received a copy of the GNU General Public
 *  * License along with this program; if not, write to the
 *  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  * Boston, MA 02111-1307, USA.
 *  *
 * Author: Igalia S.L.
 *  */

#include "config.h"
#include "cuaderno19.h"

#include "enterprise_model.h"
#include "invoice.h"
#include "control_util.h"
#include "parameter_manager.h"
#include "client_model.h"
#include "account_model.h"
#include "invoice_model.h"
#include "util.h"

#include <glib.h>
#include <time.h>

/**
 * I_BOOK_PRESENTER_HEADER_CODE:
 *
 * Define el cdigo de la cabecera de presentador del cuaderno 19.
 *
 */
#define I_BOOK_PRESENTER_HEADER_CODE "5180"
/**
 * I_BOOK_ISSUER_HEADER_CODE:
 *
 * Define el cdigo de la cabecera de ordenante del cuaderno 19.
 *
 */
#define I_BOOK_ISSUER_HEADER_CODE "5380"
/**
 * I_BOOK_INDIVIDUAL_MANDATORY_CODE:
 *
 * Define el cdigo del registro individual del cuaderno 19.
 *
 */
#define I_BOOK_INDIVIDUAL_MANDATORY_CODE "5680"
/**
 * P_ISSUER_TOTAL_CODE:
 *
 * Define el cdigo del total por ordenante del cuaderno 19.
 *
 */
#define I_BOOK_ISSUER_TOTAL_CODE "5880"
/**
 * I_BOOK_GENERAL_TOTAL_CODE:
 *
 * Define el cdigo del total por general del cuaderno 19.
 *
 */
#define I_BOOK_GENERAL_TOTAL_CODE "5980"

/**
 * I_BOOK_ISSUER_TOTAL:
 *
 * Define el cdigo del total por ordenante del cuaderno 19. DUPLICADO.
 *
 */
#define I_BOOK_ISSUER_TOTAL "5880"
/**
 * I_BOOK_GENERAL_TOTAL:
 *
 * Define el cdigo del total por general del cuaderno 19. DUPLICADO.
 *
 */
#define I_BOOK_GENERAL_TOTAL "5980"
/**
 * I_BOOK_STANDARD_SUFFIX:
 *
 * Indica el nmero de sufijo estndar que se inserta despues de los cdigos de cabeceras.
 *
 */
#define I_BOOK_STANDARD_SUFFIX "000"
/**
 * I_BOOK_FIRST_PROCEDURE:
 *
 * Cdigo que identifica el procedimiento primero.
 *
 */
#define I_BOOK_FIRST_PROCEDURE "01"

/**
 * I_BOOK19_FILE:
 *
 * Parte general del nombre del fichero en el que se generar el cuaderno 19.
 *
 */
#define I_BOOK19_FILE "cuaderno19"

/**
 * I_BOOK_DECIMAL_CHARACTER:
 *
 * Indica cual es el carcter utilizado para los decimales.
 *
 */
#define I_BOOK_DECIMAL_CHARACTER ","

typedef struct _file_and_total {
  FILE * out_file;
  GList * totals_list;
} FileAndTotal;

static void book19_write_each_issuer(gchar* company_id, GList * invoices, FileAndTotal * file_and_total);
static void book19_write_each_individual(Invoice * invoice, FILE * out_file, gchar * issuer_code, gdouble * importe_individual);
static gchar * convert_to_string_without_decimal_character(gdouble number, gint length);
static gdouble calculate_total_from_gdouble_glist(GList * list);

/* Returns the gdouble with the sum over all the elements in the GList */
/* It is assumed that the list is not NULL */
static gdouble calculate_total_from_gdouble_glist(GList * list) {
  GList * element;
  gdouble total = 0;
  gdouble * total_pointer;

  element = g_list_first(list);

  if (element == NULL) {
    return 0;
  }
  else {
    do {
      total_pointer = (gdouble *) element->data;
      total = total + (*total_pointer);
      element = g_list_next(element);    
    } 
    while (element != NULL);
  }
  
  return total;
} 


/* Converts a gdouble to a string of size 'length' without decimal delimiter */
static gchar * convert_to_string_without_decimal_character(gdouble number, gint length)
{
  gchar * string;
  gchar ** total_tokens;
  gchar * total_without_decimal_character;

  string = g_strdup_printf("%.2f", number);
  total_tokens = g_strsplit (string,I_BOOK_DECIMAL_CHARACTER,2); 
  g_free(string);

  string = g_strconcat(total_tokens[0], total_tokens[1],NULL);
  g_strfreev(total_tokens);

  total_without_decimal_character = u_string_fill_left ('0', length, string);

  return total_without_decimal_character;
}


static void free_gdouble_list (GList * list) {
  GList * element;

  element = g_list_first(list);

  do {
    g_free(element->data);
    element = g_list_next(element);
  }
  while (element != NULL);

  g_list_free(list);
}

/**
 * i_book19_generate_from_invoice_list:
 * @invoices: Arra de facturas que contiene todas las facturas que estn actualmente en memoria.
 * @selected: #GArray que contiene la lista de facturas seleccionadas.
 *
 * Genera el cuaderno 19 a partir de la lista de facturas y de las seleccionadas de la misma. El fichero
 * generado es cuaderno19-fecha_de_hoy. La lista de facturas se recorre y las que se han seleccionado se almacenan
 * en una hash segn el cliente ordenante. Se procesan y se escribe el fichero con el cuaderno 19.
 *
 */
void i_book19_generate_from_invoice_list (Invoice ** invoices, GArray * selected) {
  
  g_print(_("(cuaderno19.c) -cuaderno19_generate_from_invoice_list- Generating Cuaderno19...\n"));
  
  if (invoices != NULL) {
    FILE * out_file;
    gint index = 0; 
    gchar * company_id;
    GList * content = NULL;
    gchar * presenter_header;
    gchar * presenter_client_name;
    gchar * presenter_client_name_filled;
    gchar * today_char;
    gchar * receiver_entity;
    gchar * office;    
    time_t timer;
    gchar * spaces6;
    gchar * spaces20;
    gchar * spaces66;
    GHashTable *hash_table = g_hash_table_new(g_str_hash, g_str_equal);
    GList * account_list;
    GList * element;
    Participant * participant;
    gchar * presenter_legal_id;    
    FileAndTotal * file_and_total;

    gchar * general_total;
    gchar * spaces52;
    gchar * issuer_number;
    gchar * spaces16;
    gchar * total_sum;
    gchar * direct_debit_total_number;
    gchar * spaces38;
    gchar * records_total_number;

    gchar * book19_file;
    gint i;

    /* Obtaining the date for today as a string in today_char */
    today_char=(gchar *) g_malloc(sizeof(gchar)*10);
    timer=time(NULL);
    strftime(today_char,sizeof(gchar)*10,"%d%m%y",localtime(&timer));
    
    book19_file = g_strconcat(AA_DATA_DIR,"/",I_BOOK19_FILE,"-",today_char,NULL);
    out_file = fopen(book19_file, "w");       
    g_print(_("(cuaderno19.c) -cuaderno19_generate_from_invoice_list- The file will be written at: %s"), book19_file);
    g_free(book19_file);

    g_print(_("(cuaderno19.c) -cuaderno19_generate_from_invoice_list- Obtaining account data\n"));

    /* We take the PRESENTADOR from the configuration parameters of the app */ 
    presenter_client_name = u_parameter_manager_get_parameter ("empresa");
    participant = m_client_get_participant_by_name (presenter_client_name,M_CLIENT_ENTERPRISE_CODE,TRUE);
    presenter_legal_id= u_string_fill_left ('0',9,g_strdup_printf("%s",d_participant_get_legal_id (participant)));

 
    
    account_list = m_account_find_by_participant_code (d_participant_get_code (participant));
    d_participant_clear (participant); /* Was created by get_participant_by_name */

    element = g_list_first(account_list);

    if (element != NULL) {      
      receiver_entity = g_strdup_printf("%04d",d_office_get_bank_code (d_account_get_office (element->data)));
      office = g_strdup_printf("%04d",d_office_get_office_code (d_account_get_office (element->data)));   
      d_account_free_glist (account_list); /* Was created by model_find_account_by_party_code */
    }
    else {
      c_util_open_info_window (_("The host being processed hasn't got bank account information\n Question marks are going to be added at the correspondet place in the debits file"));         
      receiver_entity = g_strdup("????");
      office = g_strdup("????");
    }
      
    /* Obtaining the date for today as a string in today_char */
    today_char=(gchar *) g_malloc(sizeof(gchar)*10);
    timer=time(NULL);
    strftime(today_char,sizeof(gchar)*10,"%d%m%y",localtime(&timer));
    
    spaces6 = g_strnfill(6,' ');
    spaces20 = g_strnfill(20,' ');  
    spaces66 = g_strnfill(66,' '); 

    presenter_client_name_filled = g_strdup_printf("%-40s",presenter_client_name);
    g_free(presenter_client_name);

    g_print(_("(cuaderno19.c) -cuaderno19_generate_from_invoice_list- Generating 'presenter_header'\n"));

    /* Standard head for the PRESENTADOR in 'Cuaderno 19' */
    presenter_header =
      g_strconcat(
		  I_BOOK_PRESENTER_HEADER_CODE, /* (4) */
		  presenter_legal_id, /* (9) */
		  I_BOOK_STANDARD_SUFFIX, /* (3) */
		  today_char, /* Current time (6) DDMMAA */
		  spaces6, /* (6) */
		  presenter_client_name_filled, /* Nombre del cliente presentador (40) */
		  spaces20, /* */
		  receiver_entity, /* */
		  office,
		  spaces66, /* (12+40+14) */
		  "\r\n",
		  NULL);  

    g_free(today_char);
    g_free(presenter_client_name_filled);
    g_free(spaces20);
    g_free(receiver_entity);
    g_free(office);
    g_free(spaces66);    
    
    fprintf(out_file, presenter_header);
    g_free(presenter_header);

    g_print(_("(cuaderno19.c) -cuaderno19_generate_from_invoice_list- 'presenter_header' generated succesfully\n"));
    
    g_print(_("(cuaderno19.c) -cuaderno19_generate_from_invoice_list- The list of invoices is not empty\n"));

    for (i=0; i< selected->len; i++) {
      index  = g_array_index(selected,gint,i); /* Index takes only the selected invoices */

      if (d_payment_get_is_direct_debit (d_invoice_get_payment (invoices[index]))
	  &&
	  (d_invoice_get_state (invoices[index]) != D_INVOICE_PAID)
	  &&
	  (d_invoice_get_state (invoices[index]) != D_INVOICE_MANAGING)
	  ) {
	g_print(_("(cuaderno19.c) -cuaderno19_generate_from_invoice_list-  The invoice number %d is a payment by 'domiciliacion'\n"), index);
	
	/* Gets the identifier of the company that created the current invoice */
        company_id = g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_invoice_get_enterprise (invoices[index]))));
	content = g_hash_table_lookup(hash_table, company_id);	       	
	g_hash_table_insert (hash_table, company_id, g_list_append(content,invoices[index]));
      }
    }	     
    
    g_print(_("(cuaderno19.c) -cuaderno19_generate_from_invoice_list- After processing the invoices, the size of the hash table: %d\n"),
	    g_hash_table_size(hash_table));
    
    if (g_hash_table_size(hash_table) == 0) c_util_open_error_window (_("There aren't any selected invoices are debits don't paid\n cuaderno19 will be generated empty."));

    /* Now we are going to process each of the entries in the hash table and their associated list of invoices */

    /* file_and_total is used for passing foreach functional argument one parameter and to obtain a total as result */
    file_and_total = (FileAndTotal *) g_malloc0(sizeof(FileAndTotal));
    file_and_total->out_file = out_file;
    file_and_total->totals_list = NULL;
    g_hash_table_foreach(hash_table, (void *) (*book19_write_each_issuer), file_and_total);      

    g_print(_("(cuaderno19.c) -cuaderno19_generate_from_invoice_list- Generating 'total general'\n"));

    spaces52 = g_strnfill(52, ' ');
    issuer_number = g_strdup_printf("%04d",g_hash_table_size(hash_table));
    spaces16 = g_strnfill(16, ' ');

    /* Adds all the 'totals' returned by the different 'write_each_ordenante' functions */
    total_sum = convert_to_string_without_decimal_character(calculate_total_from_gdouble_glist(file_and_total->totals_list), 10);

    direct_debit_total_number = g_strdup_printf("%010d", selected->len);
    records_total_number = g_strdup_printf("%010d", selected->len + g_hash_table_size(hash_table)*2 + 2 ); 
                                                   /* FIXME: Could change if using the 'complex' method */

    spaces38 = g_strnfill(38, ' ');

    general_total = 
      g_strconcat(
		  I_BOOK_GENERAL_TOTAL_CODE,
		  presenter_legal_id, /* (9) */
		  I_BOOK_STANDARD_SUFFIX, /* (3) */
		  spaces52, 
		  issuer_number,
		  spaces16,
		  total_sum,
		  spaces6,
		  direct_debit_total_number,
		  records_total_number,
		  spaces38,
		  "\r\n",
		  NULL);


    g_free(presenter_legal_id);
    g_free(spaces52);
    g_free(issuer_number);
    g_free(spaces16);
    g_free(total_sum);
    g_free(spaces6);
    g_free(direct_debit_total_number);
    g_free(records_total_number);
    g_free(spaces38);

    fprintf(out_file, general_total);
    g_free(general_total);    
    
    g_print(_("(cuaderno19.c) -cuaderno19_generate_from_invoice_list- 'total general' succesfully generated\n"));    

    g_print(_("(cuaderno19.c) -cuaderno19_generate_from_invoice_list- Cuaderno19 generated\n"));  
    
    fclose(out_file);
    if (file_and_total->totals_list) free_gdouble_list(file_and_total->totals_list); /* NULL if empty cuaderno19 file */
    g_free(file_and_total);    
    g_hash_table_destroy(hash_table); 

  }
  else c_util_open_info_window (_("It's impossible to generate Cuaderno19 file from a list of empty invoices."));   
}


/* Frees the memory used by company_id and the GList BUT NOT invoices thenselves */ 
static void book19_write_each_issuer(gchar* company_id, GList * invoices, FileAndTotal * file_and_total) {  
  gchar * issuer_header;
  Participant * company;
  time_t timer;
  gchar * today_char;
  GList * element;
  gint shop_code;
  Enterprise * enterprise;
  gchar * filled_bank_code ;
  gchar * filled_office_code ;
  gchar * filled_number ;
  gchar * complete_account_number;
  gchar * control_digit;
  gchar * issuer_code;
  gchar * spaces6;
  gchar * spaces8;
  gchar * spaces10;
  gchar * spaces40;
  gchar * spaces14;
  gchar * issuer_client_name;
  gint counter;
  gchar * issuer_total_sum;
  gdouble individual_total;

  gdouble * total_pointer_double;
  gdouble total;

  gchar * direct_debit_number_gchar;
  gchar * record_number;
  gchar * issuer_total;
  gchar * spaces72;
  gchar * spaces38;

  GList * account_list;
  GList * element_account;

  g_print("(cuaderno19.c) -write_each_ordenante- Starting to process one of the 'ordenantes'\n");

  /* Getting the shop code in order to be able to obtain the enterprise and company for this invoice */
  /* In 'element->data' we have the invoice object */

  element = g_list_first(invoices);

  shop_code = d_participant_get_code (d_shop_get_participant (d_invoice_get_shop (element->data)));

  /* We need this because the Enterprise is not in memory (no included in de do_search result set) */
  enterprise = m_enterprise_find_by_shop (shop_code);

  company = d_enterprise_get_participant (enterprise);

  /* Obtaining the date for today as a string in today_char */
  today_char=(gchar *) g_malloc(sizeof(gchar)*10);
  timer=time(NULL);
  strftime(today_char,sizeof(gchar)*10,"%d%m%y",localtime(&timer));

 /* Obtaining the CCC for this client (included the calculation of the control code) */
  account_list = m_account_find_by_participant_code (d_participant_get_code (company));
  if (account_list == NULL) 
    {
      g_print(_("(cuaderno19.c) -write_each_ordenante- This orderer hasn't got accounts !!!\n"));
      c_util_open_info_window (_("The orderer being processed hasn't got bank account information\n Question marks are going to be added at the correspondent place in the debits file."));   
      complete_account_number = g_strdup("????????????????????");
    }
  else {
    element_account = g_list_first(account_list);
    filled_bank_code = g_strdup_printf("%04d",d_office_get_bank_code (d_account_get_office (element_account->data)));
    filled_office_code = g_strdup_printf("%04d",d_office_get_office_code (d_account_get_office (element_account->data)));   
    filled_number = u_string_fill_left ('0',10,g_strdup_printf("%s",d_account_get_account_number (element_account->data)));  
    control_digit = u_calculate_control_digit (filled_bank_code,filled_office_code,filled_number);
    complete_account_number = g_strconcat(filled_bank_code, filled_office_code, control_digit, filled_number, NULL);
    d_account_free_glist (account_list); /* Was created by model_find_account_by_party_code */
    
    g_free(filled_bank_code);
    g_free(filled_office_code);
    g_free(filled_number);
    g_free(control_digit);
  }

  issuer_code = u_string_fill_left ('0',9,d_participant_get_legal_id (company));
  spaces8 = g_strnfill(8,' ');  
  spaces10 = g_strnfill(10,' ');
  spaces40 = g_strnfill(40,' ');
  spaces14 = g_strnfill(14,' ');  
  
  issuer_client_name = g_strdup_printf("%-40s",d_participant_get_nick (company));
  
  g_print(_("(cuaderno19.c) -write_each_ordenante- Generating 'cabecera_de_ordenante'\n"));
  
  /* Standard head for the ORDENANTE in 'Cuaderno 19' */
  issuer_header =
    g_strconcat(
		I_BOOK_ISSUER_HEADER_CODE, /* (4) */
		issuer_code, /* (9) */
		I_BOOK_STANDARD_SUFFIX, /* (3) */
		today_char, /* Current time (6) DDMMAA */
		today_char, /* Fecha de cargo (6) DDMMAA */
		issuer_client_name, /* Nombre del cliente ordenante (40) */
		complete_account_number, /* CCC (4+4+2+10) */
		spaces8, /* (8) */
		I_BOOK_FIRST_PROCEDURE, /* (2) */
		spaces10, /* (10) */
		spaces40, /* (40) */
		spaces14, /* (14) */
		"\r\n",
		NULL);   

  g_free(issuer_client_name);
  g_free(complete_account_number);
  g_free(spaces8);
  g_free(spaces10);
  g_free(spaces40);
  g_free(spaces14);
  
  fprintf(file_and_total->out_file, issuer_header);
  g_free(issuer_header);
  
  
  g_free(today_char);
  g_print(_("(cuaderno19.c) -write_each_ordenante- 'cabecera de ordenante' successfully generated\n"));
  
  counter = 0;
  total = 0;
  g_print(_("(cuaderno19.c) -write_each_ordenante- Organizing the invoices in the hash_table\n"));

  do { /* For each of the invoices */
    book19_write_each_individual(element->data, file_and_total->out_file, issuer_code, &individual_total); 
    counter++;
    total = total + individual_total;
    element = g_list_next(element);
  }
  while (element != NULL); 
    
  g_print(_("(cuaderno19.c) -write_each_ordenante- Organized the invoices in the hash_table\n"));

  total_pointer_double = (gdouble *) g_malloc0(sizeof(gdouble));  
  (*total_pointer_double) = total;
  file_and_total->totals_list = g_list_append(file_and_total->totals_list , total_pointer_double);
    
  issuer_total_sum = convert_to_string_without_decimal_character(total, 10);
  direct_debit_number_gchar = g_strdup_printf("%010d", counter);  
  record_number =  g_strdup_printf("%010d", counter+2); /* FIXME: This could change if using the complex method */
  /* +2 because the header and total are included */
  
  spaces72 = g_strnfill(72,' ');  
  spaces6 = g_strnfill(6,' ');
  spaces38 = g_strnfill(38,' ');

  g_print(_("(cuaderno19.c) -write_each_ordenante- Generating total line for this ordenante\n"));
  
  issuer_total = 
    g_strconcat(
		I_BOOK_ISSUER_TOTAL_CODE,
		issuer_code, /* (9) */
		I_BOOK_STANDARD_SUFFIX, /* (3) */		
		spaces72, /* (12 + 40 +20) */
		issuer_total_sum, /* (10) */
		spaces6, /* (6) */
		direct_debit_number_gchar, /* (10) */
		record_number, /* (10) */
		spaces38,
		"\r\n",
		NULL);		

  g_free(issuer_code);
  g_free(spaces72);
  g_free(issuer_total_sum);
  g_free(spaces6);
  g_free(direct_debit_number_gchar);
  g_free(record_number);
  g_free(spaces38);
  
  fprintf(file_and_total->out_file, issuer_total);
  g_print(_("(cuaderno19.c) -write_each_ordenante- Generated total line for this ordenante\n"));  

  g_free(issuer_total);
  g_list_free(invoices); /* Freing the GList structure */    
  
  d_enterprise_clear (enterprise);
}

static void book19_write_each_individual(Invoice * invoice, FILE * out_file, gchar * issuer_code, gdouble * individual_total)
{
  gchar * compulsory_individual;
  gchar * filled_bank_code ;
  gchar * filled_office_code ;
  gchar * filled_number ;
  gchar * complete_account_number;
  gchar * control_digit;
  gchar * reference_code;
  gchar * holder_name;
  gchar * total;
  gchar * total_without_dot;
  gchar * return_code;
  gchar * section;
  gchar * spaces8;
  
  gchar ** total_tokens;

  gchar * issue_date;
  gchar * desc;
  time_t date;

  filled_bank_code = g_strdup_printf("%04d",d_payment_get_bank_code (d_invoice_get_payment (invoice)));
  filled_office_code = g_strdup_printf("%04d",d_payment_get_office_code (d_invoice_get_payment (invoice)));
  filled_number = u_string_fill_left ('0',10,g_strdup_printf("%s",d_payment_get_account_number (d_invoice_get_payment (invoice))));  
  control_digit = u_calculate_control_digit (filled_bank_code,filled_office_code,filled_number);
  complete_account_number = g_strconcat(filled_bank_code, filled_office_code, control_digit, filled_number, NULL);

  g_free(filled_bank_code);
  g_free(filled_office_code);
  g_free(filled_number);
  g_free(control_digit);

  reference_code = g_strdup_printf("%012lu", d_participant_get_code (d_invoice_get_participant (invoice)));
  holder_name = g_strdup_printf("%-40s", d_participant_get_nick (d_invoice_get_participant (invoice)));

  (*individual_total) = d_invoice_get_total (invoice); /* Return parameter of the function */

  total = g_strdup_printf("%011.2f", d_invoice_get_total (invoice));
  total_tokens = g_strsplit (total,",",2); 
  g_free(total);
  total_without_dot = g_strconcat(total_tokens[0], total_tokens[1],NULL);
  g_strfreev(total_tokens);


  return_code = g_strdup_printf("%016lu", d_invoice_get_invoice_code (invoice));

  issue_date=(gchar *) g_malloc(sizeof(gchar)*20);
  date = d_invoice_get_issue_date (invoice);
  strftime(issue_date,sizeof(gchar)*20,"%d-%m-%Y",localtime(&date));
  desc = g_strconcat(_("Repair."), 
		     issue_date, 
		     d_participant_get_nick (d_shop_get_participant (d_invoice_get_shop (invoice))), 
		     NULL);

  g_free(issue_date);
  section = g_strdup_printf("%-40s", desc);  /* FIXMENOW: Hablar con Tere para seleccionar el concepto */
  g_free(desc);

  g_print(section);

  spaces8 = g_strnfill(8,' ');

  compulsory_individual =    
    g_strconcat(
		I_BOOK_INDIVIDUAL_MANDATORY_CODE,
		issuer_code, /* NIF */
		I_BOOK_STANDARD_SUFFIX, /* Sufijo */	
		reference_code, /* Codigo de referencia (12) */
		holder_name, /* Nombre del titular (40)  */				
		complete_account_number, /* CCC (20) */				
		total_without_dot, /* Importe (10) */		
		return_code, /* Codigo Devoluciones  & Codigo Referencia (6+10) */		  		
		section, /* Primer campo de concepto (40) */
		spaces8, /* Libre (8) */
		"\r\n",
		NULL);

  g_free(reference_code);
  g_free(holder_name);
  g_free(complete_account_number);
  g_free(total_without_dot);
  g_free(return_code);
  g_free(section);
  g_free(spaces8);

  fprintf(out_file, compulsory_individual);        

  g_free(compulsory_individual);

  /* Update the invoice in the DB as 'en gestion' intermediate state */
  m_invoice_update_state (invoice, D_INVOICE_MANAGING);
}

