
/* print_utils.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "print_utils.h"

/* IMPLEMENTATION */

/**
 * p_print_show_xy_and_move:
 * @pc: Contexto en el que se indicar lo que se quiere imprimir.
 * @font: Tipo de fuente actual del contexto.
 * @x: Punto horizonal en el que empieza a imprimir.
 * @y: Punto vertical en el que se imprimir el texto.
 * @text: Texto a imprimir.
 *
 * Imprime un texto en un punto y despus coloca el puntero del contexto en el punto en el
 * que finaliz la impresin del texto.
 *
 */
void p_print_show_xy_and_move (GnomePrintContext *pc, GnomeFont *font, double x, double y, char * text)
{
   gnome_print_moveto (pc,x,y);
   p_print_show (pc, text);
   gnome_print_moveto (pc, x + gnome_font_get_width_utf8(font,text), y);
}

/**
 * p_print_n_text:
 * @pc: Contexto en el que se indicar lo que se quiere imprimir.
 * @font: Tipo de fuente actual del contexto.
 * @n: Nmero de palabras que se imprimirn.
 * @x: Punto horizontal en el que se empezar a imprimir.
 * @y: Punto vertical en el que se empezar a imprimir.
 * @texts: Los n textos que se imprimirn desde el punto (#x,#y) hasta el punto (#x,#y+#z) donde #z
 * es el numero de textos por la altura de las palabras al imprimir. Es decir, se va moviendo el
 * puntero verticalmente hacia abajo segn se imprimen los textos.
 *
 * Imprime #n frases en filas, empezando en el punto(#x,#y) y acabando en el punto (#x,#y+#z) donde #z
 * es el numero de textos por la altura de las palabras al imprimir
 *
 */
void p_print_n_text (GnomePrintContext *pc, GnomeFont *font, double n, double x, double y, char **texts)
{
   int i;
   int newX = x;
   
   for (i=0; i<n; i++)
     {
	gnome_print_moveto (pc, newX, y);	
	p_print_show (pc, texts[i]);
	newX = newX + gnome_font_get_width_utf8(font, texts[i]);
     }   
}

/**
 * p_print_two_text:
 * @pc: Contexto en el que se indicar lo que se quiere imprimir.
 * @font: Tipo de fuente actual del contexto.
 * @x: Punto horizontal en el que se empezar a imprimir.
 * @y: Punto vertical en el que se empezar a imprimir.
 * @text1: Primer texto a imprimir.
 * @text2: Segundo texto a imprimir.
 *
 * Imprime dos textos en horizontal. Primero escribe uno y luego escribe el otro donde acab el primero.
 *
 */
void p_print_two_text (GnomePrintContext *pc, GnomeFont *font,  double x, double y, char *text1, char *text2)
{
   gnome_print_moveto (pc, x, y);
   p_print_show (pc, text1);
   gnome_print_moveto (pc, x + gnome_font_get_width_utf8(font, text1), y);
   p_print_show (pc, text2);   
}

/**
 * p_print_centered_text:
 * @pc: Contexto en el que se indicar lo que se quiere imprimir.
 * @font: Tipo de fuente actual del contexto.
 * @x: Punto horizontal en el que se empezar a imprimir.
 * @y: Punto vertical en el que se empezar a imprimir.
 * @width: Anchura del recuadro imaginario en el que se desea centrar.
 * @text: Texto a imprimir.
 *
 * Imprime un texto centrado desde #x a #x+#width.
 *
 */
void p_print_centered_text (GnomePrintContext *pc, GnomeFont *font,
			  double x, double y, double width, char* text)
{   
   int w = gnome_font_get_width_utf8 (font, text);
   gnome_print_moveto (pc, x+(width-w)/2, y);
   p_print_show (pc, text);
}

/**
 * p_print_text_righted:
 * @pc: Contexto en el que se indicar lo que se quiere imprimir.
 * @font: Tipo de fuente actual del contexto.
 * @x: Punto horizontal en el que se empezar a imprimir.
 * @y: Punto vertical en el que se empezar a imprimir.
 * @width: Anchura del texto.
 * @text: Texto a imprimir.
 *
 * Imprime un text alineado a la derecha de la linea imaginaria formada por #x y #x+#width .
 *
 */
void p_print_text_righted (GnomePrintContext *pc, GnomeFont *font,
			  double x ,double y,double width, char* text)

{
   int w = gnome_font_get_width_utf8 (font, text);
   gnome_print_moveto (pc, x+width-w, y);
   p_print_show (pc, text);
}

/**
 * p_print_box:
 * @pc: Contexto en el que se indicar lo que se quiere imprimir.
 * @x: Punto horizontal en el que se empezar a imprimir.
 * @y: Punto vertical en el que se empezar a imprimir.
 * @width: Ancho del cuadro.
 * @height: Alto del cuadro.
 *
 * Pinta un cuadrado transparente en el punto (#x,#y) con la altura #height
 * y con la anchura #width.
 *
 */
void p_print_box (GnomePrintContext *pc, double x, double y,
		       double width, double height)
{
   gnome_print_moveto (pc,x,y);
   gnome_print_lineto (pc,x+width,y);
   gnome_print_lineto (pc,x+width, y-height);
   gnome_print_lineto (pc,x, y-height);
   gnome_print_closepath (pc);
   gnome_print_stroke (pc);
}

/**
 * p_black_box:
 * @pc: Contexto en el que se indicar lo que se quiere imprimir.
 * @x: Punto horizontal en el que se empezar a imprimir.
 * @y: Punto vertical en el que se empezar a imprimir.
 * @width: Ancho del cuadro.
 * @height: Alto del cuadro.
 *
 * Pinta un cuadrado negro en el punto (#x,#y) con la altura #height
 * y con la anchura #width.
 *
 */
void p_black_box (GnomePrintContext *pc, double x, double y,
		       double width, double height)
{
   gnome_print_moveto (pc,x,y);
   gnome_print_lineto (pc,x+width,y);
   gnome_print_lineto (pc,x+width, y-height);
   gnome_print_lineto (pc,x, y-height);
   gnome_print_closepath (pc);
   gnome_print_fill(pc);
}

/**
 * p_text_centered_in_box:
 * @pc: Contexto en el que se indicar lo que se quiere imprimir.
 * @font: Tipo de fuente actual del contexto.
 * @x: Punto horizontal en el que se empezar a imprimir.
 * @y: Punto vertical en el que se empezar a imprimir.
 * @width: Ancho del cuadro.
 * @height: Alto del cuadro.
 * @text: Texto a imprimir.
 *
 * Imprime un texto centrado y un recuadro, donde el texto aparece centrado en el recuadro.
 * El recuadro se imprime en el punto (#x,#y) y con anchura #width y altura #height.
 *
 */
void p_text_centered_in_box (GnomePrintContext *pc, GnomeFont *font,
					double x, double y, double width, double height,
					char* text)
{       
   int texth;
   
   p_print_box (pc, x, y, width, height);
   texth = gnome_font_get_size(font);
   p_print_centered_text (pc, font, x, y-(height+texth)/2, width, text);
}

/**
 * p_print_table:
 * @pc: Contexto en el que se indicar lo que se quiere imprimir.
 * @font: Tipo de fuente actual del contexto.
 * @x: Punto horizontal en el que se empezar a imprimir.
 * @y: Punto vertical en el que se empezar a imprimir.
 * @numRows: Nmero de filas de la tabla.
 * @width: Anchura de la tabla.
 * @rowHeight: Altura de cada celda de la tabla.
 * @contents: Contenidos de la tabla. Doble puntero ya que contiene los textos a imprimir.
 *
 * Imprime una tabla de una columna con el texto centrado como elemento de cada celda.
 * La tabla se imprime desde el punto (#x,#y) con la anchura #width y la altura
 * #numRows*#rowHeight.
 *
 */
void p_print_table (GnomePrintContext *pc, GnomeFont *font, double x, double y,
		  double numRows, double width, double rowHeight, char ** contents)
{
   int i;   
   p_print_box (pc, x, y, width, numRows*rowHeight);   
   p_print_centered_text (pc, font, x+P_CELL_LEFT_MARGIN, 
			y-rowHeight+P_CELL_BOTTOM_MARGIN, width, contents[0]);
      
   for (i=1; i<numRows; i++)
     {	
	int newY = y - rowHeight*i;
	gnome_print_moveto (pc, x, newY);
	gnome_print_lineto (pc, x+width, newY);
	gnome_print_moveto (pc, x+P_CELL_LEFT_MARGIN,
			    newY-rowHeight+P_CELL_BOTTOM_MARGIN);	
	p_print_show (pc, contents[i]);
     }
   
   gnome_print_stroke (pc);
}


/**
 * p_print_table_with_condition:
 * @pc: Contexto en el que se indicar lo que se quiere imprimir.
 * @font: Tipo de fuente actual del contexto.
 * @x: Punto horizontal en el que se empezar a imprimir.
 * @y: Punto vertical en el que se empezar a imprimir.
 * @numRows: Nmero de filas de la tabla.
 * @width: Anchura de la tabla.
 * @rowHeight: Altura de cada celda de la tabla.
 * @contents: Contenidos de la tabla. Doble puntero ya que contiene los textos a imprimir.
 * @show: Permite decidir si se muestran las lneas de la tabla.
 *
 * Imprime una tabla de una columna con el texto centrado como elemento de cada celda.
 * La tabla se imprime desde el punto (#x,#y) con la anchura #width y la altura
 * #numRows*#rowHeight.
 * Adems, permite elegir si
 * se imprimen las lneas de la tabla.
 *
 */
void p_print_table_with_condition (GnomePrintContext *pc, GnomeFont *font, double x, double y,
		  double numRows, double width, double rowHeight, char ** contents,gboolean show)
{
   int i;   
   if (show!=FALSE) {
       p_print_box (pc, x, y, width, numRows*rowHeight);
   }
   p_print_centered_text (pc, font, x+P_CELL_LEFT_MARGIN, 
			y-rowHeight+P_CELL_BOTTOM_MARGIN, width, contents[0]);
      
   for (i=1; i<numRows; i++)
     {	
	int newY = y - rowHeight*i;
	gnome_print_moveto (pc, x, newY);
	if (show!=FALSE) {
	    gnome_print_lineto (pc, x+width, newY);
	}
	gnome_print_moveto (pc, x+P_CELL_LEFT_MARGIN,
			    newY-rowHeight+P_CELL_BOTTOM_MARGIN);	
	p_print_show (pc, contents[i]);
     }
   if (show!=FALSE)
          gnome_print_stroke (pc);
}

/**
 * p_print_complex_table:
 * @pc: Contexto en el que se indicar lo que se quiere imprimir.
 * @font: Tipo de fuente actual del contexto.
 * @x: Punto horizontal en el que se empezar a imprimir.
 * @y: Punto vertical en el que se empezar a imprimir.
 * @numRows: Nmero de filas de la tabla.
 * @numCols: Nmero de columnas de la tabla.
 * @colWidth: Anchura de la tabla.
 * @rowHeight: Altura de cada celda de la tabla.
 * @contents: Contenidos de la tabla. Doble puntero ya que contiene los textos a imprimir.
 * @numRowsWithContent: Nmero de filas con contenido.
 * @printLines: Permite decidir si se imprimen las lneas.
 *
 * Imprime una tabla con columnas y filas que comienza en el punto (#x,#y) y termina
 * en el punto (#x+#numCols*#colWidth,#y+#numRows*#rowHeigth). Permite decidir si se desean
 * imprimir las lneas de la tabla.
 *
 */
void p_print_complex_table (GnomePrintContext *pc, GnomeFont *font, 
			  double x, double y,
			 double numRows, double numCols,
			  double *colWidth, double rowHeight, 
			  char **contents, double numRowsWithContent, 
			  double printLines)
{
   int i,j;   
   int fontSize = gnome_font_get_size(font);     
   int acumWidth;

   /* Print all the matrix values */
   acumWidth = 0;
   for (j=0; j<numCols; j++)
     {
       /* Print the header for this column */
       p_print_centered_text (pc, font, x+acumWidth, 
			    y-rowHeight+(rowHeight-fontSize)/2, 
			    colWidth[j], contents[j]);
       /* Print the rest of the values for this column */
       for (i=1; i<numRowsWithContent; i++)      
       {	   
	 gnome_print_moveto (pc, x+acumWidth+P_CELL_LEFT_MARGIN,
			     y-(i+1)*rowHeight+(rowHeight-fontSize)/2);	
	 p_print_show (pc, contents[i*(int)numCols+j]);
       }
       acumWidth += colWidth[j];
     }
   
   p_print_box (pc, x, y, acumWidth, numRows*rowHeight);  

   /* Print the horizontal line for the header */
   gnome_print_moveto (pc, x, y-rowHeight);
   gnome_print_lineto (pc, x+acumWidth, y-rowHeight);

   /* Print all horizontal lines (excluded the header) */
   if (printLines)
     {
       for (j=2; j<numRows; j++)
	 {
	   gnome_print_moveto (pc, x, y-rowHeight*j);
	   gnome_print_lineto (pc, x+acumWidth, y-rowHeight*j);
	 };
     };

   /* Print all vertical lines */
   acumWidth = 0;
   for (j=0; j<numCols; j++)
     {                     
       acumWidth += colWidth[j];
       gnome_print_moveto(pc, x+acumWidth, y);
       gnome_print_lineto(pc, x+acumWidth, y-numRows*rowHeight);
     };
   
   gnome_print_stroke (pc);
}


/**
 * p_print_complex_table_normal:
 * @pc: Contexto en el que se indicar lo que se quiere imprimir.
 * @font: Tipo de fuente actual del contexto.
 * @x: Punto horizontal en el que se empezar a imprimir.
 * @y: Punto vertical en el que se empezar a imprimir.
 * @numRows: Nmero de filas de la tabla.
 * @numCols: Nmero de columnas de la tabla.
 * @colWidth: Anchura de la tabla.
 * @rowHeight: Altura de cada celda de la tabla.
 * @contents: Contenidos de la tabla. Doble puntero ya que contiene los textos a imprimir.
 * @numRowsWithContent: Nmero de filas con contenido.
 * @printLines: Permite decidir si se imprimen las lneas.
 *
 * Imprime una tabla con columnas y filas que comienza en el punto (#x,#y) y termina
 * en el punto (#x+#numCols*#colWidth,#y+#numRows*#rowHeigth). Permite decidir si se desean
 * imprimir las lneas de la tabla. Esta funcin est implementada para la platilla normal
 * de forma que cuando la descripcin es demasiado larga se imprime debajo lo que queda.
 *
 */
void p_print_normal_complex_table (GnomePrintContext *pc, GnomeFont *font,
			  double x, double y,
			 double numRows, double numCols,
			  double *colWidth, double rowHeight, 
			  char **contents, double numRowsWithContent, 
				 double printLines)
{
    int i,j;
    int fontSize = gnome_font_get_size(font);
    int acumWidth;
    gchar * descripcion=NULL;
    gchar * descripcion1=NULL;
    gchar * descripcion2=NULL;
    gint * fin=(gint *) g_malloc0(sizeof(gint));
    gint lengthDesc;
    /* Print all the matrix values */
    acumWidth = 0;
    for (j=0; j<numCols; j++)
    {
	/* Print the header for this column */
	p_print_centered_text (pc, font, x+acumWidth,
			     y-rowHeight+(rowHeight-fontSize)/2,
			     colWidth[j], contents[j]);
	/* Print the rest of the values for this column */
	for (i=1; i<numRowsWithContent; i++)
	{
	    descripcion = contents[i*(int)numCols+j];
	    g_strstrip(descripcion);
	    if (j==1) {
		lengthDesc = strlen(descripcion);
		if (lengthDesc>40) {
		    font = gnome_font_new_closest ("Helvetica", GNOME_FONT_BOOK, 1, 8);
		    if (font == NULL)
		 {
		     g_warning(_("Font cannot be found"));
		 }
		    gnome_print_setfont (pc, font);
		}
		if (lengthDesc<=40) {
		    descripcion1 = g_strdup(descripcion);
		} else {
                    gint fin2;
		    *fin = 40;

		    descripcion1 = u_get_substring_complete_words (descripcion,0,fin);
		    fin2 = *fin;
                    *fin = strlen(descripcion);
		    descripcion2 = u_get_substring_complete_words (descripcion,fin2,fin);
                    g_print("\n%s\n",descripcion2);
		}




	    } else {
		descripcion1 = g_strdup(descripcion);
	    }
	    gnome_print_moveto (pc, x+acumWidth+P_CELL_LEFT_MARGIN,
				y-(i+1)*rowHeight+(rowHeight-fontSize)/2);
	    p_print_show (pc, descripcion1);
	    if (descripcion2!=NULL) {
		gnome_print_moveto (pc, x+acumWidth+P_CELL_LEFT_MARGIN,
				    y-(i+1)*rowHeight-10+(rowHeight-fontSize)/2);
		p_print_show (pc, descripcion2);
	    }
	    if (descripcion1!=NULL) {
		g_free(descripcion1);
                descripcion1=NULL;
	    }
	    if (descripcion2!=NULL) {
		g_free(descripcion2);
                descripcion2=NULL;
	    }
	    font = gnome_font_new_closest ("Helvetica", GNOME_FONT_BOOK, 1, 9);
	    if (font == NULL)
	    {
		g_warning(_("Font cannot be found"));
	    }
	    gnome_print_setfont (pc, font);

	}
	acumWidth += colWidth[j];
     }

    p_print_box (pc, x, y, acumWidth, numRows*rowHeight);

    /* Print the horizontal line for the header */
   gnome_print_moveto (pc, x, y-rowHeight);
   gnome_print_lineto (pc, x+acumWidth, y-rowHeight);

   /* Print all horizontal lines (excluded the header) */
   if (printLines)
   {
       for (j=2; j<numRows; j++)
       {
	   gnome_print_moveto (pc, x, y-rowHeight*j);
	   gnome_print_lineto (pc, x+acumWidth, y-rowHeight*j);
       };
   };

   /* Print all vertical lines */
   acumWidth = 0;
   for (j=0; j<numCols; j++)
   {
       acumWidth += colWidth[j];
       gnome_print_moveto(pc, x+acumWidth, y);
       gnome_print_lineto(pc, x+acumWidth, y-numRows*rowHeight);
   };

   gnome_print_stroke (pc);
}




/* tomada de gnumeric/src/print-cell.c --- NO EST SIENDO USADA*/
static inline void
  print_hline (GnomePrintContext *context,
	       float x1, float x2, float y)
{    
   gnome_print_moveto (context, x1, y);
   gnome_print_lineto (context, x2, y);
   gnome_print_stroke (context);
}

/**
 * p_print_show:
 * @pc: Contexto en el que se indicar lo que se quiere imprimir.
 * @text: Tipo de fuente actual del contexto.
 *
 * Imprime un texto en el punto actual del cursor del contexto.
 * El texto es transformado a UTF8 con respecto a que est definido %_PROPER_I18N
 * y sino devuelve el texto en ISO8859.
 *
 * Returns:
 * La anchura del texto en UTF8 o en ISO8859.
 */
int  p_print_show (GnomePrintContext *pc, char const *text)
{   
#ifdef _PROPER_I18N
   wchar_t* wcs,wcbuf[4096];
   char* utf8,utf8buf[4096];
   
   size_t conv_status;
   int n = strlen (text);
   int retval;
   const int wcbuf_len = sizeof (wcbuf) / sizeof (wcbuf[0]);
      
   g_return_val_if_fail (pc && text, -1);
   
   if ( n > wcbuf_len)
     wcs = g_new (wchar_t,n);
   else
     wcs = wcbuf;
   
   conv_status = mbstowcs (wcs, text, n);
   
   if (conv_status == (size_t)(-1))
     {	
	if (wcs != wcbuf)
	  g_free (wcs);
	return 0;
     };
   if (conv_status * 6 > sizeof (utf8buf))
     utf8 = g_new (gchar, conv_status * 6);
   else
     utf8 = utf8buf;
   
     {	
	size_t i;
	char* p = utf8;
	for(i = 0; i < conv_status; ++i)
	  p += g_unichar_to_utf8 ( (gint) wcs[i], p);
	if (wcs != wcbuf)
	  g_free (wcs);
	retval = gnome_print_show_sized (pc, utf8, p - utf8);
     }      
   
   if (utf8 != utf8buf)
     g_free (utf8);
   return retval;
#else
   return print_show_iso8859_1 (pc, text);
#endif
};
			   					 
/**
 * p_print_image:
 * @ctx: Contexto en el que se indicar lo que se quiere imprimir.
 * @x: Punto horizontal en el que se empezar a imprimir.
 * @y: Punto vertical en el que se empezar a imprimir.
 * @filename: Fichero con el nombre de la imagen.
 * @scale: Procentaje con el que se desea escalar la imagen.
 *
 * Imprime una imagen escalada en tamao desde el punto (#x,#y).
 *
 */
void p_print_image (GnomePrintContext *ctx, double x, double y, char* filename, float scale)
{
   GdkPixbuf *pixbuf;

   gnome_print_gsave (ctx);
   g_print(":%s:",filename);
   pixbuf = gdk_pixbuf_new_from_file (filename);
   if (pixbuf == NULL)
     {
     g_warning (_("Error trying to create a pixbuf from the given filename"));	
     return;
     }

   gnome_print_translate (ctx, x, y-gdk_pixbuf_get_height(pixbuf)*scale);
   gnome_print_rotate (ctx,90);

   gnome_print_scale (ctx, gdk_pixbuf_get_width(pixbuf)*scale, gdk_pixbuf_get_height(pixbuf)*scale);
   /*my_print_image_from_pixbuf(ctx,pixbuf); */
   gnome_print_pixbuf (ctx, pixbuf);
   gnome_print_grestore (ctx);
}

/**
 * p_print_image_filling:
 * @ctx: Contexto en el que se indicar lo que se quiere imprimir.
 * @x: Punto horizontal en el que se empezar a imprimir.
 * @y: Punto vertical en el que se empezar a imprimir.
 * @filename: Path del fichero que contiene la imagen.
 *
 * Imprime una imagen rotandola y moviendola para fijarla en el cuadro de la plantilla de Royal.
 *
 */
void p_print_image_filling (GnomePrintContext *ctx, double x, double y, char* filename)
{
    GdkPixbuf *pixbuf;
    gdouble height,width;
    gdouble scale_h = 1.0;
    gdouble scale_w = 1.0;
   gnome_print_gsave (ctx);
   pixbuf = gdk_pixbuf_new_from_file (filename);
   if (pixbuf == NULL)
     {
     g_warning (_("Error trying to create a pixbuf from the given filename"));	
     return;
     }

//   gnome_print_translate (ctx, x, y-gdk_pixbuf_get_height(pixbuf)*scale_h);
   gnome_print_translate (ctx, x, y);
   gnome_print_rotate (ctx,90);

   height = gdk_pixbuf_get_height(pixbuf);
   width = gdk_pixbuf_get_width(pixbuf);
   scale_h = 250 / height;
   scale_w = 250 / width;
   if (scale_h>scale_w) {
       scale_h = scale_w;
   } else {
       scale_w = scale_h;
   }
   gnome_print_scale (ctx, gdk_pixbuf_get_width(pixbuf)*scale_w, gdk_pixbuf_get_height(pixbuf)*scale_h);
   /*my_print_image_from_pixbuf(ctx,pixbuf); */
   gnome_print_pixbuf (ctx, pixbuf);
   gnome_print_grestore (ctx);
}


/**
 * p_print_images_horizontal:
 * @ctx: Contexto en el que se indicar lo que se quiere imprimir.
 * @x: Punto horizontal en el que se empezar a imprimir.
 * @y: Punto vertical en el que se empezar a imprimir.
 * @filename: Path del fichero que contiene la primera imagen.
 * @filename2: Path del fichero que contiene la segunda imagen.
 *
 * Imprime una o dos imgenes dentro del cuadro de la plantilla de Royal,
 * orientadas horizontalmente. Cuando en path es %NULL entonces solo se imprime una
 * encajada al cuadro, y los dos path's son distinto de %NULL se encajan las dos juntas.
 *
 */
void p_print_images_horizontal (GnomePrintContext *ctx, double x, double y, char* filename,char * filename2)
{
    GdkPixbuf *pixbuf=NULL;
    gdouble height,width;
    gdouble scale_h = 1.0;
    gdouble scale_w = 1.0;
    gnome_print_gsave (ctx);
    if (filename!=NULL) {
	pixbuf = gdk_pixbuf_new_from_file (filename);
    } else if (filename2!=NULL) {
	pixbuf = gdk_pixbuf_new_from_file (filename2);

    }
       g_print(_("\nFile1: %s\n"),filename);
       g_print(_("\nFile2: %s\n"),filename2);

    if (pixbuf == NULL)
     {
     g_warning (_("Error trying to create a pixbuf from the given filename"));	
     return;
     }

//   gnome_print_translate (ctx, x, y-gdk_pixbuf_get_height(pixbuf)*scale_h);
   gnome_print_translate (ctx, x, y);
//   gnome_print_rotate (ctx,90);

   height = gdk_pixbuf_get_height(pixbuf);
   width = gdk_pixbuf_get_width(pixbuf);
   if (((filename2 ==NULL) && (filename!=NULL)) || ((filename2!=NULL) && (filename==NULL))){
       scale_h = 250 / height;
       scale_w = 250 / width;
       if (scale_h>scale_w) {
	   scale_h = scale_w;
       } else {
	   scale_w = scale_h;
       }
       gnome_print_scale (ctx, gdk_pixbuf_get_width(pixbuf)*scale_w, gdk_pixbuf_get_height(pixbuf)*scale_h);
       /*my_print_image_from_pixbuf(ctx,pixbuf); */
       gnome_print_pixbuf (ctx, pixbuf);
   } else if ((filename!=NULL) && (filename2!=NULL)) {
       GdkPixbuf *pixbuf2;
       gdouble height2=0.0;
       gdouble width2=0.0;
       gdouble scale_h2 = 1.0;
       gdouble scale_w2 = 1.0;


       scale_h = 165 / height;
       scale_w = 165 / width;
       if (scale_h>scale_w) {
	   scale_h = scale_w;
       } else {
	   scale_w = scale_h;
       }
       pixbuf2 = gdk_pixbuf_new_from_file (filename2);
       height2 = gdk_pixbuf_get_height(pixbuf2);
       width2 = gdk_pixbuf_get_width(pixbuf2);

       scale_h2 = 165 / height2;
       scale_w2 = 165 / width2;
       if (scale_h2>scale_w2) {
	   scale_h2 = scale_w2;
       } else {
	   scale_w2 = scale_h2;
       }
       gnome_print_scale (ctx, gdk_pixbuf_get_width(pixbuf)*scale_w, gdk_pixbuf_get_height(pixbuf)*scale_h);
       gnome_print_pixbuf (ctx, pixbuf);
       gnome_print_grestore (ctx);
       gnome_print_gsave (ctx);

       
       
       gnome_print_translate (ctx, x,y+125);
       gnome_print_scale (ctx, gdk_pixbuf_get_width(pixbuf2)*scale_w2, gdk_pixbuf_get_height(pixbuf2)*scale_h2);
       gnome_print_pixbuf (ctx, pixbuf2);


   }
   gnome_print_grestore (ctx);
}



/* p_print_text: taken from evolution/calendar/gui/print.c */
/* Still not in use, but interesting for the future */
/**
 * p_print_text:
 * @pc: Contexto en el que se indicar lo que se quiere imprimir.
 * @font: Tipo de fuente actual del contexto.
 * @text: Texto que se desea imprimir.
 * @align: Enumeracin que indica si se alinea a la izquierda, derecha o centrado.
 * @l: Punto utilizado para calcular la #x, que indica el punto ms a la izquierda del texto.
 * @r: Punto utilizado para calcular la #x, que indica el punto ms a la derecha del texto.
 * @t: Punto utilizado para calcular la #y, que indica el punto ms alto del texto.
 * @b: Punto utilizado para calcular la #y, que indica el punto ms bajo del texto.
 *
 * Imprime un texto en un cuadro imaginario formado por los puntos (#l,#t) , (#r,#t) , (#l,#b) y (#r,#b)
 * alineado segn #align.
 *
 */
void
  p_print_text (GnomePrintContext *pc, GnomeFont *font, const char *text,
	     enum align_box align, double l, double r, double t, double b)
{
   double w, x, y;
   
   gnome_print_gsave (pc);
   
   w = gnome_font_get_width_utf8 (font, text);
   
   switch (align & 3)
     {		
      case ALIGN_LEFT:
      default:
	x = l;
	break;
      case ALIGN_RIGHT:
	x = r - w;
	break;
      case ALIGN_CENTER:
	x = l + ((r - l) - w) / 2;
	break;
     }
   
   /* Make sure we don't go off the left edge. */
   x = MAX (l, x);
   
   /* Now calculate the baseline. */
   y = t - gnome_font_get_ascender (font);
   
   /* Set a clipping rectangle. */
   gnome_print_moveto (pc, l, t);
   gnome_print_lineto (pc, r, t);
   gnome_print_lineto (pc, r, b);
   gnome_print_lineto (pc, l, b);
   gnome_print_closepath (pc);
   gnome_print_clip (pc);
   
   gnome_print_newpath (pc);
   gnome_print_moveto (pc, x, y);
   gnome_print_setfont (pc, font);
   gnome_print_setrgbcolor (pc, 0, 0, 0);
   gnome_print_show (pc, text);
   
   gnome_print_grestore (pc);
}

