/* associated_comp_document_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */
#include "associated_comp_documents_model.h"

#include "util.h"
#include "company.h"
#include "company_model.h"
#include "enterprise_model.h"
#include "shop_model.h"
#include "warehouse_model.h"
#include "parameter_manager.h"

/**
 * m_document_find_associated_companies:
 * @document: Documento en cuestin.
 * 
 * Busca las compaas de seguros que tienen el documento asociado.
 * Existen documentos necesarios para cada compaa, a la hora de reparar o facturar. Pero
 * algunos documentos coinciden para varias compaas. como por ej. el DNI, por lo que se necesitan
 * buscar las compaas que necesitan un documento en particular.
 *
 * Returns: Lista de comaas de seguros que tienen asociado el documento.
 */
Company **m_document_find_associated_companies (Document *document) {

    GdaCommand    *command    = gda_command_new();
    GdaConnection *connection = m_model_get_connection ();
    GdaRecordset  *record_set = NULL;
    GdaField      *field      = NULL;

    gchar  *gchar_query = NULL;
    gchar  *temp        = NULL;
    gint    index       = 0;
    gulong  tuples      = 0;
    gulong *company_ids = NULL;
    Company **companies = NULL;
    gboolean  correct    = FALSE;

    gda_command_set_connection(command,connection);

    temp = g_strdup_printf("%lu",d_document_get_code (document));
    gchar_query = g_strconcat("SELECT codigoaseguradora ",
			      " FROM necesitadocumento",
			      " WHERE codigodocumento = ",
			      temp,
			      NULL);

    g_free(temp);

    correct = m_model_command_execute (&record_set, gchar_query, connection,
				    command, &tuples,0, TRUE);
    g_free(gchar_query);

    if (correct) {
	if (record_set!=NULL) {
	    if (tuples!=0) {

		company_ids = (gulong *) g_malloc(sizeof(gulong *)*(tuples+1));

		index = gda_recordset_move_first(record_set);
		while (!gda_recordset_eof(record_set)) {

		    if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;

		    field = gda_recordset_field_name(record_set,"codigoaseguradora");
		    if (field!=NULL) {
			if (!gda_field_is_null(field)) {
			    company_ids[index-1] =
				igalia_field_get_bigint_value (field);
			}
			gda_field_free(field);
		    }

		    index = gda_recordset_move_next(record_set);
		}
		company_ids[index] = ULONG_MAX;
		companies = (Company **) g_malloc(sizeof(Company *)*(tuples+1));
		m_company_find_by_array_code (companies, company_ids);
		return companies;
	    }
	    gda_recordset_free(record_set);
	}
    }
    igalia_command_free(command);

    return NULL;

}

/**
 * m_document_update_associated_companies: 
 * @document: Documento a lque se van a asociar las compaas.
 * @company_ids: Array de identificadores do compaa a los que se le asociar el
 * documento, en caso de no lo tuviese asociado ya.
 * 
 * Modifica la tabla NecesitaDocumento en caso de que ya est asociada la compaa al
 * documento e inserta una nueva tupla si no lo est.
 * La tabla NecesitaDocumento
 * indica la asociacin entre compaas de seguros y los documentos necesarios.
 *
 * Returns: %TRUE si la operacin concluy correctamente.
 */
gboolean m_document_update_associated_companies (Document *document, gulong *company_ids) {
    GdaCommand    *command    = gda_command_new();
    GdaConnection *connection = m_model_get_connection ();
    GdaRecordset  *record_set = NULL;

    gchar  *gchar_query   = NULL;
    gchar  *document_code = NULL;
    gchar  *company_code  = NULL;
    gint    index         = 0;
    gulong  tuples        = 0;
    gboolean  correct     = FALSE;

    gulong logged_shop_code;
    gulong logged_enterprise_code;
    gulong logged_warehouse_code;
    gchar * temp_name;

    temp_name = u_parameter_manager_get_parameter ("empresa");
    logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("tienda");
    logged_shop_code = m_shop_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("almacen");
    logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

    g_free(temp_name);

    // First, we delete all document relationship
    gda_command_set_connection(command,connection);

    m_model_connection_begin_mutex_transaction_logged (connection,
						    logged_enterprise_code,
						    logged_shop_code,
						    logged_warehouse_code);

    document_code = g_strdup_printf("%lu",d_document_get_code (document));
    gchar_query = g_strconcat(" DELETE ",
			      " FROM necesitadocumento ",
			      " WHERE codigodocumento = ",
			      document_code,
			      NULL);

    correct = m_model_command_execute (&record_set, gchar_query, connection,
				    command, &tuples,0, FALSE);
    g_free(gchar_query);
    gchar_query = NULL;

    gda_recordset_free(record_set);
    record_set = NULL;
    
    // Now, we create all relationship between document and companies
    index = 0;
    if(company_ids!=NULL) {
	while(company_ids[index]!=ULONG_MAX) {
	    company_code = g_strdup_printf("%lu",company_ids[index]);
	    gchar_query = g_strconcat("INSERT INTO necesitadocumento",
				      " VALUES (",
				      company_code,",",
				      document_code, ") ",
				      NULL);

	    correct = correct && m_model_command_execute (&record_set, gchar_query,
						       connection, command,
						       &tuples,0, FALSE);
	    g_free(gchar_query);
	    gchar_query=NULL;
	    g_free(company_code);
	    company_code=NULL;
	    gda_recordset_free(record_set);
            record_set=NULL;
            index++;
	}

    }

    if (document_code!=NULL) {
	g_free(document_code);
    }

    correct = m_model_execute_mutex_modification_logged (connection) && correct;
    igalia_command_free(command);

    return correct;
}

