
/* company_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "company_model.h"
#include "participant_model.h"

#include "util.h"

Company ** companies_list = NULL;
gint length_of_list;

#define TABLE         "Compania"
#define IDENTIFICATOR "codigoParty"
#define ATTRIBUTES    "codigoConsorcio as codigoconsorciocompany"




/**
 * m_company_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_company_get_table () {
    return TABLE;
}

/**
 * m_company_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_company_get_attributes () {
    return ATTRIBUTES;
}


/**
 * m_company_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Company con los campos rellenados con los datos de la base de datos.
 */
Company *m_company_create (GdaRecordset *rs) {
    GdaField * field;
    Company * company = d_company_create_empty();

    company->participant = m_participant_create (rs,"company");

    if (d_participant_get_code (d_company_get_participant (company)) != ULONG_MAX) {
      field = gda_recordset_field_name(rs,"codigoconsorciocompany");
      if (field!=NULL) {
	  if (!gda_field_is_null(field)) {
	      company->consortium_code = igalia_field_get_bigint_value (field);
	  }
	  gda_field_free(field);
      }

      field = gda_recordset_field_name(rs,"dtocompany");
      if (field!=NULL) {
	  if(!gda_field_is_null(field)) {
	      company->discount = igalia_field_get_double_value (field);
	  }
	  gda_field_free(field);
      }

      field = gda_recordset_field_name(rs,"plantillacompany");
      if (field!=NULL) {
	  if(!gda_field_is_null(field)) {
	      company->template = gda_field_get_integer_value(field);
	  }
	  gda_field_free(field);
      }
     }

    return company;
}


/** 
 * m_company_find_by_array_code:
 * @companies: lista de compaias.
 * @company_ids: Lista de identificadores de compaas de seguros.
 *
 * Busca las compaas segn una lista de cdigos.
 * con un filtro de bsqueda.
 *
 */
void m_company_find_by_array_code (Company **companies, gulong *company_ids) {
    gint index = 0;
    gint search_index = 0;
    gint size  = 0;
    gulong code = ULONG_MAX;
    Company **result = NULL;

    if (company_ids!=NULL) {
	while (company_ids[size++]!=ULONG_MAX);
	result = (Company **) g_malloc(sizeof(Company *)*(size+1));

	if (companies_list==NULL) {
	    m_company_find_by_code (ULONG_MAX);
	}

	while(companies_list[index]!=NULL) {
	    code = d_participant_get_code (d_company_get_participant (companies_list[index]));
	    if(code==company_ids[search_index]) {
		companies[search_index] = d_company_duplicate (companies_list[index]);
		index = 0;
		search_index++;
	    } else {
		index++;
	    }
	}
	companies[search_index] = NULL;
    }

}


/** 
 * m_company_find_by_code:
 * @code: Cdigo de la compaa deseada o %ULONG_MAX si se necesitan todas las compaas de seguros.
 *
 * Busca la compaa identificada por el codigo %code, o todas las compaas si
 * el cdigo es %ULONG_MAX.
 *
 * Returns: Lista con todas las compaas de seguros o con una sola compaa de seguros.
 */
Company ** m_company_find_by_code (gulong code) {

  if ((companies_list==NULL)||(code!=ULONG_MAX)) {
    GdaCommand         * my_command;
    GdaConnection      * my_connection = m_model_get_connection ();
    GdaRecordset       * record_set = NULL;
    Company * company = NULL;
    gchar * gchar_query = NULL;
    gulong * tuples;
    gint * column;
    gint index;

    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);
    
    gchar_query = g_strconcat("SELECT ",
			      " participante.codigoparty as codigopartyparticipantcompany, ",
			      " participante.nombre as nombreparticipantcompany, ",
			      " participante.cif as cifparticipantcompany, ",
			      " participante.nick as nickparticipantcompany, ",
			      " participante.telefono as telefonoparticipantcompany, ",
			      " participante.telefono2 as telefono2participantcompany, ",
			      " participante.fax as faxparticipantcompany, ",
			      " participante.email as emailparticipantcompany, ",
			      " participante.calle as calleparticipantcompany, ",
			      " participante.numero as numeroparticipantcompany, ",
			      " participante.piso as pisoparticipantcompany, ",
			      " participante.cp as cpparticipantcompany, ",
			      " participante.municipio as municipioparticipantcompany, ",
			      " participante.provincia as proviciaparticipantcompany, ",
			      " participante.nombreCorto as shortNameparticipantcompany, ",
			      " Compania.codigoconsorcio as codigoconsorciocompany,",
			      " datosck.codigock as codigockcompany,",
			      " to_char(Compania.dtocompania,'999D99') as dtocompany, ",
			      " Compania.plantilla as plantillacompany ",
			      " FROM participante, compania left outer join datosck on (compania.codigoPArty=datosck.codigoParty) ",
			      " WHERE participante.compania=true",
			      " AND participante.codigoparty=compania.codigoparty", 
			      NULL);

    if (code != ULONG_MAX){
      gchar * gchar_query_aux = NULL;
      gchar *code_gchar = g_strdup_printf("%lu", code);

      gchar_query_aux = g_strconcat(gchar_query," AND participante.codigoparty='",code_gchar,"'",NULL);
      g_free(gchar_query);
      g_free(code_gchar);
      gchar_query =  gchar_query_aux;
      gchar_query_aux = NULL;
    }else {
      gchar * gchar_query_aux = NULL;
      
      gchar_query_aux = g_strconcat(gchar_query," ORDER BY participante.nombre",NULL);
      g_free(gchar_query);
      gchar_query =  gchar_query_aux;
      gchar_query_aux = NULL;
    }

    tuples = (gulong *) g_malloc0(sizeof(gulong));
    m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);
    g_print(gchar_query);
    g_free(gchar_query);
    
    length_of_list = *tuples;
    column = (gint *) g_malloc0(sizeof(gint));
    
    if (record_set!=NULL) {
      if (*tuples!=0) {
	if (code == ULONG_MAX){
	  companies_list = (Company **) g_malloc0(sizeof(Company*)*(*tuples+1));
	  index = gda_recordset_move_first(record_set);
	  while (!gda_recordset_eof(record_set)) {
	    if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
	    (*column)=0;
	    company = m_company_create (record_set);
	    companies_list[index-1] = company;
	    index = gda_recordset_move_next(record_set);
	  }
	  companies_list[length_of_list]=NULL;
	} else {	  
	  index = gda_recordset_move_first(record_set);
	  company = m_company_create (record_set);
	}
      } else {
	if (code == ULONG_MAX) companies_list=NULL;
      }
      gda_recordset_close(record_set);
    } else {
      if (code == ULONG_MAX) companies_list=NULL;
    }
    
    igalia_command_free(my_command);

    gda_recordset_free(record_set);
    
    g_print(_("\nCommand and recordset freeing completed\n"));
    g_free(tuples);
    g_free(column);

    if (code != ULONG_MAX) {
      Company **companies = (Company **) g_malloc0(sizeof(Company*)*(2));
      companies[0]=company;
      companies[1]=NULL;
      return companies;
    }
  }

  return companies_list;
}


/** 
 * m_company_clear_cached_array:
 *
 * Existe una variable global que contiene todas las compaas de seguros. Acta como una cach de
 * de compaas. Esta variable solo se debe liberar al final de la aplicacin o cuando se introduzcan
 * nuevas compaas para recuperar las nuevas compaas en memoria.
 *
 */
void m_company_clear_cached_array () {
  
    if (companies_list!=NULL) {
	gint index = 0;
	while (companies_list[index]!=NULL) {
	  d_company_clear (companies_list[index]);
	  index++;
	}
    }

    g_free(companies_list);   
    
    companies_list = NULL;
    
}

/** 
 * m_company_clear_array:
 * @list: Array de compaas a liberar.
 *
 * Libera la memoria utilizada por un array de compaas.
 *
 */
void m_company_clear_array (Company **list) {
  
    if (list!=NULL) {
	gint index = 0;
	while (list[index]!=NULL) {
	  d_company_clear (list[index]);
	  index++;
	}
    }

    g_free(list);
    
    list = NULL;
    
}
