
/* account.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "account.h"
#include "model.h"
#include "util.h"
#include "stdlib.h"

static gint account_init(Account *account);




/**
 * d_account_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto account vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Account *d_account_create_empty () {
  Account *account = (Account *) g_malloc (sizeof(Account));

  account_init(account);
  
  return account;
}



/**
 * d_account_clear:
 * @account: Cuenta corriente a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_account_clear (Account *account) {

    if (account!=NULL) {
        d_office_clear (account->office);
        if (account->account_number!=NULL)
	    g_free(account->account_number);
	g_free(account);
    }

  return 0;
}


/**
 * d_account_set_control_digit:
 * @account: Objeto en el que se establece el atributo.
 * @control_digit: Atributo a insertar.
 *
 * Establece el atributo @control_digit en el objeto. El atributo @control_digit
 * indica el dgito de control de una cuenta.
 *
 */
void d_account_set_control_digit (Account *account,gint control_digit) {
    account->control_digit = control_digit;
    
}

/**
 * d_account_set_initial_balance:
 * @account: Objeto en el que se establece el atributo.
 * @initial_balance: Atributo a insertar.
 *
 * Establece el atributo @initial_balance en el objeto. El atributo @initial_balance
 * indica el saldo inicial de una cuenta corriente de una empresa del grupo.
 *
 */
void d_account_set_initial_balance (Account *account,gdouble  initial_balance){
    account->initial_balance = initial_balance;
}


/**
 * d_account_set_account_number:
 * @account: Objeto en el que se establece el atributo.
 * @account_number: Atributo a insertar.
 *
 * Establece el atributo @account_number en el objeto. El atributo @account_number
 * indica solo el nmero de cuenta de una cuenta bancaria.
 *
 */
void d_account_set_account_number (Account *account, gchar *  account_number){
    if (account->account_number!=NULL) {
        g_free(account->account_number);
    }
    account->account_number = g_strdup(account_number);
}



/**
 * d_account_set_office:
 * @account: Objeto en el que se establece el atributo.
 * @office: Atributo a insertar.
 *
 * Establece el atributo @office en el objeto. El atributo @office
 * indica el objeto "oficina bancaria" de la cuenta corriente.
 *
 */
void d_account_set_office (Account *account, Office * office){
    account->office = office;
}

/**
 * d_account_get_control_digit:
 * @account: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns:
 * El valor del atributo solicitado
 *
 * Returns:
 * El valor del atributo solicitado
 */
gint d_account_get_control_digit (Account *account){
  return account->control_digit;
}

/**
 * d_account_get_account_number:
 * @account: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns:
 * El valor del atributo solicitado
 */
gchar *  d_account_get_account_number (Account *account){
  return g_strdup(account->account_number);
}

/**
 * d_account_get_initial_balance:
 * @account: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns:
 * El valor del atributo solicitado
 */
gdouble d_account_get_initial_balance (Account *account){
  return account->initial_balance;
}

/**
 * d_account_get_office:
 * @account: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns:
 * El valor del atributo solicitado
 */
Office * d_account_get_office (Account *account){
  return account->office;
}


/**
 * d_account_get_complete_number:
 * @bank_code: Cdigo del banco (nmero sin ceros de relleno, hasta 4 cifras).
 * @office_code: Cdigo de la oficina (nmero sin ceros de relleno, hasta 4 cifras).
 * @dc: Dgito de control (sin rellenar hasta 2 cifras).
 * @account_number: Nmero de cuenta.
 *
 * Devuelve si el nmero de cuenta es correcto.
 *
 * Returns:
 * %TRUE si la cuenta es correcta.
 */
gboolean d_account_is_correct (gchar * bank_code,gchar * office_code,gchar * dc,gchar * account_number) {
    gboolean result = FALSE;
    gchar * bank_code_aux = NULL;
    gchar * office_code_aux= NULL;
    gchar * account_number_aux= NULL;
    gchar * dc_aux= NULL;
    gchar * correctDC = NULL;

    if ((bank_code!=NULL) && (office_code!=NULL) && (account_number!=NULL) && (dc!=NULL)) {
	bank_code_aux = u_fill_with_zeros_until (bank_code,4);
	office_code_aux = u_fill_with_zeros_until (office_code,4);
	account_number_aux = u_fill_with_zeros_until (account_number,10);
	dc_aux = u_fill_with_zeros_until (dc,2);

	correctDC = u_calculate_control_digit (bank_code_aux,office_code_aux,account_number_aux);

	g_print(_("\nControl digit:%s\n"),correctDC);

	if (correctDC !=NULL) {
	    if (!strcmp(correctDC,dc_aux)) {
		result = TRUE;
	    }
	}
	if (bank_code_aux!=NULL)
	    g_free(bank_code_aux);
	if (office_code_aux!=NULL)
	    g_free(office_code_aux);
	if (account_number_aux!=NULL)
	    g_free(account_number_aux);
	if (dc_aux!=NULL)
	    g_free(dc_aux);

    }
    return result;
}

/**
 * d_account_get_complete_number:
 * @account: Objeto del que se desea obtener el resultado.
 *
 * Obtiene a partir de los atributos de una cuenta corriente una cadena de caracteres con el
 * nmero de cuenta completa.
 *
 * Returns:
 * Nmero de cuenta completo, es decir, el cdigo cuenta cliente (CCC).
 */
gchar * d_account_get_complete_number (Account * account) {
    gchar * bank_code;
    gchar * office_code ;
    gchar * account_number ;
    gchar * complete_account_number= NULL ;
    gchar * control_digit=NULL ;
    bank_code = g_strdup_printf("%d",d_office_get_bank_code (d_account_get_office (account)));
    office_code = g_strdup_printf("%d",d_office_get_office_code (d_account_get_office (account)));
    account_number = d_account_get_account_number (account);
    if ((bank_code!=NULL) && (office_code!=NULL) && (account_number!=NULL)) {
	gchar * aux;
	aux = u_fill_with_zeros_until (bank_code,4);
	g_free(bank_code);
	bank_code = aux;
	aux=NULL;

	aux = u_fill_with_zeros_until (office_code,4);
	g_free(office_code);
	office_code = aux;
	aux=NULL;

	aux = u_fill_with_zeros_until (account_number,10);
	g_free(account_number);
	account_number = aux;
	aux=NULL;

	control_digit = u_calculate_control_digit (bank_code,office_code,account_number);

	complete_account_number=g_strconcat(bank_code," ",office_code," ",control_digit," ",account_number,NULL);
    }
    return complete_account_number;
}

/* Frees the memory used by all the elements of the list and, then, the GList itself */

/**
 d_account_free_glist:
 * @list: Lista enlazada a liberar.
 *
 * Libera la memoria utilizada por una lista enlazada de cuentas bancarias.
 *
 */
void d_account_free_glist (GList * list) {
  GList * element;

  if (list != NULL)
    {
      element = g_list_first(list);
      
      do {
	d_account_clear (element->data);
	element = g_list_next(element);
      }
      while (element != NULL);
      
      g_list_free(list);
    }
}


/* Private functions*/

static gint account_init(Account *account) {

    account->account_number = NULL ;
    account->initial_balance = -1;
    account->control_digit = -1;
    account->office = NULL;
    return 0;
}



