/* cash_operation.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "cash_operation.h"
#include "stdlib.h"
#include "util.h"
#include "time.h"
static gint cash_operation_init(CashOperation *cash_operation);


/**
 * d_cash_operation_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns: Un objeto #CashOperation vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
CashOperation *d_cash_operation_create_empty () {
  CashOperation *cash_operation = (CashOperation *) g_malloc (sizeof(CashOperation));

  cash_operation_init(cash_operation);
  
  return cash_operation;
}



/**
 * d_cash_operation_clear:
 * @cash_operation: Movimiento de caja a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_cash_operation_clear (CashOperation *cash_operation) {

    if (cash_operation!=NULL) {
	d_cash_clear_cash (cash_operation->cash);
	if (cash_operation->section!=NULL) {
            g_free(cash_operation->section);
	}
        d_payment_clear (cash_operation->payment);
	g_free(cash_operation);
    }

  return 0;
}

/**
 * d_cash_operation_set_operation_code:
 * @cash_operation: Objeto en el que se establece el atributo.
 * @operation_code: Atributo a insertar.
 *
 * Establece el atributo @operation_code en el objeto. El atributo @operation_code
 * indica el cdigo interno que identifica el movimiento de caja.
 *
 */
void d_cash_operation_set_operation_code (CashOperation *cash_operation, gulong operation_code){
    cash_operation->operation_code = operation_code;
}
/**
 * d_cash_operation_set_amount:
 * @cash_operation: Objeto en el que se establece el atributo.
 * @amount: Atributo a insertar.
 *
 * Establece el atributo @amount en el objeto. El atributo @amount
 * indica la cantidad que se mueve una caja. Si es positivo se mete en
 * caja y si es negativo se quita.
 *
 */
void d_cash_operation_set_amount (CashOperation *cash_operation, gdouble amount){
    cash_operation->amount = amount;
}
/**
 * d_cash_operation_set_date:
 * @cash_operation: Objeto en el que se establece el atributo.
 * @date: Atributo a insertar.
 *
 * Establece el atributo @date en el objeto. El atributo @date
 * indica la fecha en que se realiza el movimiento de caja.
 *
 */
void d_cash_operation_set_date (CashOperation *cash_operation, time_t date){
    cash_operation->date = date;
}
/**
 * d_cash_operation_set_cash:
 * @cash_operation: Objeto en el que se establece el atributo.
 * @cash: Atributo a insertar.
 *
 * Establece el atributo @cash en el objeto. El atributo @cash
 * indica la caja en la que se realiza la operacin.
 *
 */
void d_cash_operation_set_cash (CashOperation *cash_operation, Cash * cash){
    if (cash_operation->cash!=NULL) {
        d_cash_clear_cash (cash_operation->cash);
    }
    cash_operation->cash = cash;

}
/**
 * d_cash_operation_set_section:
 * @cash_operation: Objeto en el que se establece el atributo.
 * @section: Atributo a insertar.
 *
 * Establece el atributo @section en el objeto. El atributo @section
 * indica la razn por la que se realiza el movimiento de caja.
 *
 */
void d_cash_operation_set_section (CashOperation *cash_operation, gchar * section){
    if (cash_operation->section!=NULL) {
        g_free(cash_operation->section);
    }
    cash_operation->section = g_strdup(section);
}

/**
 * d_cash_operation_set_payment:
 * @cash_operation: Objeto en el que se establece el atributo.
 * @payment: Atributo a insertar.
 *
 * Establece el atributo @payment en el objeto. El atributo @payment
 * indica el pago a partir del cual se realiza el movimiento.
 *
 */
void d_cash_operation_set_payment (CashOperation *cash_operation,Payment * payment){
    if (cash_operation->payment!=NULL) {
        d_payment_clear (cash_operation->payment);
    }
    cash_operation->payment = payment;

}
/**
 * d_cash_operation_get_operation_code:
 * @cash_operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong  d_cash_operation_get_operation_code (CashOperation *cash_operation){
    return cash_operation->operation_code;
}
/**
 * d_cash_operation_get_amount:
 * @cash_operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble  d_cash_operation_get_amount (CashOperation *cash_operation){
    return cash_operation->amount;
}
/**
 * d_cash_operation_get_section:
 * @cash_operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_cash_operation_get_section (CashOperation *cash_operation){
    return g_strdup(cash_operation->section);
}

/**
 * d_cash_operation_get_cash:
 * @cash_operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Cash * d_cash_operation_get_cash (CashOperation *cash_operation){
    return cash_operation->cash;
}
/**
 * d_cash_operation_get_payment:
 * @cash_operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Payment * d_cash_operation_get_payment (CashOperation *cash_operation){
    return cash_operation->payment;
}

/**
 * d_cash_operation_get_date:
 * @cash_operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t  d_cash_operation_get_date (CashOperation *cash_operation) {
    return cash_operation->date;
}

/* Private functions*/

static gint cash_operation_init(CashOperation *cash_operation) {

    cash_operation->cash = NULL ;
    cash_operation->payment = NULL ;
    cash_operation->operation_code = ULONG_MAX;
    cash_operation->amount = -1;
    cash_operation->section =NULL;
    cash_operation->date = time(NULL);
    return 0;
}
