
/* exchange.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "exchange.h"
#include "util.h"


static gint exchange_init(Exchange *exchange);

/**
 * d_exchange_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns: Un objeto #Exchange vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Exchange * d_exchange_create_empty () {
    Exchange *exchange = (Exchange *) g_malloc(sizeof(Exchange));

    exchange_init(exchange);

    return exchange;                           
}


/**
 * d_exchange_clear:
 * @exchange: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_exchange_clear (Exchange *exchange) {
    if (exchange!=NULL) {
	if (exchange->consignee_warehouse_name!=NULL) g_free(exchange->consignee_warehouse_name);

	g_free(exchange);
        exchange = NULL;
    }
    return 0;
}

/**
 * d_exchange_set_enterprise_code:
 * @exchange: Objeto en el que se establece el atributo.
 * @enterprise_code: Atributo a insertar.
 *
 * Establece el atributo @enterprise_code en el objeto. El atributo @enterprise_code
 * indica el cdigo de la empresa a la que pertenece el almacn en la que se realiz el intercambio.
 *
 */
void d_exchange_set_enterprise_code (Exchange *exchange, gulong enterprise_code) {
    exchange->enterprise_code = enterprise_code;
}

/**
 * d_exchange_set_in:
 * @exchange: Objeto en el que se establece el atributo.
 * @in: Atributo a insertar.
 *
 * Establece el atributo @in en el objeto. El atributo @in
 * indica el intercambio es de entrada o salida de un almacn en particular.
 *
 */
void d_exchange_set_in (Exchange *exchange, gdouble in) {
    exchange->in = in;
}

/**
 * d_exchange_set_warehouse_code:
 * @exchange: Objeto en el que se establece el atributo.
 * @warehouse_code: Atributo a insertar.
 *
 * Establece el atributo @warehouse_code en el objeto. El atributo @warehouse_code
 * indica el almacn en el que produjo el intercambio
 *
 */
void d_exchange_set_warehouse_code (Exchange *exchange, gulong warehouse_code) {
    exchange->warehouse_code = warehouse_code;
}

/**
 * d_exchange_set_product_code:
 * @exchange: Objeto en el que se establece el atributo.
 * @product_code: Atributo a insertar.
 *
 * Establece el atributo @product_code en el objeto. El atributo @product_code
 * indica el producto para el que produjo el intercambio.
 *
 */
void d_exchange_set_product_code (Exchange *exchange, gulong product_code) {
    exchange->product_code = product_code;
}

/**
 * d_exchange_set_quantity:
 * @exchange: Objeto en el que se establece el atributo.
 * @quantity: Atributo a insertar.
 *
 * Establece el atributo @quantity en el objeto. El atributo @quantity
 * indica la unidades que se intercambiaron en el proceso de intercambio.
 *
 */
void d_exchange_set_quantity (Exchange *exchange, gdouble quantity) {
    exchange->quantity = quantity;
}

/**
 * d_exchange_set_consignee_enterprise_code:
 * @exchange: Objeto en el que se establece el atributo.
 * @consignee_enterprise_code: Atributo a insertar.
 *
 * Establece el atributo @consignee_enterprise_code en el objeto. El atributo @consignee_enterprise_code
 * indica el cdigo de la empresa a la que pertenece el almacn destino del intercambio.
 *
 */
void d_exchange_set_consignee_enterprise_code (Exchange *exchange, gulong consignee_enterprise_code) {
    exchange->consignee_enterprise_code = consignee_enterprise_code;
}

/**
 * d_exchange_set_consignee_warehouse_code:
 * @exchange: Objeto en el que se establece el atributo.
 * @consignee_warehouse_code: Atributo a insertar.
 *
 * Establece el atributo @consignee_warehouse_code en el objeto. El atributo @consignee_warehouse_code
 * indica el cdigo del almacn destino del intercambio.
 *
 */
void d_exchange_set_consignee_warehouse_code (Exchange *exchange, gulong consignee_warehouse_code) {
    exchange->consignee_warehouse_code = consignee_warehouse_code;
}

/**
 * d_exchange_set_consignee_warehouse_name:
 * @exchange: Objeto en el que se establece el atributo.
 * @consignee_warehouse_name: Atributo a insertar.
 *
 * Establece el atributo @consignee_warehouse_name en el objeto. El atributo @consignee_warehouse_name
 * indica el nombre del almacn destino del intercambio.
 *
 */
void d_exchange_set_consignee_warehouse_name (Exchange *exchange, gchar * consignee_warehouse_name) {
    exchange->consignee_warehouse_name = g_strdup(consignee_warehouse_name);
}

/**
 * d_exchange_get_enterprise_code:
 * @exchange: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_exchange_get_enterprise_code (Exchange *exchange) {
    return exchange->enterprise_code;
}

/**
 * d_exchange_get_in:
 * @exchange: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_exchange_get_in (Exchange *exchange) {
    return exchange->in;
}

/**
 * d_exchange_get_product_code:
 * @exchange: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_exchange_get_product_code (Exchange *exchange) {
    return exchange->product_code;
}

/**
 * d_exchange_get_warehouse_code:
 * @exchange: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_exchange_get_warehouse_code (Exchange *exchange) {
    return exchange->warehouse_code;
}

/**
 * d_exchange_get_quantity:
 * @exchange: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_exchange_get_quantity (Exchange *exchange) {
    return exchange->quantity;
}

/**
 * d_exchange_get_date:
 * @exchange: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t d_exchange_get_date (Exchange *exchange) {
    return exchange->date;
}

/**
 * d_exchange_get_consignee_enterprise_code:
 * @exchange: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_exchange_get_consignee_enterprise_code (Exchange *exchange) {
    return exchange->consignee_enterprise_code;
}

/**
 * d_exchange_get_consignee_warehouse_code:
 * @exchange: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_exchange_get_consignee_warehouse_code (Exchange *exchange) {
    return exchange->consignee_warehouse_code;
}

/**
 * d_exchange_get_consignee_warehouse_name:
 * @exchange: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_exchange_get_consignee_warehouse_name (Exchange *exchange) {
    return g_strdup(exchange->consignee_warehouse_name);
}

/* Private functions*/

static gint exchange_init(Exchange *exchange) {
    exchange->enterprise_code = ULONG_MAX;
    exchange->warehouse_code = ULONG_MAX;
    exchange->movement_code = ULONG_MAX;
    exchange->quantity = G_MINDOUBLE;
    exchange->date = time(NULL);
    exchange->consignee_enterprise_code = ULONG_MAX;
    exchange->consignee_warehouse_code = ULONG_MAX;
    exchange->consignee_warehouse_name = NULL;

    return 0;
}


