/* expense.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


/*Gastos, suma de facturas de cargo agrupadas por proveedor*/

#include "config.h"
#include "expense.h"
#include "participant.h"
#include "util.h"

#define D_EXPENSE_CONTENT_SIZE 1

static gint expense_init(Expense *expense);

/**
 * d_expense_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Expense vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Expense * d_expense_create_empty () {

    Expense * expense;

    expense = (Expense *) g_malloc0(sizeof(Expense));

    expense_init(expense);

    return expense;
}


/**
 * d_expense_set_supplier:
 * @expense: Objeto en el que se establece el atributo.
 * @supplier: Atributo a insertar.
 *
 * Establece el atributo @supplier en el objeto. El atributo @supplier
 * indica el proveedor que emiti la factura de cargo que se listar en
 * el libro de Iva.
 *
 */
void d_expense_set_supplier (Expense * expense, Participant * supplier) {
    if (expense->supplier!= NULL) {
        d_participant_clear (expense->supplier);
    }
    expense->supplier = supplier;
}

/**
 * d_expense_set_total_summary:
 * @expense: Objeto en el que se establece el atributo.
 * @total_summary: Atributo a insertar.
 *
 * Establece el atributo @total_summary en el objeto. El atributo @total_summary
 * indica si el objeto es de resumen de los totales.
 *
 */
void d_expense_set_total_summary (Expense * expense, gboolean total_summary) {
    expense->total_summary = total_summary;
}

/**
 * d_expense_set_amount:
 * @expense: Objeto en el que se establece el atributo.
 * @amount: Atributo a insertar.
 *
 * Establece el atributo @amount en el objeto. El atributo @amount
 * indica la suma de la cantidades de las lneas de una factura con el
 * mismo tipo de Iva.
 *
 */
void d_expense_set_amount (Expense * expense, gfloat amount) {
    expense->amount = amount;
}

/**
 * d_expense_get_total_summary:
 * @expense: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_expense_get_total_summary (Expense * expense) {
    return expense->total_summary;
}

/**
 * d_expense_get_amount:
 * @expense: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gfloat d_expense_get_amount (Expense * expense){
    return expense->amount;
}

/**
 * d_expense_get_supplier:
 * @expense: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Participant * d_expense_get_supplier (Expense * expense) {
    return expense->supplier;
}


/**
 * d_expense_clear:
 * @expense: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_expense_clear (Expense * expense) {

    if (expense!=NULL) {
	d_participant_clear (expense->supplier);
	g_free(expense);
        expense = NULL;
    }
    return 0;
}


/**
 * d_expense_array_to_content:
 * @array: Array de objetos #Expense que se transformarn en listables.
 * @reccount: Puntero a un %gulong que indicar el total de objetos transformados.
 *
 * Funcin utilizada para transformar un array de objetos en un array de arrays de cadenas
 * de caracteres listables. Es utilizada para realizar los listados impresos en papel.
 *
 * Returns:
 * Un triple puntero a %gchar, es decir, un puntero a un array de arrays de cadenas de caracteres. Se
 * podra ver como un puntero a una tabla con filas y colummnas de cadenas de caracteres.
 */
gchar ***d_expense_array_to_content (Expense **array, gulong *reccount) {
    gint     array_size = 0;
    gchar ***content = NULL;

    *reccount = 0;
    while(array[array_size++]!=NULL);

    content = (gchar ***) malloc(sizeof(gchar **)*array_size);

    while(array[*reccount] != NULL) {
	content[*reccount] = d_expense_get_content (array[*reccount]);
        (*reccount)++;
    }

    content[*reccount] = NULL;

    return content;

}

/**
 * d_expense_get_content_title:
 *
 * Devuelve las cabaceras de cada pgina de los listados para el objeto factura de cargo.
 *
 * Returns:
 * Un array de cadena de caracteres con las cabeceras de pgina. EL array acaba en NULL.
 */
gchar **d_expense_get_content_title () {
    gchar **content = (gchar **) malloc (sizeof(gchar *)*d_expense_get_content_size ()+1);
    gint index = 0;

    d_participant_get_content_title (content, &index);
    index--;
    content[index++] = g_strdup_printf(_("COST"));
    content[index++] = NULL;
    return content;
}


/**
 * d_expense_get_listing_title:
 *
 * Obtiene el ttulo del listado.
 *
 * Returns:
 * El ttulo del listado.
 */
gchar *d_expense_get_listing_title () {
    gchar *content = g_strdup_printf(_("EXPENSE DECLARATION"));
    return content;
}

/**
 * d_expense_get_listing_columns_width:
 *
 * Indica los anchos de columna de los listados de gastos.
 *
 * Returns:
 * Un array de enteros que indican el ancho de cada columna en el listado. EL array acaba en -1.
 */
gint *d_expense_get_listing_columns_width () {
    gint *widths = (gint *) malloc (sizeof(gint)*d_expense_get_content_size ()+1);
    gint index = 0;

    d_participant_get_listing_columns_width (widths, &index);
    index--;
    widths[index++] = 5;
    widths[index++] = -1;

    return widths;
}

/**
 * d_expense_get_listing_columns_alignment:
 *
 * Obtiene el alineamiento de las columnas del listado. Los alineamientos pueden ser U_LEFT, RIGHT o
 * CENTER.
 *
 * Returns:
 * Un array con los alineamientos de cada columna. EL array acaba en -1.
 */
gint *d_expense_get_listing_columns_alignment () {
    gint *orientation = (gint *) malloc (sizeof(gint)*d_expense_get_content_size ()+1);
    gint index = 0;

    d_participant_get_listing_columns_alignment (orientation, &index);
    index--;
    orientation[index++] = U_RIGHT;
    orientation[index++] = -1;

    return orientation;
}

/**
 * d_expense_get_content_size:
 *
 * Obtiene el nmero de columnas que tendr el listado.
 *
 * Returns:
 * El nmero de columnas
 */
gint d_expense_get_content_size () {
    return (D_EXPENSE_CONTENT_SIZE+ d_participant_get_content_size ());
}


/**
 * d_expense_get_content:
 * @expense: Objeto que represena un gasto a partir de la que se obtendrn los valores para el listado.
 *
 * Transforma una objeto gasto en un array de cadenas de caracteres para aadir al listado
 * de facturas de cargo.
 *
 * Returns:
 * Array con los valores a sacar por listado en modo de cadenas de caracteres.
 */
gchar **d_expense_get_content (Expense *expense) {
    gchar **content = (gchar **)malloc(sizeof(gchar*) *
				       d_expense_get_content_size ()+1);
    gint    index   = 0;

    if (d_expense_get_total_summary (expense)==FALSE)
	content = d_participant_get_content (d_expense_get_supplier (expense),
					 content, &index);
    else {
	content[index++]=g_strdup("");
        content[index++]=g_strdup("");
        content[index++]=g_strdup("");
        content[index++]=g_strdup("");
        content[index++]=g_strdup("");
        content[index++]=g_strdup("");

    }
    index--;
    if (d_expense_get_total_summary (expense)==FALSE)
	content[index++] = g_strdup_printf("%.2f", d_expense_get_amount (expense));
    else {
	content[index++] = g_strdup_printf(_("TOTAL: %.2f"), d_expense_get_amount (expense));
        content[index++]=g_strdup("");

    }
    content[index++] = NULL;

    return content;
}

/**
 * d_expense_calculate_total_data:
 * @expenses: Array con los gastos de los que se quiere calcular el
 * resumen por totales.
 *
 * Calcula el total acumulado por todos los gastos y crea un objeto gasto
 * nuevo con el atributo #resumentTotales a %TRUE.
 *
 *
 * Returns:
 * Un array con un solo elemento que contiene el resumen de los totales.
 */
Expense **  d_expense_calculate_total_data (Expense ** expenses) {
    Expense * expense=NULL;
    Expense ** result=(Expense **) g_malloc0((sizeof(Expense*)*2));
    gulong index=0;
    gdouble total_sum = 0;

    if (expenses!=NULL) {
	while (expenses[index]!=NULL) {
	    gdouble total=0;

	    total=d_expense_get_amount (expenses[index]);

            total_sum = total_sum + total;

	    index++;
	}
    }
    expense = d_expense_create_empty ();
    d_expense_set_total_summary (expense,TRUE);
    d_expense_set_amount (expense,total_sum);
    result[0]=expense;

    return result;
    
}

/**
 * d_expense_concat:
 * @expenses: Primer array a concatenar.
 * @expenses2: Segundo array a concatenar.
 *
 * Concatena dos arrays de objetos gasto. #expense2 ir en segundo lugar en la concatenacin.
 *
 * Returns:
 * EL nuevo array que contiene los dos arrays pasados como parmetros.
 */
Expense ** d_expense_concat (Expense ** expenses,Expense ** expenses2) {
    gulong size=0;
    gulong size2=0;
    gulong index=0;
    gulong index2=0;
    Expense ** ib=NULL;
    if (expenses!=NULL) {
	while (expenses[size]!=NULL) {
	    size++;
	}


	if (expenses2!=NULL) {
	    while (expenses2[size2]!=NULL) {
		size2++;
	    }
	}
	ib = (Expense **) g_malloc0(sizeof(Expense*)*(size+size2+1));
	index = 0;
	while (index<(size)) {
	    ib[index]=expenses[index];
	    expenses[index]=NULL;
	    index++;
	    
	}

	index=size;
	while (index<(size+size2)) {
	    ib[index]=expenses2[index2];
	    expenses2[index2]=NULL;
	    index++;
	    index2++;
	}
        g_print("\n%lu\n",size+size2+1);

    }
    return ib;
}

/* Private functions */

static gint expense_init(Expense *expense) {
    expense->amount = -1.0;
    expense->supplier = NULL;
    expense->total_summary = FALSE;

    return 0;
}

