/* payment.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "payment.h"
#include "time.h"                   
#include "util.h"
#include "stdlib.h"

static gint payment_init(Payment *payment);

/**
 * d_payment_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Payment vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Payment * d_payment_create_empty () {

    Payment * payment;

    payment = (Payment *) g_malloc0(sizeof(Payment));
    payment_init(payment);

    return payment;
}
/**
 * d_payment_clear:
 * @payment: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_payment_clear (Payment *payment) {
    if (payment!=NULL) {
	d_enterprise_clear (payment->enterprise);
        d_shop_clear (payment->shop);
	if (payment->account_number!=NULL)
            g_free(payment->account_number);
	if (payment->cheque_number!=NULL)
            g_free(payment->cheque_number);
	if (payment->credit_card_number!=NULL)
            g_free(payment->credit_card_number);
	g_free(payment);

    }
    return 0;
}

/**
 * d_payment_set_enterprise:
 * @payment: Objeto en el que se establece el atributo.
 * @enterprise: Atributo a insertar.
 *
 * Establece el atributo @enterprise en el objeto. El atributo @enterprise
 * indica la empresa a la que pertenece la tienda en la que se genera el pago.
 *
 */
void d_payment_set_enterprise (Payment * payment,Enterprise * enterprise){
    payment->enterprise = enterprise;
    
}
/**
 * d_payment_set_shop (
 * @payment: Objeto en el que se establece el atributo.
 * @shop: Atributo a insertar.
 *
 * Establece el atributo @shop en el objeto. El atributo @shop
 * indica la tienda en la que se genera el pago.
 *
 */
void d_payment_set_shop (Payment * payment,Shop * shop){
    payment->shop = shop;
}

/**
 * d_payment_set_payment_code:
 * @payment: Objeto en el que se establece el atributo.
 * @payment_code: Atributo a insertar.
 *
 * Establece el atributo @payment_code en el objeto. El atributo @payment_code
 * indica el cdigo que identifica internamente el pago generado.
 *
 */
void d_payment_set_payment_code (Payment * payment,gulong payment_code){
    payment->payment_code = payment_code;

}

/**
 * d_payment_set_achieved_payments:
 * @payment: Objeto en el que se establece el atributo.
 * @achieved_payments: Atributo a insertar.
 *
 * Establece el atributo @achieved_payments en el objeto. El atributo @achieved_payments
 * indica la cantidad de pagos realizados. Este campo se utiliza solo en el caso de
 * pagos a plazos en las facturas de cargo.
 *
 */
void d_payment_set_achieved_payments (Payment * payment,gulong achieved_payments){
    payment->achieved_payments = achieved_payments;
}

/**
 * d_payment_set_achieved_date:
 * @payment: Objeto en el que se establece el atributo.
 * @achieved_date: Atributo a insertar.
 *
 * Establece el atributo @achieved_date en el objeto. El atributo @achieved_date
 * indica la fecha en la que se debe realizar el pago generado.
 *
 */
void d_payment_set_achieved_date (Payment * payment,time_t achieved_date){
    payment->achieved_date = achieved_date;
}

/**
 * d_payment_set_entry_percentage:
 * @payment: Objeto en el que se establece el atributo.
 * @entry_percentage: Atributo a insertar.
 *
 * Establece el atributo @entry_percentage en el objeto. El atributo @entry_percentage
 * indica  el porcentaje de entrada que se pagar en los pagos a plazos sobre el
 * el total a pagar.
 *
 */
void d_payment_set_entry_percentage (Payment * payment,gdouble entry_percentage){
    payment->entry_percentage = entry_percentage;
}
/**
 * d_payment_set_number_expiries:
 * @payment: Objeto en el que se establece el atributo.
 * @number_expiries: Atributo a insertar.
 *
 * Establece el atributo @number_expiries en el objeto. El atributo @number_expiries
 * indica el nmero de vencimientos del pago a plazos.
 *
 */
void d_payment_set_number_expiries (Payment * payment,gshort number_expiries){
    payment->number_expiries = number_expiries;
}
/**
 * d_payment_set_init_date:
 * @payment: Objeto en el que se establece el atributo.
 * @init_date: Atributo a insertar.
 *
 * Establece el atributo @init_date en el objeto. El atributo @init_date
 * indica la fecha en la que inicia el pago de los pagos a plazos y cuando toca
 * realizar el siguiente.
 *
 */
void d_payment_set_init_date (Payment * payment,time_t init_date){
    payment->init_date = init_date;
}
/**
 * d_payment_set_interval_days:
 * @payment: Objeto en el que se establece el atributo.
 * @interval_days: Atributo a insertar.
 *
 * Establece el atributo @interval_days en el objeto. El atributo @interval_days
 * indica los das de aplazamiento en los pagos aplazados y los das entre plazos
 * en los pagos a plazos.
 *
 */
void d_payment_set_interval_days (Payment * payment,gshort interval_days){
    payment->interval_days = interval_days;
}
/**
 * d_payment_set_is_hire_purchase:
 * @payment: Objeto en el que se establece el atributo.
 * @is_hire_purchase: Atributo a insertar.
 *
 * Establece el atributo @is_hire_purchase en el objeto. El atributo @is_hire_purchase
 * indica si el pago se realiza a plazos.
 *
 */
void d_payment_set_is_hire_purchase (Payment * payment,gboolean is_hire_purchase){
    payment->is_hire_purchase = is_hire_purchase;
}
/**
 * d_payment_set_is_transfer:
 * @payment: Objeto en el que se establece el atributo.
 * @is_transfer: Atributo a insertar.
 *
 * Establece el atributo @is_transfer en el objeto. El atributo @is_transfer
 * indica si el pago se realiza mediante una transferencia.
 *
 */
void d_payment_set_is_transfer (Payment * payment,gboolean is_transfer){
    payment->is_transfer = is_transfer;
}
/**
 * d_payment_set_is_cheque:
 * @payment: Objeto en el que se establece el atributo.
 * @is_cheque: Atributo a insertar.
 *
 * Establece el atributo @is_cheque en el objeto. El atributo @is_cheque
 * indica si el pago se realiza por taln.
 *
 */
void d_payment_set_is_cheque (Payment * payment,gboolean is_cheque){
    payment->is_cheque = is_cheque;
}
/**
 * d_payment_set_cheque_number:
 * @payment: Objeto en el que se establece el atributo.
 * @cheque_number: Atributo a insertar.
 *
 * Establece el atributo @cheque_number en el objeto. El atributo @cheque_number
 * indica el nmero de taln en caso de pago mediente taln.
 *
 */
void d_payment_set_cheque_number (Payment * payment,gchar * cheque_number){
    if (payment->cheque_number!=NULL) {
        g_free(payment->cheque_number);
    }
    payment->cheque_number = g_strdup(cheque_number);
}
/**
 * d_payment_set_is_credit_card:
 * @payment: Objeto en el que se establece el atributo.
 * @is_credit_card: Atributo a insertar.
 *
 * Establece el atributo @is_credit_card en el objeto. El atributo @is_credit_card
 * indica si el pago se realiza mediante tarjeta.
 *
 */
void d_payment_set_is_credit_card (Payment * payment,gboolean is_credit_card){
    payment->is_credit_card = is_credit_card;
}
/**
 * d_payment_set_is_direct_debit:
 * @payment: Objeto en el que se establece el atributo.
 * @is_direct_debit: Atributo a insertar.
 *
 * Establece el atributo @is_direct_debit en el objeto. El atributo @is_direct_debit
 * indica si el pago se realiza mediante domiciliacin.
 *
 */
void d_payment_set_is_direct_debit (Payment * payment,gboolean is_direct_debit){
    payment->is_direct_debit = is_direct_debit;
}
/**
 * d_payment_set_credit_card_number:
 * @payment: Objeto en el que se establece el atributo.
 * @credit_card_number: Atributo a insertar.
 *
 * Establece el atributo @credit_card_number en el objeto. El atributo @credit_card_number
 * indica el nmero de tarjeta en caso de tratarse de pago mediante tarjeta.
 *
 */
void d_payment_set_credit_card_number (Payment * payment,gchar * credit_card_number){
    if (payment->credit_card_number!=NULL) {
        g_free(payment->credit_card_number);
    }
    payment->credit_card_number = g_strdup(credit_card_number);
}
/**
 * d_payment_set_expiry_date:
 * @payment: Objeto en el que se establece el atributo.
 * @expiry_date: Atributo a insertar.
 *
 * Establece el atributo @expiry_date en el objeto. El atributo @expiry_date
 * indica la fecha de caducidad de la tarjeta.
 *
 */
void d_payment_set_expiry_date (Payment * payment,time_t expiry_date){
    payment->expiry_date = expiry_date;
}
/**
 * d_payment_set_is_paid_cash:
 * @payment: Objeto en el que se establece el atributo.
 * @is_paid_cash: Atributo a insertar.
 *
 * Establece el atributo @is_paid_cash en el objeto. El atributo @is_paid_cash
 * indica si el pago se realiza al contado.
 *
 */
void d_payment_set_is_paid_cash (Payment * payment,gboolean is_paid_cash){
    payment->is_paid_cash = is_paid_cash;

}

/**
 * d_payment_set_bank_code:
 * @payment: Objeto en el que se establece el atributo.
 * @bank_code: Atributo a insertar.
 *
 * Establece el atributo @bank_code en el objeto. El atributo @bank_code
 * indica el cdigo del banco al que pertenece la cuenta corriente.
 *
 */
void d_payment_set_bank_code (Payment * payment,gint bank_code){
    payment->bank_code=bank_code;
}

/**
 * d_payment_set_office_code:
 * @payment: Objeto en el que se establece el atributo.
 * @office_code: Atributo a insertar.
 *
 * Establece el atributo @office_code en el objeto. El atributo @office_code
 * indica el cdigo de oficina de la sucursal a la que pertenece la cuenta corriente.
 *
 */
void d_payment_set_office_code (Payment * payment,gint office_code){
    payment->office_code=office_code;
}
/**
 * d_payment_set_account_number:
 * @payment: Objeto en el que se establece el atributo.
 * @account_number: Atributo a insertar.
 *
 * Establece el atributo @account_number en el objeto. El atributo @account_number
 * indica el nmero de cuenta de la cuenta corriente.
 *
 */
void d_payment_set_account_number (Payment * payment,gchar * account_number){
    if (payment->account_number!=NULL) {
        g_free(payment->account_number);
    }

    payment->account_number = g_strdup(account_number);
}


/**
 * d_payment_get_enterprise:
 * @payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Enterprise * d_payment_get_enterprise (Payment * payment){
    return payment->enterprise;
}



/**
 * d_payment_get_shop:
 * @payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Shop * d_payment_get_shop (Payment * payment){
    return payment->shop;
}

/**
 * d_payment_get_payment_code:
 * @payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_payment_get_payment_code (Payment * payment){
    return payment->payment_code;
}

/**
 * d_payment_get_achieved_payments:
 * @payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_payment_get_achieved_payments (Payment * payment){
    return payment->achieved_payments;
}

/**
 * d_payment_get_achieved_date:
 * @payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t d_payment_get_achieved_date (Payment * payment){
    return payment->achieved_date;
}

/**
 * d_payment_get_entry_percentage:
 * @payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_payment_get_entry_percentage (Payment * payment){
    return payment->entry_percentage;
}

/**
 * d_payment_get_number_expiries:
 * @payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gshort d_payment_get_number_expiries (Payment * payment){
    return payment->number_expiries;
}

/**
 * d_payment_get_init_date:
 * @payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t d_payment_get_init_date (Payment * payment){
    return payment->init_date;
}

/**
 * d_payment_get_interval_days:
 * @payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gshort d_payment_get_interval_days (Payment * payment){
    return payment->interval_days;
}

/**
 * d_payment_get_is_hire_purchase:
 * @payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_payment_get_is_hire_purchase (Payment * payment){
    return payment->is_hire_purchase;
}

/**
 * d_payment_get_is_transfer:
 * @payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_payment_get_is_transfer (Payment * payment){
    return payment->is_transfer;
}


/**
 * d_payment_get_is_at_moment:
 * @payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve si el pago es en efectivo.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_payment_get_is_paid_in_cash (Payment * payment) {

    gboolean result = FALSE;


    if ((d_payment_get_is_transfer (payment)==FALSE) &&
	(d_payment_get_is_cheque (payment)==FALSE) &&
	(d_payment_get_is_direct_debit (payment)==FALSE) &&
	(d_payment_get_is_credit_card (payment)==FALSE))
	result =TRUE;

    return result;
}

/**
 * d_payment_get_is_cheque:
 * @payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_payment_get_is_cheque (Payment * payment){
    return payment->is_cheque;
}

/**
 * d_payment_c_util_get_cheque_number:
 * @payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_payment_c_util_get_cheque_number (Payment * payment){
    return g_strdup(payment->cheque_number);
}

/**
 * d_payment_get_is_credit_card:
 * @payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_payment_get_is_credit_card (Payment * payment){
    return payment->is_credit_card;
}

/**
 * d_payment_get_is_direct_debit:
 * @payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_payment_get_is_direct_debit (Payment * payment){
    return payment->is_direct_debit;
}

/**
 * d_payment_get_credit_card_number:
 * @payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_payment_get_credit_card_number (Payment * payment){
    return g_strdup(payment->credit_card_number);
}

/**
 * d_payment_get_expiry_date:
 * @payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t d_payment_get_expiry_date (Payment * payment){
    return payment->expiry_date;
}

/**
 * d_payment_get_is_paid_cash:
 * @payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_payment_get_is_paid_cash (Payment * payment){
    return payment->is_paid_cash;
}


/**
 * d_payment_get_bank_code:
 * @payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gint d_payment_get_bank_code (Payment * payment){
return payment->bank_code;
}

/**
 * d_payment_get_office_code:
 * @payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gint d_payment_get_office_code (Payment * payment){
return payment->office_code;
}

/**
 * d_payment_get_account_number:
 * @payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *  d_payment_get_account_number (Payment * payment){
return  g_strdup(payment->account_number);
}


/* Private functions */

static gint payment_init(Payment *payment) {

    payment->enterprise = NULL;
    payment->shop = NULL;
    payment->payment_code = ULONG_MAX;
    payment->achieved_payments = ULONG_MAX;
    payment->entry_percentage = -1;
    payment->number_expiries = -1;
    payment->init_date = time(NULL);
    payment->interval_days = -1;
    payment->is_hire_purchase = FALSE;
    payment->is_transfer = FALSE;
    payment->is_cheque = FALSE;
    payment->cheque_number = NULL;
    payment->is_credit_card = FALSE;
    payment->is_direct_debit = FALSE;
    payment->credit_card_number = NULL;
    payment->expiry_date = time(NULL);
    payment->achieved_date = time(NULL);
    payment->is_paid_cash = FALSE;
    payment->bank_code = -1;
    payment->office_code = -1;
    payment->account_number = NULL;

    return 0;
}

/**
 * d_payment_duplicate:
 * @payment_to_dup: Objeto del que se desea obtener el duplicado.
 *
 * Duplica el objeto payment.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
Payment * d_payment_duplicate (Payment * payment_to_dup) {

    Payment * payment = d_payment_create_empty ();
    payment->enterprise = d_enterprise_duplicate (d_payment_get_enterprise (payment_to_dup));
    payment->shop = d_shop_duplicate (d_payment_get_shop (payment_to_dup));
    payment->payment_code = d_payment_get_payment_code (payment_to_dup);
    payment->achieved_payments = d_payment_get_achieved_payments (payment_to_dup);
    payment->entry_percentage = d_payment_get_entry_percentage (payment_to_dup);
    payment->number_expiries =d_payment_get_number_expiries (payment_to_dup);
    payment->init_date =d_payment_get_init_date (payment_to_dup);
    payment->interval_days =d_payment_get_interval_days (payment_to_dup);
    payment->is_hire_purchase = d_payment_get_is_hire_purchase (payment_to_dup);
    payment->is_transfer = d_payment_get_is_transfer (payment_to_dup);
    payment->is_cheque = d_payment_get_is_cheque (payment_to_dup);
    payment->cheque_number = d_payment_c_util_get_cheque_number (payment_to_dup);
    payment->is_credit_card = d_payment_get_is_credit_card (payment_to_dup);
    payment->is_direct_debit = d_payment_get_is_direct_debit (payment_to_dup);
    payment->credit_card_number = d_payment_get_credit_card_number (payment_to_dup);
    payment->expiry_date = d_payment_get_expiry_date (payment_to_dup);
    payment->achieved_date = d_payment_get_achieved_date (payment_to_dup);
    payment->is_paid_cash = d_payment_get_is_paid_cash (payment_to_dup);
    payment->bank_code = d_payment_get_bank_code (payment_to_dup);
    payment->office_code = d_payment_get_office_code (payment_to_dup);
    payment->account_number =d_payment_get_account_number (payment_to_dup);


    return payment;
}



