/* piece.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "piece.h"
#include "util.h"

static gint piece_init(Piece *piece);

/**
 * d_piece_create_empty:
 *
 * Crea un objeto vaco, inicializado a unos valores dependientes del tipo.
 *
 * Returns: Un objeto #Piece con los campos vacos.
 */
Piece * d_piece_create_empty () {
    Piece *piece = (Piece *) g_malloc(sizeof(Piece));
    Product product;

    piece_init(piece);

    product = d_product_create_empty ();
    d_product_set_is_piece (&product, TRUE);
    d_piece_set_product (piece, product);

    return piece;
}


/**
 * d_piece_clear:
 * @piece: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_piece_clear (Piece *piece) {
    if (piece!=NULL) {
	d_product_clear ((Product *)piece);
	if (piece->euroCode!=NULL) {
	    g_free(piece->euroCode);
            piece->euroCode = NULL;
	}
	g_free(piece);
        piece=NULL;
    }
    return 0;
}

/**
 * d_piece_set_euroCode:
 * @piece: Objeto en el que se establece el atributo.
 * @euroCode: Atributo a insertar.
 *
 * Establece el atributo @euroCode en el objeto. El atributo @euroCode
 * indica identificador de la pieza.
 *
 */
void d_piece_set_euroCode (Piece *piece, gchar *euroCode) {
    if ((piece->euroCode) != NULL) {
        g_free(piece->euroCode);
    }
    piece->euroCode = g_strdup (euroCode);
}

/**
 * d_piece_set_supplier_code:
 * @piece: Objeto en el que se establece el atributo.
 * @supplier_code: Atributo a insertar.
 *
 * Establece el atributo @supplier_code en el objeto. El atributo @supplier_code
 * indica identificador del proveedor asignado a dicha pieza.
 *
 */
void  d_piece_set_supplier_code (Piece *piece, gulong supplier_code) {
    piece->supplier_code = supplier_code;
}

/**
 * d_piece_set_product:
 * @piece: Objeto en el que se establece el atributo.
 * @product: Atributo a insertar.
 *
 * Establece el atributo @product en el objeto. El atributo @product
 * indica el objeto producto del que hereda todos sus atributos y sus mtodos.
 *
 */
void d_piece_set_product (Piece *piece, Product product) {
    d_product_clear (&piece->product);
    piece->product = product;
}

/**
 * d_piece_set_width:
 * @piece: Objeto en el que se establece el atributo.
 * @width: Atributo a insertar.
 *
 * Establece el atributo @width en el objeto. El atributo @width
 * indica el ancho de la pieza.
 *
 */
void d_piece_set_width (Piece *piece, gshort width) {
    piece->width = width;
}

/**
 * d_piece_set_length:
 * @piece: Objeto en el que se establece el atributo.
 * @length: Atributo a insertar.
 *
 * Establece el atributo @length en el objeto. El atributo @length
 * indica el largo de la pieza.
 *
 */
void d_piece_set_length (Piece *piece, gshort length) {
    piece->length = length;
}

/**
 * d_piece_set_cost_price:
 * @piece: Objeto en el que se establece el atributo.
 * @price: Atributo a insertar.
 *
 * Establece el atributo @cost_price en el objeto. El atributo @cost_price
 * indica el precio de coste de la pieza.
 *
 */
void d_piece_set_cost_price (Piece *piece, gdouble price){
    piece->cost_price = price;
}

/**
 * d_piece_set_meassure_code:
 * @piece: Objeto en el que se establece el atributo.
 * @meassure_code: Atributo a insertar.
 *
 * Establece el atributo @meassure_code en el objeto. El atributo @meassure_code
 * indica el cdigo de la medida a la que se asociar un paquete/rollo de pieza.
 *
 */
void   d_piece_set_meassure_code (Piece *piece, gulong meassure_code) {
    piece->meassure_code = meassure_code;

}

/**
 * d_piece_set_units_per_packet:
 * @piece: Objeto en el que se establece el atributo.
 * @units: Atributo a insertar.
 *
 * Establece el atributo @units_per_packet en el objeto. El atributo @units_per_packet
 * indica las unidades de pieza por paquete del mismo.
 *
 */
void   d_piece_set_units_per_packet (Piece *piece, gdouble units) {
    piece->units_per_packet = units;
}


/**
 * d_piece_get_euroCode:
 * @piece: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_piece_get_euroCode (Piece *piece) {
    return g_strdup(piece->euroCode);
}

/**
 * d_piece_get_supplier_code:
 * @piece: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_piece_get_supplier_code (Piece *piece) {
    return piece->supplier_code;
}

/**
 * d_piece_get_product:
 * @piece: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Product d_piece_get_product (Piece *piece) {
  return piece->product;
}

/**
 * d_piece_get_width:
 * @piece: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gshort d_piece_get_width (Piece *piece) {
    return piece->width;
}

/**
 * d_piece_get_length:
 * @piece: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gshort d_piece_get_length (Piece *piece) {
    return piece->length;
}

/**
 * d_piece_get_cost_price:
 * @piece: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_piece_get_cost_price (Piece *piece){
    return (piece->cost_price);
}

/**
 * d_piece_get_meassure_code:
 * @piece: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_piece_get_meassure_code (Piece *piece) {
    return piece->meassure_code;
}

/**
 * d_piece_get_units_per_packet:
 * @piece: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_piece_get_units_per_packet (Piece *piece) {
    return piece->units_per_packet;
}


/**
 * d_piece_duplicate:
 * @piece: Objeto del que se desea obtener el duplicado.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
Piece *d_piece_duplicate (Piece *piece) {
  
  Piece *result = d_piece_create_empty ();
  gchar *eurocode = d_piece_get_euroCode (piece);
  gshort width = d_piece_get_width (piece);
  gshort length = d_piece_get_length (piece);
  gdouble cost_price = d_piece_get_cost_price (piece);  
  Product product = d_piece_get_product (piece);
  Product *productduplicated = d_product_duplicate (&product);
  
  d_piece_set_euroCode (result, eurocode);
  d_piece_set_width (result, width);
  d_piece_set_length (result, length);
  d_piece_set_meassure_code (result, d_piece_get_meassure_code (piece));
  d_piece_set_units_per_packet (result, d_piece_get_units_per_packet (piece));
  d_piece_set_supplier_code (result, d_piece_get_supplier_code (piece));
  d_piece_set_length (result, length);
  d_piece_set_cost_price (result, cost_price);
  d_piece_set_product (result, *productduplicated);
  
  g_free(productduplicated);
  g_free(eurocode);

  return result;
}


/* Private functions*/

static gint piece_init(Piece *piece) {
    piece->product = d_product_create_empty ();
    piece->euroCode = NULL;
    piece->length = -1;
    piece->width = -1;
    piece->units_per_packet = G_MINDOUBLE;
    piece->meassure_code = ULONG_MAX;
    piece->supplier_code = ULONG_MAX;
    piece->cost_price = -1.0;

    return 0;
}

