/* product.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */



#include "product.h"
#include "util.h"

/**
 * d_product_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Product vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Product d_product_create_empty () {
    Product product;

    d_product_init (&product);

    return product;
}



/**
 * d_product_clear:
 * @product: Producto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_product_clear (Product *product) {

  if (product!=NULL){
      if (product->family_code!=NULL) {
	  g_free(product->family_code);
	  product->family_code = NULL;
      }

      if (product->description!=NULL) {
	  g_free(product->description);
	  product->description = NULL;
      }
  }

  return 0;
}

/**
 * d_product_set_product_code:
 * @product: Objeto en el que se establece el atributo.
 * @product_code: Atributo a insertar.
 *
 * Establece el atributo @product_code en el objeto. El atributo @product_code
 * indica cdigo interno que identifica el producto.
 *
 */
void d_product_set_product_code (Product *product, gulong product_code){
    product->product_code= product_code;
}

/**
 * d_product_set_family_code:
 * @product: Objeto en el que se establece el atributo.
 * @family_code: Atributo a insertar.
 *
 * Establece el atributo @family_code en el objeto. El atributo @family_code
 * indica el cdigo de la familia a la que pertenece la pieza.
 *
 */
void d_product_set_family_code (Product *product, gchar *family_code){
    if ((product->family_code) != NULL) {
        g_free(product->family_code);
    }
    product->family_code = g_strdup (family_code);
}

/**
 * d_product_set_description:
 * @product: Objeto en el que se establece el atributo.
 * @description: Atributo a insertar.
 *
 * Establece el atributo @description en el objeto. El atributo @description
 * indica la descripcin del producto.
 *
 */
void d_product_set_description (Product *product, gchar *description){
    if ((product->description) != NULL) {
        g_free(product->description);
    }
    product->description = g_strdup (description);
}

/**
 * d_product_set_vat_type:
 * @product: Objeto en el que se establece el atributo.
 * @vat_type: Atributo a insertar.
 *
 * Establece el atributo @vat_type en el objeto. El atributo @vat_type
 * indica el tipo de Iva del producto.
 *
 */
void d_product_set_vat_type (Product *product, gdouble vat_type){
    product->vat_type = vat_type;
}

/**
 * d_product_set_is_piece:
 * @product: Objeto en el que se establece el atributo.
 * @is_piece: Atributo a insertar.
 *
 * Establece el atributo @is_piece en el objeto. El atributo @is_piece
 * indica si el producto es una pieza o no. Si es una pieza establece directamente
 * que no se trata de una mano de obra
 *
 */
void d_product_set_is_piece (Product *product, gboolean is_piece){
    product->is_piece = is_piece;
    product->is_man_power = !is_piece;
}

/**
 * d_product_set_is_man_power:
 * @product: Objeto en el que se establece el atributo.
 * @is_man_power: Atributo a insertar.
 *
 * Establece el atributo @is_man_power en el objeto. El atributo @is_man_power
 * indica si el producto es una mano de obra o no. Si es una mano de obra establece directamente
 * que no es una pieza.
 *
 */
void d_product_set_is_man_power (Product *product, gboolean is_man_power){
    product->is_man_power = is_man_power;
    product->is_piece = !is_man_power;
}

/**
 * d_product_set_price:
 * @product: Objeto en el que se establece el atributo.
 * @price: Atributo a insertar.
 *
 * Establece el atributo @price en el objeto. El atributo @price
 * indica el precio una unidad del producto.
 *
 */
void d_product_set_price (Product *product, gdouble price){
    product->price = price;
}

/**
 * d_product_set_max_discount:
 * @product: Objeto en el que se establece el atributo.
 * @max_discount: Atributo a insertar.
 *
 * Establece el atributo @max_discount en el objeto. El atributo @max_discount
 * indica el descuento mximo que orientativamente se le prodra realizar a
 * una reparacin.
 *
 */
void d_product_set_max_discount (Product *product, gdouble max_discount){
    if (max_discount>100)
	product->max_discount=100;
    else if (max_discount<0)
	product->max_discount=0;
    else
	product->max_discount=max_discount;
}

/**
 * d_product_get_product_code:
 * @product: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_product_get_product_code (Product *product){
    return (product->product_code);
}

/**
 * d_product_get_family_code:
 * @product: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_product_get_family_code (Product *product){
    return g_strdup(product->family_code);
}

/**
 * d_product_get_description:
 * @product: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_product_get_description (Product *product){
    return g_strdup(product->description);
}

/**
 * d_product_get_vat_type:
 * @product: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_product_get_vat_type (Product *product){
    return (product->vat_type);
}

/**
 * d_product_get_is_piece:
 * @product: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_product_get_is_piece (Product *product){
    return (product->is_piece);
}

/**
 * d_product_get_is_man_power:
 * @product: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_product_get_is_man_power (Product *product){
    return(product->is_man_power);
}


/**
 * d_product_get_price:
 * @product: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_product_get_price (Product *product){
    return (product->price);
}

/**
 * d_product_get_max_discount:
 * @product: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_product_get_max_discount (Product *product){
    return (product->max_discount);
}

/**
 * d_product_duplicate:
 * @product: Objeto del que se desea obtener el duplicado.
 *
 * Duplica el objeto.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
Product *d_product_duplicate (Product *product){

  Product *result = (Product *) g_malloc (sizeof(Product));
  gulong product_code = d_product_get_product_code (product);
  gchar *family_codetype = d_product_get_family_code (product);
  gchar *description = d_product_get_description (product);
  gdouble vat_type = d_product_get_vat_type (product);
  gboolean es_pieza = d_product_get_is_piece (product);
  gboolean es_mano = d_product_get_is_man_power (product);
  gdouble price  = d_product_get_price (product);
  gdouble max_discount = d_product_get_max_discount (product);

  d_product_init (result);

  d_product_set_product_code (result, product_code);
  d_product_set_family_code (result, family_codetype);
  d_product_set_description (result, description);
  d_product_set_vat_type (result, vat_type);
  result->is_piece = es_pieza;
  result->is_man_power = es_mano;
  d_product_set_price (result, price);
  d_product_set_max_discount (result, max_discount);

  g_free(family_codetype);
  g_free(description);

  return result;
}

/**
 * d_product_init:
 * @product: Producto al que se le inicializarn todos los atributos.
 *
 * Inicializa los atributos de un producto a un valor predeterminado segn el tipo.
 *
 * Returns:
 * 0 si la funcin ha acabado correctamente.
 */
gint d_product_init (Product *product) {
    product->product_code = ULONG_MAX;
    product->family_code  = NULL;
    product->description  = NULL;
    product->vat_type      = -1;
    product->is_piece      = FALSE;
    product->is_man_power = FALSE;
    product->price = G_MINDOUBLE;
    product->max_discount= -1;

    return 0;
}

