/* received_book.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "received_book.h"
#include "participant.h"
#include "util.h"


#define D_RECEIVED_CONTENT_SIZE 8

static gint received_book_init(ReceivedBook *received_book);



/**
 * d_received_book_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #ReceivedBook vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
ReceivedBook * d_received_book_create_empty () {

    ReceivedBook * received_book;

    received_book = (ReceivedBook *) g_malloc0(sizeof(ReceivedBook));

    received_book_init(received_book);

    return received_book;
}




/**
 * d_received_book_set_invoice_number:
 * @received_book: Objeto en el que se establece el atributo.
 * @invoice_number: Atributo a insertar.
 *
 * Establece el atributo @invoice_number en el objeto. El atributo @invoice_number
 * indica el nmero de la factura de que estableci el proveedor.
 *
 */
void d_received_book_set_invoice_number (ReceivedBook * received_book, gchar * invoice_number) {
    if (received_book->invoice_number!= NULL) {
        g_free(received_book->invoice_number);
    }
    received_book->invoice_number = g_strdup(invoice_number);
}

/**
 * d_received_book_set_issue_date:
 * @received_book: Objeto en el que se establece el atributo.
 * @issue_date: Atributo a insertar.
 *
 * Establece el atributo @issue_date en el objeto. El atributo @issue_date
 * indica la fecha en la que el proveedor emiti la factura.
 *
 */
void d_received_book_set_issue_date (ReceivedBook * received_book, time_t issue_date) {
    received_book->issue_date = issue_date;
}
/**
 * d_received_book_set_number:
 * @received_book: Objeto en el que se establece el atributo.
 * @number: Atributo a insertar.
 *
 * Establece el atributo @number en el objeto. El atributo @number
 * indica el nmero de recepcin de la factura tratada en esta tupla.
 *
 */
void d_received_book_set_number (ReceivedBook * received_book, gulong number) {
    received_book->number = number;
}
/**
 * d_received_book_set_supplier_code:
 * @received_book: Objeto en el que se establece el atributo.
 * @supplier_code: Atributo a insertar.
 *
 * Establece el atributo @supplier_code en el objeto. El atributo @supplier_code
 * indica el cdigo del proveedor que emiti la factura.
 *
 */
void d_received_book_set_supplier_code (ReceivedBook * received_book, gulong supplier_code) {
    received_book->supplier_code = supplier_code;
}

/**
 * d_received_book_set_trade_name:
 * @received_book: Objeto en el que se establece el atributo.
 * @trade_name: Atributo a insertar.
 *
 * Establece el atributo @trade_name en el objeto. El atributo @trade_name
 * indica la razn social del proveedor.
 *
 */
void d_received_book_set_trade_name (ReceivedBook * received_book, gchar * trade_name) {
    if (received_book->trade_name!= NULL) {
        g_free(received_book->trade_name);
    }
    received_book->trade_name= g_strdup(trade_name);
}

/**
 * d_received_book_set_vat_type:
 * @received_book: Objeto en el que se establece el atributo.
 * @vat_type: Atributo a insertar.
 *
 * Establece el atributo @vat_type en el objeto. El atributo @vat_type
 * indica el tipo de iva de las lneas que van agrupadas en este objeto.
 *
 */
void d_received_book_set_vat_type (ReceivedBook * received_book, gdouble vat_type) {
    received_book->vat_type = vat_type;
}
/**
 * d_received_book_set_total:
 * @received_book: Objeto en el que se establece el atributo.
 * @total: Atributo a insertar.
 *
 * Establece el atributo @total en el objeto. El atributo @total
 * indica el total de las lneas de la factura agrupadas con el iva aplicado.
 *
 */
void d_received_book_set_total (ReceivedBook * received_book, gdouble total) {
    received_book->total = total;
}


/**
 * d_received_book_set_line_subtotal:
 * @received_book: Objeto en el que se establece el atributo.
 * @line_subtotal: Atributo a insertar.
 *
 * Establece el atributo @line_subtotal en el objeto. El atributo @line_subtotal
 * indica la suma del total de las lneas de la factura sin el iva.
 *
 */
void d_received_book_set_line_subtotal (ReceivedBook * received_book, gdouble line_subtotal) {
    received_book->line_subtotal=line_subtotal;
}

/**
 * d_received_book_set_total_summary:
 * @received_book: Objeto en el que se establece el atributo.
 * @total_summary: Atributo a insertar.
 *
 * Establece el atributo @total_summary en el objeto. El atributo @total_summary
 * indica si el objeto contiene el resumen de totales.
 *
 */
void d_received_book_set_total_summary (ReceivedBook * received_book, gboolean total_summary) {
    received_book->total_summary=total_summary;
}


/**
 * d_received_book_set_vat_summary:
 * @received_book: Objeto en el que se establece el atributo.
 * @vat_summary: Atributo a insertar.
 *
 * Establece el atributo @vat_summary en el objeto. El atributo @vat_summary
 * indica si el objeto contiene el resumen por Iva.
 *
 */
void d_received_book_set_vat_summary (ReceivedBook * received_book, gboolean vat_summary) {
    received_book->vat_summary=vat_summary;
}


/**
 * d_received_book_get_number:
 * @received_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_received_book_get_number (ReceivedBook * received_book) {
    return (received_book->number);
}

/**
 * d_received_book_get_total_summary:
 * @received_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_received_book_get_total_summary (ReceivedBook * received_book) {
    return (received_book->total_summary);
}

/**
 * d_received_book_get_vat_summary:
 * @received_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_received_book_get_vat_summary (ReceivedBook * received_book) {
    return (received_book->vat_summary);
}

/**
 * d_received_book_get_supplier_code:
 * @received_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_received_book_get_supplier_code (ReceivedBook * received_book) {
    return (received_book->supplier_code);
}


/**
 * d_received_book_get_invoice_number:
 * @received_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_received_book_get_invoice_number (ReceivedBook * received_book) {
    return g_strdup(received_book->invoice_number);
}

/**
 * d_received_book_get_issue_date:
 * @received_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t d_received_book_get_issue_date (ReceivedBook * received_book) {
    return (received_book->issue_date);

}


/**
 * d_received_book_get_trade_name:
 * @received_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_received_book_get_trade_name (ReceivedBook * received_book) {
    return g_strdup(received_book->trade_name);

}

/**
 * d_received_book_get_vat_type:
 * @received_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_received_book_get_vat_type (ReceivedBook * received_book) {
    return (received_book->vat_type);

}

/**
 * d_received_book_get_total:
 * @received_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_received_book_get_total (ReceivedBook * received_book) {
    return (received_book->total);

}


/**
 * d_received_book_get_line_subtotal:
 * @received_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_received_book_get_line_subtotal (ReceivedBook * received_book) {
    return received_book->line_subtotal;
}

/**
 * d_received_book_clear:
 * @received_book: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_received_book_clear (ReceivedBook * received_book) {

    if (received_book!=NULL) {

	if (received_book->invoice_number!=NULL) {
	    g_free(received_book->invoice_number);
	}
	if (received_book->trade_name!=NULL) {
	    g_free(received_book->trade_name);
	}
	g_free(received_book);
	received_book = NULL;
    }

    return 0;
}


/**
 * d_received_book_array_to_content:
 * @array: Array de grupo de lneas de facturas de pago que se transformarn en listables.
 * @reccount: Puntero a un %gulong que indicar la suma total de los objetos transformados.
 *
 * Funcin utilizada para transformar un array de objetos en un array de arrays de cadenas
 * de caracteres listables. Es utilizada para realizar los listados impresos en papel.
 *
 * Returns:
 * Un triple puntero a %gchar, es decir, un puntero a un array de arrays de cadenas de caracteres. Se
 * podra ver como un puntero a una tabla con filas y colummnas de cadenas de caracteres.
 */
gchar ***d_received_book_array_to_content (ReceivedBook **array, gulong *reccount) {
    gint     array_size = 0;
    gchar ***content = NULL;
    gulong * receipt_number = (gulong *) g_malloc0(sizeof(gulong));
    gchar **invoice_number=(gchar **) g_malloc0(sizeof(gchar *)*2);

    invoice_number[1]=NULL;
    *reccount = 0;
    *receipt_number = 1;
    while(array[array_size++]!=NULL);

    content = (gchar ***) malloc(sizeof(gchar **)*array_size);

    while(array[*reccount] != NULL) {
	content[*reccount] = d_received_book_get_content (array[*reccount],receipt_number,invoice_number);
	(*reccount)++;
        (*receipt_number)++;
    }

    if (invoice_number[0]!=NULL)
	g_free(invoice_number[0]);
    g_free(invoice_number);
    content[*reccount] = NULL;

    return content;

}

/**
 * d_received_book_get_content_title:
 *
 * Devuelve las cabaceras de cada pgina de los listados para el objeto factura recibida del libro de iva.
 *
 * Returns:
 * Un array de cadena de caracteres con las cabeceras de pgina. EL array acaba en NULL.
 */
gchar **d_received_book_get_content_title () {
    gchar **content = (gchar **) g_malloc0 (sizeof(gchar *)*d_received_book_get_content_size ()+1);
    gint index = 0;

    content[index++] = g_strdup(_("No."));
    content[index++] = g_strdup(_("DATE"));
    content[index++] = g_strdup(_("INV. NO."));
    content[index++] = g_strdup(_("TRADE NAME"));
    content[index++] = g_strdup(_("T.I."));
    content[index++] = g_strdup(_("VAT"));
    content[index++] = g_strdup(_("S. Q."));
    content[index++] = g_strdup(_("TOTAL"));
    content[index++] = NULL;

    return content;
}

/**
 * d_received_book_get_listing_title:
 *
 * Obtiene el ttulo del listado.
 *
 * Returns:
 * El ttulo del listado.
 */
gchar *d_received_book_get_listing_title () {
    gchar *content = g_strdup_printf(_("RECEIVED INVOICES REGISTER BOOK"));
    return content;
}

/**
 * d_received_book_get_listing_columns_width:
 *
 * Indica los anchos de columna de los listados facturas emitidas.
 *
 * Returns:
 * Un array de enteros que indican el ancho de cada columna en el listado. EL array acaba en -1.
 */
gint *d_received_book_get_listing_columns_width () {
    gint *widths = (gint *) g_malloc0 (sizeof(gint)*d_received_book_get_content_size ()+1);
    gint index = 0;

    widths[index++] = 5;
    widths[index++] = 9;
    widths[index++] = 9;
    widths[index++] = 45;
    widths[index++] = 9;
    widths[index++] = 6;
    widths[index++] = 8;
    widths[index++] = 9;
    widths[index++] = -1;

    return widths;
}

/**
 * d_received_book_get_listing_columns_alignment:
 *
 * Obtiene el alineamiento de las columnas del listado. Los alineamientos pueden ser U_LEFT, RIGHT o
 * CENTER.
 *
 * Returns:
 * Un array con los alineamientos de cada columna. EL array acaba en -1.
 */
gint *d_received_book_get_listing_columns_alignment () {
    gint *orientation = (gint *) g_malloc0 (sizeof(gint)*d_received_book_get_content_size ()+1);
    gint index = 0;

    orientation[index++] = U_LEFT;
    orientation[index++] = U_LEFT;
    orientation[index++] = U_LEFT;
    orientation[index++] = U_LEFT;
    orientation[index++] = U_RIGHT;
    orientation[index++] = U_RIGHT;
    orientation[index++] = U_RIGHT;
    orientation[index++] = U_RIGHT;
    orientation[index++] = -1;

    return orientation;
}

/**
 * d_received_book_get_content_size:
 *
 * Obtiene el nmero de columnas que tendr el listado.
 *
 * Returns:
 * El nmero de columnas.
 */
gint d_received_book_get_content_size () {
    return (D_RECEIVED_CONTENT_SIZE);
}


/**
 * d_received_book_get_content:
 * @received_book: Objeto factura a partir de la que se obtendrn los valores para el listado.
 * @received_num: Orden de recepcin de factura.
 * @previous_invoice_number: Nmero de factura tratado en la anterior tupla, para comprobar si las lneas factura
 * son de la misma factura o no.
 *
 * Transforma un objeto factura en un array de cadenas de caracteres para aadir al listado
 * del facturas recibidas.
 *
 * Returns:
 * Array con los valores a sacar por listado en modo de cadenas de caracteres.
 */
gchar **d_received_book_get_content (ReceivedBook *received_book,gulong * received_num,gchar ** previous_invoice_number) {
    gchar **content = (gchar **)g_malloc0(sizeof(gchar*) *
				       d_received_book_get_content_size ()+1);
    gint    index   = 0;
    gulong number=ULONG_MAX;
    time_t issue_date = d_received_book_get_issue_date (received_book);
    gchar  *temp = u_get_parsed_date (&issue_date);
    gdouble subtotal;
    gdouble vat_share;
    gdouble total;
    gdouble vat_temp= d_received_book_get_vat_type (received_book);
    gchar * temp_number=NULL;
    gchar * temp_invoice_number=NULL;


    temp_invoice_number=d_received_book_get_invoice_number (received_book);
    if ((d_received_book_get_total_summary (received_book)==FALSE) &&
	(d_received_book_get_vat_summary (received_book)==FALSE)) {
	if (d_received_book_get_number (received_book)==ULONG_MAX) {
	    if (previous_invoice_number[0]!=NULL) {
		if (!strcmp(previous_invoice_number[0],temp_invoice_number)) {
		    (*received_num)--;
		}
	    }
	    d_received_book_set_number (received_book,*received_num);
	}
	number = d_received_book_get_number (received_book);
    }
    subtotal = d_received_book_get_line_subtotal (received_book);

    total = subtotal * (1+((vat_temp*1.0)/100.0));

    vat_share =((vat_temp*1.0)/100.0) * subtotal;

    u_delete_intros (temp);


    temp_number =d_received_book_get_invoice_number (received_book);
    if (d_received_book_get_total_summary (received_book)==FALSE) {
	if (d_received_book_get_vat_summary (received_book)==FALSE) {
	    content[index++] = g_strdup_printf("%lu",number);
	    content[index++] = temp;
	    content[index++] = temp_number;
	    content[index++] = d_received_book_get_trade_name (received_book);
	    content[index++] = g_strdup_printf("%.2f", subtotal);
	    if ((u_round_to_decimal (vat_temp,2))>0.0)
		content[index++] = g_strdup_printf("%.2f", vat_temp);
	    else
		content[index++] = g_strdup("EX.");
	    content[index++] = g_strdup_printf("%.2f", vat_share);
	    content[index++] = g_strdup_printf("%.2f", total);
	    content[index++] = NULL;
	} else {
	    content[index++] = g_strdup("IVA:");
	    content[index++] = g_strdup_printf("%.2f", d_received_book_get_vat_type (received_book));
	    content[index++] = g_strdup_printf(_("SUPPORTED Q. TOTAL SUM: %.2f"), d_received_book_get_total (received_book));
	    content[index++] = g_strdup("");
	    content[index++] = g_strdup(_("T.I. TOTAL SUM"));
	    content[index++] = g_strdup("");
	    content[index++] = g_strdup_printf("%.2f", d_received_book_get_line_subtotal (received_book));
	    content[index++] = NULL;
	}
    } else {
	/*En total esta a suma dos iva's*/
        /*En subtotal lneas esta a suma de base imponibles*/
	content[index++] = g_strdup(_("SUPP. Q. TOTAL"));
	content[index++] = g_strdup("");
	content[index++] = g_strdup("");
	content[index++] = g_strdup_printf("%.2f", d_received_book_get_total (received_book));
	content[index++] = g_strdup(_("T.I. TOTAL"));
	content[index++] = g_strdup("");
	content[index++] = g_strdup_printf("%.2f", d_received_book_get_line_subtotal (received_book));
	content[index++] = g_strdup("");
	content[index++] = NULL;

    }

    if (previous_invoice_number[0]!=NULL)
	g_free(previous_invoice_number[0]);
    previous_invoice_number[0]=temp_invoice_number;

    return content;
}

/**
 * d_received_book_calculate_total_from_list:
 * @received_books: Objeto #IssuedBook a partir del cual se realizarn los clculos.
 *
 * Calcula los totales a partir de un array de objetos #IssuedBook.
 *
 * Returns:
 * Array con un elemento que contendr los totales calculados.
 */
ReceivedBook **  d_received_book_calculate_total_from_list (ReceivedBook ** received_books) {
    ReceivedBook * received_book=NULL;
    ReceivedBook ** result=(ReceivedBook **) g_malloc0((sizeof(ReceivedBook*)*2));
    gulong index=0;
    gdouble vat_total =0;
    gdouble subtotal_sum =0;
    gdouble total_sum = 0;

    if (received_books!=NULL) {
	while (received_books[index]!=NULL) {
	    gdouble total=0;
	    gdouble subtotal=0;
	    gdouble vat_share=0;
	    gdouble vat_temp= d_received_book_get_vat_type (received_books[index]);

	    subtotal = d_received_book_get_line_subtotal (received_books[index]);
	    total = subtotal * (1+((vat_temp*1.0)/100.0));

	    vat_share =((vat_temp*1.0)/100.0) * subtotal;

	    vat_total = vat_total + vat_share;
	    subtotal_sum = subtotal_sum + subtotal;
            total_sum = total_sum + total;
            index++;
	}
    }
    received_book = d_received_book_create_empty ();
    g_print("\n%.2f",subtotal_sum);
    g_print("\n%.2f",vat_total);
    d_received_book_set_total_summary (received_book,TRUE);
    d_received_book_set_invoice_number (received_book," ");
    d_received_book_set_trade_name (received_book,_("TOTALS:"));
    d_received_book_set_line_subtotal (received_book,subtotal_sum);
    d_received_book_set_total (received_book,vat_total);
    result[0]=received_book;
    return result;
    
}

/**
 * d_received_book_concat:
 * @received_books: Primer array a concatenar.
 * @received_books2: Segundo array a concatenar.
 *
 * Concatena dos arrays de objetos del libro de facturas recibidas. #received_book2 ir en segundo lugar en la concatenacin.
 *
 * Returns:
 * EL nuevo array que contiene los dos arrays pasados como parmetros.
 */
ReceivedBook ** d_received_book_concat (ReceivedBook ** received_books,ReceivedBook ** received_books2) {
    gulong size=0;
    gulong size2=0;
    gulong index=0;
    gulong index2=0;
    ReceivedBook ** ib=NULL;
    if (received_books!=NULL) {
	while (received_books[size]!=NULL) {
	    size++;
	}


	if (received_books2!=NULL) {
	    while (received_books2[size2]!=NULL) {
		size2++;
	    }
	}
	ib = (ReceivedBook **) g_malloc0(sizeof(ReceivedBook*)*(size+size2+1));
	index = 0;
	while (index<(size)) {
	    ib[index]=received_books[index];
	    received_books[index]=NULL;
	    index++;
	    
	}

	index=size;
	while (index<(size+size2)) {
	    ib[index]=received_books2[index2];
	    received_books2[index2]=NULL;
	    index++;
	    index2++;
	}
        g_print("\n%lu\n",size+size2+1);

    }
    return ib;
}


/* Private functions */

static gint received_book_init(ReceivedBook *received_book) {
    received_book->number=ULONG_MAX;
    received_book->supplier_code=ULONG_MAX;
    received_book->invoice_number=NULL;
    received_book->issue_date = time(NULL);
    received_book->trade_name=NULL;
    received_book->vat_type=-1;
    received_book->line_subtotal=-1;
    received_book->total=-1;
    received_book->total_summary=FALSE;
    received_book->vat_summary=FALSE;

    return 0;
}

