/* repair_line_object.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "model.h"
#include "repair_line_object.h"
#include "util.h"


static gint repair_line_init(Repair_line *repair_line);

/**
 * d_repair_line_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Repair_line vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Repair_line  *d_repair_line_create_empty (){					    
    Repair_line *repair_line = (Repair_line *) g_malloc(sizeof(Repair_line));

    repair_line_init(repair_line);
    
    return repair_line;
}


/**
 * d_repair_line_clear:
 * @repair_line: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_repair_line_clear (Repair_line *repair_line) {

  if (repair_line!=NULL) {
    if (repair_line->product!=NULL) {
      if (repair_line->product->is_piece) {
	d_piece_clear ((Piece *) repair_line->product);
      } else { 
	if (repair_line->product->is_man_power)
	  d_man_power_clear ((ManPower *) repair_line->product);
	else {
	  d_product_clear (repair_line->product);
	}
      }
    }
    
    repair_line->product=NULL;
    
    d_stock_clear (repair_line->stock);

    repair_line->stock=NULL;

    g_free(repair_line);
    repair_line=NULL;
  }
  
  return 0;
}

/**
 * d_repair_line_set_line_number:
 * @repair_line: Objeto en el que se establece el atributo.
 * @line_number: Atributo a insertar.
 *
 * Establece el atributo @line_number en el objeto. El atributo @line_number
 * indica el nmero de la lnea de la reparacin.
 *
 */
void d_repair_line_set_line_number (Repair_line * repair_line, gulong line_number) {
  repair_line->line_number = line_number;
}

/**
 * d_repair_line_set_line_position_number:
 * @repair_line: Objeto en el que se establece el atributo.
 * @line_position_number: Atributo a insertar.
 *
 * Establece el atributo @line_position_number en el objeto. El atributo @line_position_number
 * indica el nmero de lnea dentro de la reparacin.
 *
 */
void d_repair_line_set_line_position_number (Repair_line * repair_line, gulong line_position_number) {
  repair_line->line_position_number = line_position_number;
}

/**
 * d_repair_line_set_quantity:
 * @repair_line: Objeto en el que se establece el atributo.
 * @quantity: Atributo a insertar.
 *
 * Establece el atributo @quantity en el objeto. El atributo @quantity
 * indica el nmero de unidades de una pieza en la lnea de reparacin.
 *
 */
void d_repair_line_set_quantity (Repair_line * repair_line, gdouble quantity) {
  repair_line->quantity = quantity;
}

/**
 * d_repair_line_set_earlier_quantity:
 * @repair_line: Objeto en el que se establece el atributo.
 * @earlier_quantity: Atributo a insertar.
 *
 * Establece el atributo @earlier_quantity en el objeto. El atributo @earlier_quantity
 * indica las unidades que haba antes de modificar el nmero de unidades.
 *
 */
void d_repair_line_set_earlier_quantity (Repair_line * repair_line, gdouble earlier_quantity) {
  repair_line->earlier_quantity = earlier_quantity;
}

/**
 * d_repair_line_set_price:
 * @repair_line: Objeto en el que se establece el atributo.
 * @price: Atributo a insertar.
 *
 * Establece el atributo @price en el objeto. El atributo @price
 * indica el precio de la lnea de reparacin. Este precio solo se multiplica por
 * el nmero de unidades en el clculo de totales de lnea.
 *
 */
void d_repair_line_set_price (Repair_line * repair_line, gdouble price) {
  repair_line->price = price;
}

/**
 * d_repair_line_set_discount:
 * @repair_line: Objeto en el que se establece el atributo.
 * @discount: Atributo a insertar.
 *
 * Establece el atributo @discount en el objeto. El atributo @discount
 * indica el descuento de la lnea de la reparacin.
 *
 */
void d_repair_line_set_discount (Repair_line * repair_line, gdouble discount) {
    if (discount>100)
	repair_line->discount=100;
    else if (discount<0)
	repair_line->discount=0;
    else
	repair_line->discount=discount;
}

/**
 * d_repair_line_set_cost_price:
 * @repair_line: Objeto en el que se establece el atributo.
 * @cost_price: Atributo a insertar.
 *
 * Establece el atributo @cost_price en el objeto. El atributo @cost_price
 * indica el precio de coste de la lnea de reparacin.
 *
 */
void d_repair_line_set_cost_price (Repair_line * repair_line, gdouble cost_price) {
  repair_line->cost_price = cost_price;
}

/**
 * d_repair_line_set_historical_price:
 * @repair_line: Objeto en el que se establece el atributo.
 * @historical_price: Atributo a insertar.
 *
 * Establece el atributo @historical_price en el objeto. El atributo @historical_price
 * indica el precio hitrico de la lnea de reparacin.
 *
 */
void d_repair_line_set_historical_price (Repair_line * repair_line, gdouble historical_price) {
  repair_line->historical_price = historical_price;
}

/**
 * d_repair_line_set_historical_vat_type:
 * @repair_line: Objeto en el que se establece el atributo.
 * @historical_vat_type: Atributo a insertar.
 *
 * Establece el atributo @historical_vat_type en el objeto. El atributo @historical_vat_type
 * indica el precio hitrico de la lnea de reparacin.
 *
 */
void d_repair_line_set_historical_vat_type (Repair_line * repair_line, gdouble historical_vat_type) {
  repair_line->historical_vat_type = historical_vat_type;
}

/**
 * d_repair_line_set_vat_type:
 * @repair_line: Objeto en el que se establece el atributo.
 * @vat_type: Atributo a insertar.
 *
 * Establece el atributo @vat_type en el objeto. El atributo @vat_type
 * indica el tipo de Iva de la lnea de reparacin.
 *
 */
void d_repair_line_set_vat_type (Repair_line * repair_line, gdouble vat_type) {
  repair_line->vat_type = vat_type;
}

/**
 * d_repair_line_set_product:
 * @repair_line: Objeto en el que se establece el atributo.
 * @product: Atributo a insertar.
 *
 * Establece el atributo @product en el objeto. El atributo @product
 * indica el producto al que se refiere la lnea de reparacin.
 *
 */
void d_repair_line_set_product (Repair_line * repair_line, Product *product) {
  repair_line->product = product;
}

/**
 * d_repair_line_set_stock:
 * @repair_line: Objeto en el que se establece el atributo.
 * @stock: Atributo a insertar.
 *
 * Establece el atributo @stock en el objeto. El atributo @stock
 * indica el stock del producto en un almacn.
 *
 */
void d_repair_line_set_stock (Repair_line * repair_line, Stock *stock) {
  repair_line->stock = stock;
}

/**
 * d_repair_line_get_line_number:
 * @repair_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_repair_line_get_line_number (Repair_line * repair_line) {
  return repair_line->line_number;
}

/**
 * d_repair_line_get_line_position_number:
 * @repair_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_repair_line_get_line_position_number (Repair_line * repair_line) {
  return repair_line->line_position_number;
}

/**
 * d_repair_line_get_quantity:
 * @repair_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_repair_line_get_quantity (Repair_line * repair_line) {
  return repair_line->quantity;
}

/**
 * d_repair_line_get_earlier_quantity:
 * @repair_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_repair_line_get_earlier_quantity (Repair_line * repair_line) {
  return repair_line->earlier_quantity;
}

/**
 * d_repair_line_get_price:
 * @repair_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_repair_line_get_price (Repair_line * repair_line) {
  return repair_line->price;
}

/**
 * d_repair_line_get_discount:
 * @repair_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_repair_line_get_discount (Repair_line * repair_line) {
  return repair_line->discount;
}

/**
 * d_repair_line_get_vat_type:
 * @repair_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_repair_line_get_vat_type (Repair_line * repair_line) {
  return repair_line->vat_type;
}

/**
 * d_repair_line_get_cost_price:
 * @repair_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_repair_line_get_cost_price (Repair_line * repair_line) {
  return repair_line->cost_price;
}

/**
 * d_repair_line_get_historical_price:
 * @repair_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_repair_line_get_historical_price (Repair_line * repair_line) {
  return repair_line->historical_price;
}

/**
 * d_repair_line_get_historical_vat_type:
 * @repair_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_repair_line_get_historical_vat_type (Repair_line * repair_line) {
  return repair_line->historical_vat_type;
}

/**
 * d_repair_line_get_product:
 * @repair_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Product *d_repair_line_get_product (Repair_line * repair_line) {
  return repair_line->product;
}

/**
 * d_repair_line_get_stock:
 * @repair_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Stock *d_repair_line_get_stock (Repair_line * repair_line) {
  return repair_line->stock;
}


/**
 * d_repair_line_calculate_total:
 * @repair_line: Objeto del que se desea obtener el atributo.
 *
 * Calcula el importe de la lnea de la reparacin.
 *
 * Returns: El importe.
 */
gdouble d_repair_line_calculate_total (Repair_line * repair_line) {
    Product *product = d_repair_line_get_product (repair_line);
    /*Si no hay precio por lnea se utiliza el precio del producto*/
    gdouble precio_gdouble = d_product_get_price (product);
    /*se obtiene el precio de la lnea*/
    gdouble importe_gdouble = d_repair_line_get_price (repair_line);
    gdouble unidades_gulong = u_round_to_decimal (d_repair_line_get_quantity(repair_line),3);
    /*Se obtiene el descuento de la lnea*/
    gdouble dto_line_gint = d_repair_line_get_discount (repair_line);
    gdouble result = G_MINDOUBLE;

    if ((importe_gdouble==G_MINDOUBLE)&&(unidades_gulong >= 0)){
	/*Al existir un precio de la lnea es el que se tiene en cuenta y
	 se multiplica por el nmero de unidades de la lnea. No se tiene
	 en cuenta el descuento en este caso*/
	if (precio_gdouble!=G_MINDOUBLE) {
	    if (dto_line_gint != -1)
		result = (precio_gdouble-(precio_gdouble * dto_line_gint/100))*unidades_gulong;
	    else result = precio_gdouble*unidades_gulong;
	}
    } else {
	if (importe_gdouble!=G_MINDOUBLE) {
	    if (unidades_gulong>=0) {
		result = unidades_gulong*importe_gdouble;
	    } else {
		result = importe_gdouble;
	    }
	}
    }
    return result;
}

/**
 * d_repair_line_duplicate:
 * @repair_line: Objeto del que se desea obtener el duplicado.
 *
 * Duplica el objeto lnea de la reparacin.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
Repair_line *d_repair_line_duplicate (Repair_line *repair_line) {
  
  Repair_line *result = d_repair_line_create_empty ();
  gulong line_number = d_repair_line_get_line_number (repair_line);
  gulong line_position_number = d_repair_line_get_line_position_number (repair_line);
  gdouble quantity_gulong = d_repair_line_get_quantity (repair_line);
  gdouble earlier_quantity_gulong = d_repair_line_get_earlier_quantity (repair_line);
  gdouble price_gdouble = d_repair_line_get_price (repair_line);
  gdouble line_discount_gint = d_repair_line_get_discount (repair_line);
  gdouble vat_type_gint = d_repair_line_get_vat_type (repair_line);
  gdouble cost_price_gdouble = d_repair_line_get_cost_price (repair_line);
  gdouble historical_price_gdouble = d_repair_line_get_historical_price (repair_line);
  gdouble historical_vat_type_gdouble = d_repair_line_get_historical_vat_type (repair_line);
  Product *product = d_repair_line_get_product (repair_line);
  Stock *stock = d_repair_line_get_stock (repair_line);

  d_repair_line_set_line_number (result, line_number);
  d_repair_line_set_line_position_number (result, line_position_number);
  d_repair_line_set_quantity (result, quantity_gulong);
  d_repair_line_set_earlier_quantity (result, earlier_quantity_gulong);
  d_repair_line_set_price (result, price_gdouble);
  d_repair_line_set_discount (result, line_discount_gint);
  d_repair_line_set_cost_price (result, cost_price_gdouble);
  d_repair_line_set_historical_price (result, historical_price_gdouble);
  d_repair_line_set_historical_vat_type (result, historical_vat_type_gdouble);
  d_repair_line_set_vat_type (result, vat_type_gint);
  if (stock != NULL) 
    d_repair_line_set_stock (result, d_stock_duplicate (stock));

  if (product!=NULL) {
    if (product->is_piece) {
      d_repair_line_set_product (result, (Product *) d_piece_duplicate ((Piece *) product));
    } else { 

      if (product->is_man_power)
	d_repair_line_set_product (result, (Product *)d_man_power_duplicate ((ManPower *) product));
      else {
	d_repair_line_set_product(result, d_product_duplicate (product));
      }
    }
  }
    
  return result;

  
}


/* Private functions */

static gint repair_line_init(Repair_line *repair_line) {

    repair_line->line_number = ULONG_MAX;
    repair_line->line_position_number = ULONG_MAX;
    repair_line->quantity = G_MINDOUBLE;
    repair_line->earlier_quantity = G_MINDOUBLE;
    repair_line->price = G_MINDOUBLE;
    repair_line->discount = -1;
    repair_line->vat_type = -1;
    repair_line->cost_price = G_MINDOUBLE;
    repair_line->historical_price = G_MINDOUBLE;
    repair_line->historical_vat_type = G_MINDOUBLE;
    repair_line->product = NULL;
    repair_line->stock = NULL;
  
  return 0;
}
