/* shop.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "shop.h"


static gint shop_init(Shop *shop);

/**
 * d_shop_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Shop vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Shop *d_shop_create_empty () {
  Shop * shop = (Shop *) g_malloc(sizeof(Shop));

  shop_init(shop);

  return shop;
}

/**
 * d_shop_clear:
 * @shop: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_shop_clear (Shop *shop) {

  if (shop!=NULL) {
    d_participant_clear (shop->participant);
    if (shop->group_code!=NULL)
	g_free(shop->group_code);
    if (shop->repairer_code!=NULL)
      g_free(shop->repairer_code);
	    
    g_free(shop);
  }

  return 0;
   
}

/**
 * d_shop_set_participant:
 * @shop: Objeto en el que se establece el atributo.
 * @participant: Atributo a insertar.
 *
 * Establece el atributo @participant en el objeto. El atributo @participant
 * indica el rol de participante de una tienda.
 *
 */
void d_shop_set_participant (Shop * shop, Participant * participant) {
    shop->participant = participant;
}

/**
 * d_shop_set_group_code:
 * @shop: Objeto en el que se establece el atributo.
 * @group_code: Atributo a insertar.
 *
 * Establece el atributo @group_code en el objeto. El atributo @group_code
 * indica el cdigo de cada tienda de una empresa del grupo.
 *
 */
void d_shop_set_group_code (Shop * shop, gchar * group_code) {
    if(shop->group_code!=NULL) {
	g_free(shop->group_code);
    }
    shop->group_code = g_strdup (group_code);
}

/**
 * d_shop_set_enterprise_code:
 * @shop: Objeto en el que se establece el atributo.
 * @enterprise_code: Atributo a insertar.
 *
 * Establece el atributo @enterprise_code en el objeto. El atributo @enterprise_code
 * indica el cdigo de la empresa a la que pertenece la tienda.
 *
 */
void d_shop_set_enterprise_code (Shop * shop,gulong enterprise_code) {
    shop->enterprise_code = enterprise_code;
}

/**
 * d_shop_set_repairer_code:
 * @shop: Objeto en el que se establece el atributo.
 * @repairer_code: Atributo a insertar.
 *
 * Establece el atributo @repairer_code en el objeto. El atributo @repairer_code
 * indica el cdigo de centro reparador de la tienda.
 *
 */
void d_shop_set_repairer_code (Shop * shop,gchar * repairer_code) {
    if(shop->repairer_code!=NULL) {
	g_free(shop->repairer_code);
    }
    shop->repairer_code = g_strdup (repairer_code);
}

/**
 * d_shop_get_participant:
 * @shop: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Participant *d_shop_get_participant (Shop * shop) {
    return shop->participant;
}

/**
 * d_shop_get_group_code:
 * @shop: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_shop_get_group_code (Shop * shop) {
    if (shop->group_code==NULL) {
        return g_strdup("GE");
    } else {
	return g_strdup(shop->group_code);
    }
}

/**
 * d_shop_get_enterprise_code:
 * @shop: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_shop_get_enterprise_code (Shop * shop) {
    return shop->enterprise_code;
}

/**
 * d_shop_get_repairer_code:
 * @shop: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_shop_get_repairer_code (Shop * shop) {
    return g_strdup(shop->repairer_code);
}

/**
 * d_shop_duplicate:
 * @original_shop: Objeto del que se desea obtener el duplicado.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
Shop * d_shop_duplicate (Shop * original_shop) {
    Shop * shop=d_shop_create_empty ();
    if (original_shop!=NULL) {
	Participant * participant = d_participant_duplicate (d_shop_get_participant (original_shop));
	gchar * group_code = d_shop_get_group_code (original_shop);
	gchar * repairer_code = d_shop_get_repairer_code (original_shop);

	d_shop_set_participant (shop,participant);
	d_shop_set_enterprise_code (shop,d_shop_get_enterprise_code (original_shop));
	d_shop_set_group_code (shop,group_code);
	d_shop_set_repairer_code (shop,repairer_code);

	g_free(repairer_code);
	g_free(group_code);
    }
    return shop;
}

/* Private functions*/

static gint shop_init(Shop *shop)
{
  shop->participant = NULL;
  shop->enterprise_code = ULONG_MAX;
  shop->group_code = NULL;
  shop->repairer_code = NULL;

  return 0;
}

