
/* stock.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "stock.h"
#include "util.h"

static gint stock_init(Stock *stock);

/**
 * d_stock_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #stock vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Stock * d_stock_create_empty () {
    Stock *stock = (Stock *) g_malloc(sizeof(Stock));

    stock_init(stock);

    return stock;
}


/**
 * d_stock_clear:
 * @stock: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_stock_clear (Stock *stock) {
  if (stock!=NULL) {
    if (stock->piece != NULL)
      d_piece_clear (stock->piece);
    
    if (stock->location!=NULL) {
      g_free(stock->location);
      stock->location = NULL;
    }
    g_free(stock);

    stock=NULL;
  }
  return 0;
}

/**
 * d_stock_set_piece:
 * @stock: Objeto en el que se establece el atributo.
 * @piece: Atributo a insertar.
 *
 * Establece el atributo @piece en el objeto. El atributo @piece
 * indica la pieza a la que se refiere el stock.
 *
 */
void d_stock_set_piece (Stock *stock, Piece *piece) {
    stock->piece = piece;
}

/**
 * d_stock_set_enterprise_code:
 * @stock: Objeto en el que se establece el atributo.
 * @enterprise_code: Atributo a insertar.
 *
 * Establece el atributo @enterprise_code en el objeto. El atributo @enterprise_code
 * indica el cdigo de la empresa a la que se refiere el stock.
 *
 */
void d_stock_set_enterprise_code (Stock *stock, gulong enterprise_code) {
    stock->enterprise_code = enterprise_code;
}

/**
 * d_stock_set_warehouse_code:
 * @stock: Objeto en el que se establece el atributo.
 * @warehouse_code: Atributo a insertar.
 *
 * Establece el atributo @warehouse_code en el objeto. El atributo @warehouse_code
 * indica 
 *
 */
void d_stock_set_warehouse_code (Stock *stock, gulong warehouse_code) {
    stock->warehouse_code = warehouse_code;
}

/**
 * d_stock_set_quantity:
 * @stock: Objeto en el que se establece el atributo.
 * @quantity: Atributo a insertar.
 *
 * Establece el atributo @quantity en el objeto. El atributo @quantity
 * indica la uniades en stock de esa pieza.
 *
 */
void d_stock_set_quantity (Stock *stock, gdouble quantity) {
    stock->quantity = quantity;
}

/**
 * d_stock_set_previous_quantity:
 * @stock: Objeto en el que se establece el atributo.
 * @quantity: Atributo a insertar.
 *
 * Establece el atributo @quantity en el objeto. El atributo @quantity
 * indica la uniades en stock de esa pieza.
 *
 */
void d_stock_set_previous_quantity (Stock *stock, gdouble quantity) {
    stock->previous_quantity = quantity;
}

/**
 * d_stock_set_minimum_stock:
 * @stock: Objeto en el que se establece el atributo.
 * @minimum_stock: Atributo a insertar.
 *
 * Establece el atributo @minimum_stock en el objeto. El atributo @minimum_stock
 * indica el stock mnimo de una pieza en un almacn.
 *
 */
void d_stock_set_minimum_stock (Stock *stock, gdouble minimum_stock) {
    stock->minimum_stock = minimum_stock;
}

/**
 * d_stock_set_location:
 * @stock: Objeto en el que se establece el atributo.
 * @location: Atributo a insertar.
 *
 * Establece el atributo @location en el objeto. El atributo @location
 * indica la ubicacin de una pieza en un almacn.
 *
 */
void d_stock_set_location (Stock *stock, gchar *location) {
    if (stock->location != NULL) {
	g_free(stock->location);
    }
    stock->location = g_strdup(location);
}

/**
 * d_stock_get_piece:
 * @stock: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Piece *d_stock_get_piece (Stock *stock) {
    return stock->piece;
}

/**
 * d_stock_get_enterprise_code:
 * @stock: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_stock_get_enterprise_code (Stock *stock) {
    return stock->enterprise_code;
}

/**
 * d_stock_get_warehouse_code:
 * @stock: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_stock_get_warehouse_code (Stock *stock) {
    return stock->warehouse_code;
}

/**
 * d_stock_get_quantity:
 * @stock: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_stock_get_quantity (Stock *stock) {
    return stock->quantity;
}

/**
 * d_stock_get_previous_quantity:
 * @stock: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_stock_get_previous_quantity (Stock *stock) {
    return stock->previous_quantity;
}

/**
 * d_stock_get_minimum_stock:
 * @stock: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_stock_get_minimum_stock (Stock *stock) {
    return stock->minimum_stock;
}

/**
 * d_stock_get_location:
 * @stock: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_stock_get_location (Stock *stock) {
    return g_strdup(stock->location);
}

/**
 * d_stock_duplicate:
 * @stock: Objeto del que se desea obtener el duplicado.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
Stock *d_stock_duplicate (Stock *stock) {
  
  Stock *result = d_stock_create_empty ();
  Piece *piece = d_stock_get_piece (stock);
  gulong enterprise_code = d_stock_get_enterprise_code (stock);
  gulong warehouse_code = d_stock_get_warehouse_code (stock);
  gdouble quantity = d_stock_get_quantity (stock);
  gdouble previous_quantity = d_stock_get_previous_quantity (stock);
  gdouble minimum_stock = d_stock_get_minimum_stock (stock);
  gchar *location = d_stock_get_location (stock);
  
  if (piece != NULL) 
    d_stock_set_piece (result, d_piece_duplicate (piece));
  d_stock_set_enterprise_code (result, enterprise_code);
  d_stock_set_warehouse_code (result, warehouse_code);
  d_stock_set_quantity (result, quantity);
  d_stock_set_previous_quantity (result, previous_quantity);
  d_stock_set_minimum_stock (result,minimum_stock);
  d_stock_set_location (result, location);
    

  g_free(location);

  return result;
}

/* Private functions*/

static gint stock_init(Stock *stock) {
    stock->piece = NULL;
    stock->enterprise_code = ULONG_MAX;
    stock->warehouse_code = ULONG_MAX;
    stock->quantity = G_MINDOUBLE;
    stock->previous_quantity = G_MINDOUBLE;
    stock->minimum_stock =  G_MINDOUBLE;
    stock->location = NULL;

    return 0;
}
