
/* document_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "document_model.h"

#include "util.h"
#include "enterprise_model.h"
#include "shop_model.h"
#include "warehouse_model.h"
#include "parameter_manager.h"

#define TABLE         "Documento"
#define ATTRIBUTES    "codigodocumento as codigodocumentodocument, \
    nombredocumento as nombredocumentodocument, \
    descripciondocumento as descripciondocumentodocument, \
    deFacturacion as deFacturaciondocument, \
    deReparacion as deReparaciondocument "

Document ** documents_list = NULL;
gint length_of_list;


/**
 * m_document_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_document_get_table () {
    return TABLE;
}

/**
 * m_document_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_document_get_attributes () {
    return ATTRIBUTES;
}

/**
 * m_document_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Document con los campos rellenados con los datos de la base de datos.
 */
Document *m_document_create (GdaRecordset *rs) {
    GdaField * field;
    gchar    * temp = NULL;
    Document *document = d_document_create_empty();

    field = gda_recordset_field_name(rs,"codigodocumentodocument");
    if (field!=NULL) {
	if(!gda_field_is_null(field)) {
	    document->document_code = igalia_field_get_bigint_value (field);
	    gda_field_free(field);

	    field = gda_recordset_field_name(rs,"nombredocumentodocument");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    document->name = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"descripciondocumentodocument");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    document->description = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

/*	    field = gda_recordset_field_name(rs,"descripciondocumentodocument");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    document->description = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }
*/
	    field = gda_recordset_field_name(rs,"deFacturaciondocument");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    document->from_invoice = gda_field_get_boolean_value(field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"deReparaciondocument");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    document->from_repair = gda_field_get_boolean_value(field);
		}
		gda_field_free(field);
	    }

	} else {
	    gda_field_free(field);
	}
    }
    
    return document;
}



/**
 * m_document_find_by_company: 
 * @company_code: Cdigo de la compaa de la cul se buscan los documentos.
 * @repair: Indica si los documentos son de reparacin  facturacin.
 *
 * Busca los documentos de facturacin o reparacin asociados a una compaia.
 *
 * Returns: Lista de documentos.
 */
Document ** m_document_find_by_company (gulong company_code, gboolean repair) {

    if ((documents_list==NULL)||(company_code!=ULONG_MAX)) {
	GdaCommand         * my_command;
	GdaConnection      * my_connection = m_model_get_connection ();
	GdaRecordset       * record_set = NULL;
	Document * document = NULL;
	Document **documents = NULL;
	gchar * gchar_query;
	gulong * tuples;
	gint * column;
	gint index;

	my_command = gda_command_new();
	gda_command_set_connection(my_command,my_connection);

	gchar_query = g_strconcat("SELECT ",
				  " documento.codigodocumento as codigodocumentodocument, ",
				  " documento.nombredocumento as nombredocumentodocument, ",
				  " documento.descripciondocumento as descripciondocumentodocument ",
				  " FROM documento",
				  NULL);

	if (company_code != ULONG_MAX){
	    gchar * gchar_query_aux = NULL;
	    gchar *company_code_gchar = g_strdup_printf("%lu", company_code);

	    if (repair)
		gchar_query_aux = g_strconcat(gchar_query,", necesitaDocumento "
					      " WHERE documento.codigodocumento = necesitadocumento.codigodocumento",
					      " AND documento.dereparacion = true",
					      " AND necesitadocumento.codigoaseguradora ='",company_code_gchar,"'",NULL);
	    else
		gchar_query_aux = g_strconcat(gchar_query,", necesitaDocumento "
					      " WHERE documento.codigodocumento = necesitadocumento.codigodocumento",
					      " AND documento.defacturacion = true",
					      " AND necesitadocumento.codigoaseguradora ='",company_code_gchar,"'",NULL);

	    g_free(gchar_query);
	    g_free(company_code_gchar);
	    gchar_query =  gchar_query_aux;
	    gchar_query_aux = NULL;
	}
	else {
	    gchar * gchar_query_aux = NULL;

	    if (repair)
		gchar_query_aux = g_strconcat(gchar_query,
					      " AND documento.dereparacion = true",
					      " ORDER BY participante.nombre",NULL);
	    else
		gchar_query_aux = g_strconcat(gchar_query,
					      " AND documento.defacturacion = true",
					      " ORDER BY participante.nombre",NULL);

	    g_free(gchar_query);
	    gchar_query =  gchar_query_aux;
	    gchar_query_aux = NULL;
	}

	tuples = (gulong *) g_malloc0(sizeof(gulong));
	m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);
       
       g_print(gchar_query);
	g_free(gchar_query);

	length_of_list = *tuples;
	column = (gint *) g_malloc0(sizeof(gint));

	if (record_set!=NULL) {
	    if (*tuples!=0) {

		if (company_code == ULONG_MAX) {

		    if (documents_list != NULL)
			m_document_clear_cached_array ();

		    documents_list = (Document **) g_malloc0(sizeof(Document*)*(*tuples+1));
		    documents = documents_list;
		} else documents = (Document **) g_malloc0(sizeof(Document*)*(*tuples+1));

		index = gda_recordset_move_first(record_set);
		while (!gda_recordset_eof(record_set)) {
		    if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		    (*column)=0;
		    document = m_document_create (record_set);
		    documents[index-1] = document;
		    index = gda_recordset_move_next(record_set);
		}
		documents[length_of_list]=NULL;
	    } else {
		if (company_code == ULONG_MAX) documents_list=NULL;
	    }
	    gda_recordset_close(record_set);
	} else {
	    if (company_code == ULONG_MAX) documents_list=NULL;
	}

	igalia_command_free(my_command);
	gda_recordset_free(record_set);

	g_print(_("\nCommand and recordset freeing completed\n"));
	g_free(tuples);
	g_free(column);

	if (company_code != ULONG_MAX) {
	    return documents;
	}
    }

    return documents_list;
}


// There will be only a few ids so we do n queries where n = docs_ids_size
// We avoid so do a query with a IN clause

/**
 * m_document_find_by_code_array:
 * @document_array: Array de documentos vaca con memoria reservada para los documentos.
 * @docs_ids: Lista de identificadores de documentos que deseamos buscar.
 *
 * Busca los documentos con cdigo en la lista de cdigos.
 *
 * Returns: Lista de documentos rellena con los documentos.
 */
Document ** m_document_find_by_code_array (Document **document_array, gulong *docs_ids) {

    GdaCommand         * command;
    GdaConnection      * connection = m_model_get_connection ();
    GdaRecordset       * record_set = NULL;
    gchar *gchar_query     = NULL;
    gchar *gchar_query_aux = NULL;
    gchar *temp            = NULL;
    gulong tuples      = 0;
    gint   index       = 0;
    gint   while_index = 0;

    command = gda_command_new();
    gda_command_set_connection(command,connection);

    while(docs_ids[while_index]!=ULONG_MAX) {
	gchar_query = g_strconcat("SELECT ",
				  m_document_get_attributes (),
				  " FROM ",
				  m_document_get_table (),
				  NULL);

	temp = g_strdup_printf("%lu",docs_ids[while_index]);
	gchar_query_aux = g_strconcat(gchar_query,
				      " WHERE documento.codigodocumento = ",
				      temp,
				      NULL);

	g_free(gchar_query);
	g_free(temp);
	gchar_query =  gchar_query_aux;
	gchar_query_aux = NULL;

	m_model_command_execute (&record_set, gchar_query, connection,
			      command, &tuples,0, TRUE);
	g_free(gchar_query);

	length_of_list = tuples;

	if (record_set!=NULL) {
	    if (tuples!=0) {

		index = gda_recordset_move_first(record_set);
		while (!gda_recordset_eof(record_set)) {
		    if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		    document_array[while_index] = m_document_create (record_set);

		    index = gda_recordset_move_next(record_set);
		}

	    }
	    gda_recordset_free(record_set);
	}
	while_index++;
    }

    document_array[while_index] = NULL;

    return document_array;
}


/**
 * m_document_find_by_condition:
 * @condition: Filtros de bsqueda. Se puede buscar por: nombre, descripcn del documento y
 * elegir si es de facturacin o reparacin
 * @tuples: Nmero de documentos que cumplieron la condicin. Este dato ser rellenado
 * al realizar la sentencia sql.
 *
 *  Busca los documentos que complan los filtros de la
 * estructura ConditionFindDocument.
 *
 * Returns:
 * Lista con los documentos que cumplen la condicin.
 */
Document **m_document_find_by_condition (ConditionFindDocument *condition,
					     gulong *tuples) {
    GdaCommand    *command;
    GdaConnection *connection = m_model_get_connection ();
    GdaRecordset  *record_set = NULL;
    gchar         *temp;
    gchar         *query;
    gchar         *query_aux;
    gboolean       correct;
    Document     **list=NULL;
    gint           index = -1;

    command = gda_command_new();
    gda_command_set_connection(command, connection);
    query = g_strconcat("SELECT ", m_document_get_attributes (),
			" FROM ",  m_document_get_table (),
			" WHERE ", NULL);

    if (condition->name!=NULL) {
	temp = g_strdup(condition->name);
	query_aux = g_strconcat(query, "(nombreDocumento ilike '", temp, "%') AND ", NULL);
	g_free(temp);
	g_free(query);
	query = query_aux;
	query_aux = NULL;
    }

    if (condition->description!=NULL) {
	temp = g_strdup(condition->description);
	query_aux = g_strconcat(query, "(descripcionDocumento ilike '", temp, "%') AND ", NULL);
	g_free(temp);
	g_free(query);
	query = query_aux;
	query_aux = NULL;
    }

    if (condition->repair)  {
	temp = g_strdup("true");
    } else {
	temp = g_strdup("false");
    }
    query_aux = g_strconcat(query, "(deReparacion = '", temp, "') AND ", NULL);
    g_free(temp);
    g_free(query);
    query = query_aux;
    query_aux = NULL;

    if (condition->invoice)  {
	temp = g_strdup("true");
    } else {
	temp = g_strdup("false");
    }
    query_aux = g_strconcat(query, "(deFacturacion = '", temp, "') ", NULL);
    g_free(temp);
    g_free(query);
    query = query_aux;
    query_aux = NULL;

    correct = m_model_command_execute (&record_set, query, connection, command,
				    tuples,0, TRUE);
    g_free(query);

    list=NULL;
    if (correct) {
	if (*tuples!=0) {
	    list = (Document **) g_malloc0(sizeof(Document*)*(*tuples+1));
	    index = gda_recordset_move_first(record_set);

	    while (!gda_recordset_eof(record_set)) {

		if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;

		list[index-1] = m_document_create (record_set);
		index = gda_recordset_move_next(record_set);
	    }
	    list[index] = NULL;
	} else {
	    list=NULL;
	}
    }

    gda_recordset_free(record_set);
    igalia_command_free(command);
    return list;
}


/**
 * m_document_clear_array: 
 * @list: Lista de documentos a liberar
 *
 * Libera la memoria asignada al array de documentos.
 *
 */
void m_document_clear_array (Document **list) {

    if (list!=NULL) {
	gint index = 0;
	while (list[index]!=NULL) {
	    d_document_clear (list[index]);
	    index++;
	}

	g_free(list);

	list = NULL;

    }
}

/**
 * m_document_clear_cached_array:
 *
 * Libera la memoria reservada para la variable global @documents_list.
 *
 */
void m_document_clear_cached_array () {

    if (documents_list!=NULL) {
	gint index = 0;
	while (documents_list[index]!=NULL) {
	    d_document_clear (documents_list[index]);
	    index++;
	}

	g_free(documents_list);

	documents_list = NULL;

    }
}

/**
 * m_document_insert:
 * @document: Documento a insertar en la base de datos
 *
 * Inserta un documento en la base de datos.
 *
 * Returns: %TRUE si se ha podido insertar el documento, %FALSE en otro caso.
 */
gboolean m_document_insert (Document *document) {
    gboolean result = FALSE;

    gchar         *gchar_query_document;
    gchar         *gchar_query_document_aux;
    GdaRecordset  *record_set = NULL;
    gchar         *temp;
    gulong        *tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaConnection *connection = m_model_get_connection ();
    GdaCommand    *command = gda_command_new();
    gulong logged_shop_code;
    gulong logged_enterprise_code;
    gulong logged_warehouse_code;
    gulong code;
    gchar * temp_name;

    temp_name = u_parameter_manager_get_parameter ("empresa");
    logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("tienda");
    logged_shop_code = m_shop_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("almacen");
    logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

    g_free(temp_name);

    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,logged_enterprise_code,logged_shop_code,logged_warehouse_code);

    gchar_query_document = "";
    gchar_query_document = g_strconcat("INSERT INTO ",m_document_get_table (),
				       " VALUES ",
				       "('", NULL);

    temp = m_model_nextval_sequence ("seqDocumento", connection);
    code = strtoul(temp,NULL,10);
    d_document_set_code (document,code);
    gchar_query_document_aux = g_strconcat(gchar_query_document, temp,
					   "', '", NULL);
    g_free(temp);
    g_free(gchar_query_document);
    gchar_query_document = gchar_query_document_aux;
    gchar_query_document_aux = NULL;

    temp = d_document_get_name (document);
    gchar_query_document_aux = g_strconcat(gchar_query_document, temp,
					   "', '",NULL);
    g_free(temp);
    g_free(gchar_query_document);
    gchar_query_document = gchar_query_document_aux;
    gchar_query_document_aux = NULL;

    temp = d_document_get_description (document);
    gchar_query_document_aux = g_strconcat(gchar_query_document, temp,
					   "', '",NULL);
    g_free(temp);
    g_free(gchar_query_document);
    gchar_query_document = gchar_query_document_aux;
    gchar_query_document_aux = NULL;

    if(d_document_get_from_invoice (document)) {
	temp = g_strdup("true");
    } else {
	temp = g_strdup("false");
    }

    gchar_query_document_aux = g_strconcat(gchar_query_document, temp,
					   "', '",NULL);
    g_free(temp);
    g_free(gchar_query_document);
    gchar_query_document = gchar_query_document_aux;
    gchar_query_document_aux = NULL;

    if(d_document_get_from_repair (document)) {
	temp = g_strdup("true");
    } else {
	temp = g_strdup("false");
    }

    gchar_query_document_aux = g_strconcat(gchar_query_document, temp,
					   "') ", NULL);
    g_free(temp);
    g_free(gchar_query_document);
    gchar_query_document = gchar_query_document_aux;
    gchar_query_document_aux = NULL;

    result = m_model_command_execute (&record_set, gchar_query_document, connection,command,tuples,0,FALSE);

    result = m_model_execute_mutex_modification_logged (connection) && result;

    g_free(gchar_query_document);
    g_free(tuples);

    gda_recordset_free(record_set);
    igalia_command_free(command);

    return result;
}

/**
 * m_document_update:
 * @document: Documento a modificar en la base de datos
 * 
 * Modifica un documento en la base de datos.
 *
 * Returns: %TRUE si se ha podido modificar el documento, %FALSE en otro caso.
 */
gboolean m_document_update (Document *document) {
    gboolean result = FALSE;

    gchar         *gchar_query_document;
    gchar         *gchar_query_document_aux;
    GdaRecordset  *record_set = NULL;
    gchar         *temp;
    gulong        *tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaConnection *connection = m_model_get_connection ();
    GdaCommand    *command = gda_command_new();
    gulong logged_shop_code;
    gulong logged_enterprise_code;
    gulong logged_warehouse_code;
    gchar * temp_name;

    temp_name = u_parameter_manager_get_parameter ("empresa");
    logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("tienda");
    logged_shop_code = m_shop_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("almacen");
    logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

    g_free(temp_name);

    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,logged_enterprise_code,logged_shop_code,logged_warehouse_code);

    gchar_query_document = g_strconcat("UPDATE ",m_document_get_table (),
				       " SET ", NULL);

    temp = d_document_get_name (document);
    gchar_query_document_aux = g_strconcat(gchar_query_document,
					   " nombredocumento = '",
					   temp,
					   "', ", NULL);
    g_free(temp);
    g_free(gchar_query_document);
    gchar_query_document = gchar_query_document_aux;
    gchar_query_document_aux = NULL;

    temp = d_document_get_description (document);
    gchar_query_document_aux = g_strconcat(gchar_query_document,
					   " descripciondocumento = '",
					   temp,
					   "', ", NULL);
    g_free(temp);
    g_free(gchar_query_document);
    gchar_query_document = gchar_query_document_aux;
    gchar_query_document_aux = NULL;

    if(d_document_get_from_invoice (document)) {
	temp = g_strdup("true");
    } else {
	temp = g_strdup("false");
    }

    gchar_query_document_aux = g_strconcat(gchar_query_document,
					   " deFacturacion = '",
					   temp,
					   "', ", NULL);
    g_free(temp);
    g_free(gchar_query_document);
    gchar_query_document = gchar_query_document_aux;
    gchar_query_document_aux = NULL;

    if(d_document_get_from_repair (document)) {
	temp = g_strdup("true");
    } else {
	temp = g_strdup("false");
    }

    gchar_query_document_aux = g_strconcat(gchar_query_document,
					   " deReparacion = '",
					   temp,
					   "' ", NULL);
    g_free(temp);
    g_free(gchar_query_document);
    gchar_query_document = gchar_query_document_aux;
    gchar_query_document_aux = NULL;

    temp = g_strdup_printf("%lu",d_document_get_code (document));
    gchar_query_document_aux = g_strconcat(gchar_query_document,
					   " WHERE codigoDocumento = '",temp,
					   "' ", NULL);
    g_free(temp);
    g_free(gchar_query_document);
    gchar_query_document = gchar_query_document_aux;
    gchar_query_document_aux = NULL;

    result = m_model_command_execute (&record_set, gchar_query_document, connection,command,tuples,0,FALSE);

    result = m_model_execute_mutex_modification_logged (connection) && result;

    g_print(gchar_query_document);
    g_free(gchar_query_document);
    g_free(tuples);

    gda_recordset_free(record_set);
    igalia_command_free(command);

    return result;
}

/**
 * m_document_delete: 
 * @document: Documento a borrar en la base de datos
 * 
 * Borra un documento en la base de datos.
 *
 * Returns: %TRUE si se ha podido borrar el documento, %FALSE en otro caso.
 */
gboolean m_document_delete (Document *document) {
    GdaConnection *connection = m_model_get_connection ();
    GdaCommand    *command = gda_command_new();
    GdaRecordset  *record_set = NULL;
    gulong *tuples = (gulong *) g_malloc0(sizeof(gulong));
    gulong  logged_shop_code;
    gulong  logged_enterprise_code;
    gulong  logged_warehouse_code;
    gchar  *temp_name;
    gchar  *query;
    gchar  *temp;
    gboolean result = FALSE;

    temp_name = u_parameter_manager_get_parameter ("empresa");
    logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("tienda");
    logged_shop_code = m_shop_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("almacen");
    logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

    g_free(temp_name);


    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,logged_enterprise_code,logged_shop_code,logged_warehouse_code);


    temp = g_strdup_printf("%lu",d_document_get_code (document));
    query = g_strconcat("DELETE FROM ", m_document_get_table ()," WHERE ",
			"codigoDocumento = '",temp,"'",
			NULL);
    g_free(temp);

    result = m_model_command_execute (&record_set, query, connection, command,tuples,0,FALSE);
    result = m_model_execute_mutex_modification_logged (connection) && result;

    g_free(query);
    g_free(tuples);

    gda_recordset_free(record_set);
    igalia_command_free(command);

    return result;
}

/**
 * m_document_delete_by_code:
 * @code: Cdigo del documento a borrar en la base de datos
 *
 * Borra un documento en la base de datos a partir del cdigo del documento.
 *
 * Returns: %TRUE si se ha podido borrar el documento, %FALSE en otro caso.
 */
gboolean m_document_delete_by_code (gulong code) {
    gboolean result = FALSE;
    Document *doc = d_document_create_empty ();

    d_document_set_code (doc, code);
    result = m_document_delete (doc);
    d_document_clear (doc);

    return result;
}

