/* expense_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "expense_model.h"
#include "participant_model.h"

static gint recordset_to_expenses_array(GdaRecordset *rs, Expense **expense_array);


#define TABLE         "FacturaCargo"
#define ATTRIBUTES    "codigoProveedor as codigoProveedorFacturaCargo"

#define D_SUM_ATTRIBUTES "to_char(sum(FacturaCargo.total),'99999999D99') as sumFacturaCargo"

#define D_ACCUMULATE_ATTRIBUTES "codigoProveedor as codigoProveedorFacturaCargo"




/**
 * m_expense_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_expense_get_table () {
    return TABLE;
}

/**
 * m_expense_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_expense_get_attributes () {
    return ATTRIBUTES;
}

/**
 * m_expense_get_sumexpense_attributes:
 *
 * Funcin que obtiene los atributos agrupados de los gastos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a con un alias.
 */
gchar *m_expense_get_sumexpense_attributes () {
    return D_SUM_ATTRIBUTES;
}

/**
 * m_expense_get_accumulative_attributes:
 *
 * Funcin que obtiene los atributos acumulativos de gastos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_expense_get_accumulative_attributes () {
    return D_ACCUMULATE_ATTRIBUTES;
}

/**
 * m_expense_get_not_accumulative_attributes:
 *
 * Funcin que devuelve una cadena vaca. No utilizada.
 *
 * Returns:
 * Cadena vaca
 */
gchar *m_expense_get_not_accumulative_attributes () {
    return "";
}

/**
 * m_expense_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 * El objeto #Expense se utiliza para listar las facturas de cargo agrupando
 * las lneas de factura de cargo por tipo de iva.
 *
 * Returns:
 * Un objeto #Expense con los campos rellenados con los datos de la base de datos.
 */
Expense * m_expense_create (GdaRecordset *rs) {

    GdaField * field;
    Expense * expense;

    expense = d_expense_create_empty();

    field = gda_recordset_field_name(rs,"sumFacturaCargo" );
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp=gda_stringify_value(NULL, 0, field);
	    expense->amount = atof(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }

    expense -> supplier = m_participant_create (rs, "");

    return expense;
}


/**
 * m_expense_find_by_condition:
 * @condition: Condicin de bsqueda. Permite seleccionar la empresa para la cual se quieren comprobar los
 * gastos y el perodo de bsqueda.
 * @tuples: Nmero de tuplas de gastos que cumplieron la condicin. Este dato ser rellenado
 * al realizar la sentencia sql.
 * 
 * Funcin utilizada para generar la parte de gastos del libro de IVA, que permite listar, por empresa,
 * los gastos de un perodo.
 *
 * Returns:
 * Array con los gastos del empresa.
 */
Expense ** m_expense_find_by_condition (ConditionExpense *condition, gulong * tuples) {

    gint           in;
    gchar         *query_select     = NULL;
    gchar         *query_attributes = NULL;
    gchar         *query_from       = NULL;
    gchar         *query_where      = NULL;
    gchar         *query_extra      = NULL;
    gchar         *gchar_having      = NULL;
    gchar         *temp = NULL;
    gchar         *aux  = NULL;
    GdaRecordset  *record_set = NULL;
    GdaCommand    *command = gda_command_new();
    GdaConnection *connection = m_model_get_connection ();
    Expense      **expense_array;
    gboolean       correct = FALSE;

    gda_command_set_connection(command, connection);

    temp = u_table_put_name (m_participant_get_attributes (), m_participant_get_table ());
    aux = g_strconcat(temp, " ", NULL);
    g_free(temp);
    query_attributes = aux;
    aux = NULL;

    temp = u_table_put_name (m_expense_get_attributes (), m_expense_get_table ());
    aux = g_strconcat(query_attributes, ", ", temp, " ", NULL);
    g_free(temp);
    g_free(query_attributes);
    query_attributes = aux;
    aux = NULL;

    aux = g_strconcat(" FROM ", m_expense_get_table ()," left outer join ",
		      m_participant_get_table (), " on (",
		      m_participant_get_table (), ".codigoParty=",
                      m_expense_get_table (),".codigoProveedor)",
		      NULL);
    query_from = aux;
    aux = NULL;

    aux = g_strconcat(" WHERE (", m_expense_get_table (),".fecha >= '",
		      ctime(&(condition->initial_date)), "') ", NULL);
    query_where = aux;
    aux = NULL;

    aux = g_strconcat(query_where, "AND (", m_expense_get_table (),
		      ".fecha <= '", ctime(&(condition->final_date)),
		      "') ", NULL);
    g_free(query_where);
    query_where = aux;
    aux = NULL;

    temp = g_strdup_printf("%lu",condition->enterprise_code);
    aux = g_strconcat(query_where, "AND (", m_expense_get_table (),
		      ".codigoEmpresa = '",temp, "') ", NULL);
    g_free(query_where);
    g_free(temp);
    query_where = aux;
    aux = NULL;

    temp = u_remove_aliases (query_attributes, "as", ",");
    aux = g_strconcat(" GROUP BY ", temp, NULL);
    g_free(temp);
    query_extra = aux;
    aux = NULL;

    aux = g_strconcat(query_attributes, ", ",
		      m_expense_get_sumexpense_attributes (), NULL);
    g_free(query_attributes);
    query_attributes = aux;
    aux = NULL;

    if (condition->greater_than!=FALSE) {
	gchar_having = " having SUM(FacturaCargo.total)>=CAST(3005.6 as DECIMAL(12,2))";
    } else {
	gchar_having = " ";
    }
    aux = g_strconcat("SELECT ", query_attributes, query_from,
		      query_where, query_extra, gchar_having,NULL);
    query_select = aux;
    aux = NULL;


    g_print(_("\n Expense find by condition: %s\n"), query_select);
    correct = m_model_command_execute (&record_set, query_select, connection,
				    command,tuples,0, TRUE);

    expense_array=NULL;
    if (correct) {
	if (*tuples!=0) {
	    expense_array = (Expense **) g_malloc ((sizeof(Expense *))*(*tuples +1));
	    in = recordset_to_expenses_array(record_set, expense_array);
	}
    }

    gda_recordset_free(record_set);
    igalia_command_free(command);
    g_free(query_select);
    g_free(query_from);
    g_free(query_where);
    g_free(query_extra);

    return expense_array;

}

/**
 * m_expense_clear_array:
 * @expense_array:Array de gastos.
 * 
 * Libera la memoria utilizada por un array de objetos Expense.
 *
 * Returns: void
 */
gint m_expense_clear_array (Expense ** expense_array) {
    gulong index = 0;

    if (expense_array!=NULL) {
	while(expense_array[index]!=NULL) {
	    d_expense_clear (expense_array[index]);
            expense_array[index] = NULL;
	    index++;
	}
    }
    g_free(expense_array);
    expense_array=NULL;
    return 0;
}

/**
 * recordset_to_expenses_array: 
 * @rs: El record set que se quiere transformar.
 * @expense_array: Array de gastos a rellenar. Es necesario reservar memoria previamente.
 *
 *  Transforma el recordset con las tuplas de gastos en un array de objetos
 * Expense.
 *
 * Returns:
 * El ndice del ltimo elemento del array de gastos, o lo que es lo mismo, el tamao del array.
 */
static gint recordset_to_expenses_array(GdaRecordset *rs,
					Expense **expense_array) {
    gint index;
    index = gda_recordset_move_first(rs);

    while (!gda_recordset_eof(rs)) {

	if (index==GDA_RECORDSET_INVALID_POSITION) return -1;

	expense_array[index-1] = m_expense_create (rs);

	index = gda_recordset_move_next(rs);
    }
    expense_array[index] = NULL;

    return index;
}

