
/* garage_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "garage_model.h"
#include "participant_model.h"

#include "util.h"


#define TABLE         "Taller"
#define IDENTIFICATOR "codigoParty"

/**
 * m_garage_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_garage_get_table () {
    return TABLE;
}


/**
 * m_garage_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Garage con los campos rellenados con los datos de la base de datos.
 */
Garage *m_garage_create (GdaRecordset *rs) {
    GdaField * field;
    Garage * garage =d_garage_create_empty();

    garage->participant = m_participant_create (rs,"garage");

    if (d_participant_get_code (d_garage_get_participant (garage)) != ULONG_MAX) {
       field = gda_recordset_field_name(rs,"comisiongarage");
       if (field!=NULL) {
	   if (!gda_field_is_null(field)) {
	       garage->commission = igalia_field_get_double_value (field);
	   }
	   gda_field_free(field);
       }
    }


    return garage;
}


/**
 * m_garage_find_by_name:
 * @name: Nombre por el que se desea buscar.
 * 
 * Busca los talleres cuyo nombre coincida con el pasado por parmetro
 *
 * Returns:
 * Lista de talleres que cumplan la condicin de bsqueda
 */
Garage ** m_garage_find_by_name (gchar *name) {

    Garage ** list_of_garages = NULL;
    GdaCommand         * my_command;
    GdaConnection      * my_connection = m_model_get_connection ();
    GdaRecordset       * record_set = NULL;
    Garage * garage;
    gchar * gchar_query = NULL;
    gulong * tuples;
    gint * column;
    gint index;
    gint length_of_list;
    gboolean correct = FALSE;

    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);

    gchar_query = g_strconcat("SELECT ",
			      " Participante.codigoparty as codigopartyparticipantgarage, ",
			      " Participante.cif as cifparticipantgarage, ",
			      " Participante.nick as nickparticipantgarage, ",
			      " Participante.nombre as nombreparticipantgarage, ",
			      " Participante.nombreCorto as shortNameparticipantgarage, ",
			      " Taller.comision as comisiongarage ",
			      " FROM participante",
			      " WHERE participante.taller=true",
			      " AND taller.codigoparty = participante.codigoparty",
			      NULL);

    if (name != NULL){
	gchar * gchar_query_aux = NULL;

	gchar_query_aux = g_strconcat(gchar_query," AND participante.nick ILIKE '",name,"%'",NULL);

	g_free(gchar_query);
	gchar_query =  gchar_query_aux;
	gchar_query_aux = NULL;
    }

    tuples = (gulong *) g_malloc0(sizeof(gulong));
    correct = m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);
    g_print(gchar_query);
    g_free(gchar_query);

    length_of_list = *tuples;
    column = (gint *) g_malloc0(sizeof(gint));

    list_of_garages=NULL;
    if (correct) {
	if (*tuples!=0) {
	    list_of_garages = (Garage **) g_malloc0(sizeof(Garage*)*(*tuples+1));
	    index = gda_recordset_move_first(record_set);
	    while (!gda_recordset_eof(record_set)) {
		if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		(*column)=0;
		garage = m_garage_create (record_set);
		list_of_garages[index-1] = garage;
		index = gda_recordset_move_next(record_set);
	    }
	    list_of_garages[length_of_list]=NULL;

	}
    }

    igalia_command_free(my_command);
    gda_recordset_free(record_set);

    g_free(tuples);
    g_free(column);

    return list_of_garages;
}

/**
 * m_garage_clear_array: 
 * @list_of_garages: Lista de talleres.
 * 
 * Libera la memoria asignada a un array de talleres.
 *
 */
void m_garage_clear_array (Garage **list_of_garages) {

    if (list_of_garages!=NULL) {
	gint index = 0;
	while (list_of_garages[index]!=NULL) {
	    d_garage_clear (list_of_garages[index]);
	    index++;
	}
	g_free(list_of_garages);
    }

    list_of_garages = NULL;

}
