
/* manodeobra_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "manodeobra_model.h"
#include "product_model.h"
#include "enterprise_model.h"
#include "shop_model.h"
#include "warehouse_model.h"
#include "parameter_manager.h"

#include "util.h"

#define TABLE         "ManoDeObra"
#define IDENTIFICATOR "codigoManoDeObra"
#define ATTRIBUTES    "codigoManoDeObra as codigomanodeobramanodeobra "

/**
 * m_man_power_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_man_power_get_table () {
    return TABLE;
}

/**
 * m_man_power_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_man_power_get_attributes () {
    return ATTRIBUTES;
}

/**
 * d_man_power:create_query_find
 * @razon_social: La razn social por la que se quiere buscar.
 *
 * Funcin que construye una query para buscar una mano de obra por razn social.
 *
 * Returns:
 * La query creada.
 */
gchar *m_man_power_create_query_find (gchar *razon_social) {

    return g_strconcat("SELECT ",IDENTIFICATOR," FROM ", TABLE,
		       " WHERE razonSocial=",razon_social,"\0",NULL);
}



/**
 * m_man_power_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #ManPower con los campos rellenados con los datos de la base de datos.
 */
ManPower *m_man_power_create (GdaRecordset *rs) {
    GdaField * field;
    ManPower *man_power = d_man_power_create_empty();
    Product product;

    product = m_product_create (rs);
    d_product_set_is_man_power (&product, TRUE);

    d_man_power_set_product (man_power, product);

    if (d_product_get_product_code (&product)!=-1) {
	field = gda_recordset_field_name(rs,"codigomanodeobramanodeobra");
	if (field!=NULL) {
	    if (!gda_field_is_null(field)) {
		d_man_power_set_code (man_power, igalia_field_get_bigint_value (field));
	    }
	    gda_field_free(field);
	}
    }

    return man_power;
}

/**
 * m_man_power_find_by_description:
 * @man_power_description: Descripcin por la que se desea buscar.
 * @company_code: Cdigo de compaa por la que se busca la tasacin.
 *
 *  Busca la mano de obra que se corresponde con la descripcin, tasada
 * para la compaa de seguros que se pasa como parmetro.
 *
 * Returns:
 * Lista de manos de obra que cumplen la condicin.
 */
ManPower ** m_man_power_find_by_description (gchar *man_power_description, gchar *company_code) {

    ManPower ** man_power_list = NULL;
    GdaCommand    * my_command;
    GdaConnection * my_connection = m_model_get_connection ();
    GdaRecordset  * record_set = NULL;
    ManPower    * man_power;
    gchar         * gchar_query = NULL;
    gulong        * tuples;
    gint          * column;
    gint            index;
    gint            length_of_list;
    gboolean        correct = FALSE;

    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);

    gchar_query = g_strconcat("SELECT ",
			      " producto.codigoproducto as codigoproductoproduct, ",
			      " producto.codigofamilia as codigofamiliaproduct, ",
			      " producto.descripcion as descripcionproduct, ",
			      " to_char(producto.tipoiva,'999D99') as tipoivaproduct, ",
			      " manodeobra.codigomanodeobra as codigomanodeobramanodeobra,",
			      NULL);

    if (company_code != NULL) {
	gchar * gchar_query_aux = NULL;

	gchar_query_aux = g_strconcat(gchar_query," tasacion.precio as precioproduct  ",
				      " FROM producto, manodeobra left outer join tasacion on (manodeobra.codigomanodeobra = tasacion.codigomanodeobra",
				      " AND tasacion.codigoaseguradora = '",company_code,"' ) ",
				      " WHERE producto.codigoproducto = manodeobra.codigoproducto"
				      ,NULL);
	g_free(gchar_query);
	gchar_query =  gchar_query_aux;
	gchar_query_aux = NULL;
    } else {
	gchar * gchar_query_aux = NULL;

	gchar_query_aux = g_strconcat(gchar_query," producto.precio as precioproduct ",
				      " FROM producto, manodeobra ",
				      " WHERE producto.codigoproducto = manodeobra.codigoproducto"
				      ,NULL);
	g_free(gchar_query);
	gchar_query =  gchar_query_aux;
	gchar_query_aux = NULL;
    }

    if (man_power_description != NULL){
	gchar * gchar_query_aux = NULL;

	gchar_query_aux = g_strconcat(gchar_query," AND producto.descripcion ILIKE '%",man_power_description,"%'",NULL);

	g_free(gchar_query);
	gchar_query =  gchar_query_aux;
	gchar_query_aux = NULL;
    }

    tuples = (gulong *) g_malloc0(sizeof(gulong));
    correct = m_model_command_execute (&record_set,gchar_query, my_connection,
			  my_command,tuples,0,TRUE);
    g_print(gchar_query);
    g_free(gchar_query);

    length_of_list = *tuples;
    column = (gint *) g_malloc0(sizeof(gint));

    man_power_list=NULL;
    if (correct) {
	if (*tuples!=0) {
	    man_power_list = (ManPower **) g_malloc0(sizeof(ManPower*)*(*tuples+1));
	    index = gda_recordset_move_first(record_set);
	    while (!gda_recordset_eof(record_set)) {
		if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		(*column)=0;
		man_power = m_man_power_create (record_set);
		man_power_list[index-1] = man_power;
		index = gda_recordset_move_next(record_set);
	    }
	    man_power_list[length_of_list]=NULL;

	}
    }

    gda_recordset_free(record_set);
    igalia_command_free(my_command);

    g_free(tuples);
    g_free(column);


    return man_power_list;
}

/**
 * m_man_power_clear_array:
 * @man_power_list: Lista de objetos mano de obra a liberar.
 *
 *  Libera la memoria utilizada por el array de manos de obra.
 *
 */
void m_man_power_clear_array (ManPower **man_power_list) {

    if (man_power_list!=NULL) {
	gint index = 0;
	while (man_power_list[index]!=NULL) {
	    d_man_power_clear (man_power_list[index]);
	    index++;
	}

	g_free(man_power_list);
    }

    man_power_list = NULL;

}

/**
 * m_man_power_is_valued:
 * @man_power: Cdigo de la mano de obra a comprobar.
 * @company: Compaa de seguros para la que se quiere comprobar la tasacin.
 *
 *  Comprueba si una pieza est tasada para una compaa en la base de datos .
 *
 * Returns:
 * %TRUE si est tasada, %FALSE en otro caso.
 */
gboolean m_man_power_is_valued   (gchar *man_power, gchar* company) {

    gboolean result = TRUE;
    GdaCommand *command = gda_command_new();
    gchar *gchar_query = NULL;

    GdaConnection *my_connection = m_model_get_connection ();
    GdaRecordset *record_set = NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));

    gda_command_set_connection(command,my_connection);

    gchar_query = g_strconcat("SELECT precio FROM tasacion",
			      " WHERE codigomanodeobra = ", man_power,
			      " AND codigoaseguradora  = ", company,
			      NULL);

    g_print(_("\nQuery: %s\n"),gchar_query);

    m_model_command_execute (&record_set, gchar_query, my_connection,
			  command,tuples,0,TRUE);

    result = (*tuples);

    igalia_command_free(command);

    gda_recordset_free(record_set);

    g_free(gchar_query);
    g_free(tuples);

    return result;


}

/**
 * m_man_power_update_price:
 * @man_power: Cdigo de la mano de obra que se desea tasar.
 * @company: Compaa de seguros para la que se modificar el precio de la mano de obra.
 *
 * Modifica el precio de una mano de obra para una compaia de seguros.
 *
 * Returns:
 * %TRUE si se ha modificado, %FALSE en otro caso.
 */
gboolean m_man_power_update_price (ManPower *man_power, Company *company) {

    gboolean result = FALSE;

    if (company != NULL) {

	GdaCommand *command = gda_command_new();
	GdaConnection *connection = m_model_get_connection ();
	GdaRecordset *record_set = NULL;
	gchar *query = NULL;
	gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
	gulong man_power_code_gulong = d_man_power_get_code (man_power);
	gchar *man_power_code = NULL;
	gulong company_code_gulong = d_participant_get_code (d_company_get_participant (company));
	gchar *company_code = NULL;
	gdouble price_gulong = d_product_get_price ((Product *)man_power);
	gchar *price = NULL;
	gboolean update = FALSE;
	gulong logged_shop_code;
	gulong logged_enterprise_code;
	gulong logged_warehouse_code;
	gchar * temp_name;

	temp_name = u_parameter_manager_get_parameter ("empresa");
	logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
	g_free(temp_name);
	temp_name = u_parameter_manager_get_parameter ("tienda");
	logged_shop_code = m_shop_get_code_from_name (temp_name);
	g_free(temp_name);
	temp_name = u_parameter_manager_get_parameter ("almacen");
	logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

	g_free(temp_name);

	gda_command_set_connection(command, connection);

	if (man_power_code_gulong!=ULONG_MAX)
	    man_power_code = g_strdup_printf("%lu",man_power_code_gulong);
	else man_power_code = g_strdup("NULL");

	if (company_code_gulong!=ULONG_MAX)
	    company_code = g_strdup_printf("%lu",company_code_gulong);
	else company_code = g_strdup("NULL");

	if (price_gulong!=G_MINDOUBLE) {
	    price = g_strdup_printf("%.2f",price_gulong);
	    u_commas_to_points (price);
	}
	else price = g_strdup("NULL");

	update = m_man_power_is_valued   (man_power_code, company_code);

	if (update) {
	    query = g_strconcat("UPDATE tasacion SET ",
				" precio = ",price,
				" WHERE codigoaseguradora = ", company_code,
				" AND codigomanodeobra = ", man_power_code,
				NULL);

	} else {
	    query = g_strconcat("INSERT INTO tasacion VALUES (",
				company_code,",",
				man_power_code,",",
				price,
				")",
				NULL);
	}

	result = m_model_mutex_transactional_command_execute_logged (&record_set,
							   query,
							   connection,
							   command, tuples,
							   0, FALSE,
                                                           logged_enterprise_code,logged_shop_code,logged_warehouse_code
							  );

	g_print(query);
	g_free(query);

	gda_recordset_free(record_set);
        igalia_command_free(command);

	g_free(tuples);
	g_free(man_power_code);
	g_free(company_code);
	g_free(price);
    }

    return result;
}


