/* order_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "gda_igalia_util.h"
#include "order_line_model.h"
#include "order_model.h"
#include "util.h"

#define TABLE         "Pedido"
#define ATTRIBUTES    "codigoPedido as codigopedidoorder, \
    codigoEmpresa as codigoempresaorder, \
    codigoAlmacen as codigoalmacenorder, \
    codigoProveedor as codigoproveedororder, \
    solicitado      as solicitadoorder, \
    fecha as fechaorder "

/**
 * m_order_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_order_get_table () {
    return TABLE;
}

/**
 * m_order_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_order_get_attributes () {
    return ATTRIBUTES;
}


/**
 * m_order_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Order con los campos rellenados con los datos de la base de datos.
 */
Order  *m_order_create (GdaRecordset *rs){
    GdaField    *field;
    Order *order = d_order_create_empty();

    field = gda_recordset_field_name(rs,"codigopedidoorder");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    order->order_code = igalia_field_get_bigint_value (field);
	    gda_field_free(field);

	    field = gda_recordset_field_name(rs,"codigoempresaorder");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    order->enterprise_code = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"codigoalmacenorder");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    order->warehouse_code = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"codigoproveedororder");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    order->supplier_code = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"solicitadoorder");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    order->requested = gda_field_get_boolean_value(field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"fechaorder");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp;
		    time_t * my_time;
		    temp=gda_stringify_value(NULL, 0, field);
		    my_time = u_parse_date (temp);
		    order->date = (*my_time);
		    g_free(my_time);

		    my_time = NULL;
		    g_free(temp);

		}
		gda_field_free(field);
	    }
	} else {
	    gda_field_free(field);
	}
    }
    return order;
}

/**
 * m_order_find_by_condition:
 * @condition: Filtros de bsqueda. Permite buscar por cdigo de la empresa realizadora del pedido,
 * cdigo del almacn destino del pedido, codigo de proveedor al que realizar el pedido, cdigo de
 * pedido, si ya est solicitado y la fecha del pedido.
 * @tuples: Nmero de tuplas de la base de datos que sern devueltas, es necesario reserver memoria
 * previamente.
 *
 *  Busca los pedidos que cumplan una serie de condiciones.
 *
 * Returns:
 * Array con los pedidos que cumplen la condicin de bsqueda.
 */
Order **m_order_find_by_condition (ConditionFindOrder *condition, gulong *tuples) {
    GdaCommand    *command;
    GdaConnection *connection = m_model_get_connection ();
    GdaRecordset  *record_set = NULL;
    gchar         *temp;
    gchar         *query;
    gchar         *query_aux;
    gboolean       correct;
    Order        **list=NULL;
    gint           index = -1;

    command = gda_command_new();
    gda_command_set_connection(command, connection);
    query = g_strconcat("SELECT ", m_order_get_attributes (),
			" FROM ",  m_order_get_table (),
			" WHERE ", NULL);

    if ((condition->order_code!=0) && (condition->order_code!=ULONG_MAX)) {
	temp = g_strdup_printf("%lu",condition->order_code);
	query_aux = g_strconcat(query, "(codigoPedido = ", temp, ") AND ", NULL);
	g_free(temp);
	g_free(query);
	query = query_aux;
	query_aux = NULL;
    }

    if ((condition->enterprise_code!=0) && (condition->enterprise_code!=ULONG_MAX)) {
	temp = g_strdup_printf("%lu",condition->enterprise_code);
	query_aux = g_strconcat(query, "(codigoEmpresa = ", temp, ") AND ", NULL);
	g_free(temp);
	g_free(query);
	query = query_aux;
	query_aux = NULL;
    }

    if ((condition->warehouse_code!=0) && (condition->warehouse_code!=ULONG_MAX)) {
	temp = g_strdup_printf("%lu",condition->warehouse_code);
	query_aux = g_strconcat(query, "(codigoAlmacen = ", temp, ") AND ", NULL);
	g_free(temp);
	g_free(query);
	query = query_aux;
	query_aux = NULL;
    }

    if ((condition->supplier_code!=0) && (condition->supplier_code!=ULONG_MAX)) {
	temp = g_strdup_printf("%lu",condition->supplier_code);
	query_aux = g_strconcat(query, "(codigoProveedor = ", temp, ") AND ", NULL);
	g_free(temp);
	g_free(query);
	query = query_aux;
	query_aux = NULL;
    }

    if (condition->placed) {
	query_aux = g_strconcat(query, "(solicitado = true) AND ", NULL);
	g_free(query);
	query = query_aux;
	query_aux = NULL;
    } else {
	query_aux = g_strconcat(query, "(solicitado = false) AND ", NULL);
	g_free(query);
	query = query_aux;
	query_aux = NULL;
    }

    query_aux = g_strconcat(query, "(fecha >= '",ctime(&(condition->date)),
			    "') ", NULL);
    g_free(query);
    query = query_aux;
    query_aux = NULL;


    correct = m_model_command_execute (&record_set, query, connection, command,
				    tuples,0, TRUE);
    g_free(query);

    list=NULL;
    if (correct) {
	if (*tuples!=0) {
	    list = (Order **) g_malloc0(sizeof(Order*)*(*tuples+1));
	    index = gda_recordset_move_first(record_set);

	    while (!gda_recordset_eof(record_set)) {

		if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;

		list[index-1] = m_order_create (record_set);
		d_order_set_order_lines (list[index-1],
				  m_order_line_get_from_order (list[index-1]));
		index = gda_recordset_move_next(record_set);
	    }
	    list[index] = NULL;
	} else {
	    list=NULL;
	}
    }

    gda_recordset_free(record_set);
    igalia_command_free(command);
    return list;
}

/* We don't need this (yet)
static gint recordset_to_orders_array(GdaRecordset *rs, Order **order_array) {
    gint index;
    index = gda_recordset_move_first(rs);

    while (!gda_recordset_eof(rs)) {

	if (index==GDA_RECORDSET_INVALID_POSITION) return -1;

	order_array[index-1] = m_order_create (rs);
	index = gda_recordset_move_next(rs);
    }
    order_array[index] = NULL;

    return index;
}

*/

/**
 * m_order_clear_array:
 * @orders: Array a liberar.
 *
 *  Libera la memoria utilizada por el array de pedidos.
 *
 */
void m_order_clear_array (Order ** orders) {
    if (orders!=NULL) {
	gint index = 0;
	while (orders[index]!=NULL) {
	    d_order_clear (orders[index]);
	    orders[index] = NULL;
	    index++;
	}
	g_free(orders);
	orders = NULL;
    }
}

/**
 * model_find_orders_by_condition_and_delivery_number - 
 * @condition: Filtros de bsqueda. Permite buscar por cdigo de la empresa realizadora del pedido,
 * cdigo del almacn destino del pedido, codigo de proveedor al que realizar el pedido, cdigo de
 * pedido, si ya est solicitado y la fecha del pedido.
 * @charge_delivery_payment_number: Nmero de albarn por el que se desea buscar.
 * @tuples: Nmero de tuplas de la base de datos que sern devueltas, es necesario reserver memoria
 * previamente.
 *
 * Busca pedidos por una serie de condiciones de bsqueda
 * y por nmero de albarn asociado.
 *
 * Returns:
 * Lista de pedidos que cumplen la condicin de bsqueda y tienen alguna lnea de pedido asociada al
 * albarn con nmero @charge_delivery_payment_number.
 */

Order **m_order_find_by_condition_and_delivery_payment (ConditionFindOrder *condition,
							   gchar * charge_delivery_payment_number, gulong *tuples) {

    GdaCommand    *command;
    GdaConnection *connection = m_model_get_connection ();
    GdaRecordset  *record_set = NULL;
    gchar         *temp;
    gchar         *query;
    gchar         *orderby;
    gchar         *groupby;
    gchar         *query_aux=NULL;
    gboolean       correct;
    Order        **list=NULL;
    gint           index = -1;

    command = gda_command_new();
    gda_command_set_connection(command, connection);
    query = g_strconcat("SELECT ",
			" pedido.codigoPedido as codigoPedidoOrder, ",
			" pedido.codigoEmpresa as codigoEmpresaOrder, ",
			" pedido.codigoAlmacen as codigoAlmacenOrder, ",
			" pedido.codigoProveedor as codigoProveedorOrder, ",
			" pedido.fecha as fechaOrder, ",
			" pedido.solicitado as solicitadoOrder",
			" FROM pedido,lineaPedido ",
			" WHERE pedido.codigoPedido=lineaPedido.codigoPedido and pedido.codigoAlmacen=lineaPedido.codigoAlmacen "
			," and pedido.codigoEmpresa=lineaPEdido.codigoEmpresa ", NULL);

    groupby = g_strdup(" group by pedido.codigoPEdido,pedido.codigoEMpresa,pedido.codigoAlmacen,pedido.codigoProveedor,pedido.fecha,pedido.solicitado ");

    orderby = g_strdup(" order by pedido.codigoPedido");
    if ((condition->enterprise_code!=0) && (condition->enterprise_code!=ULONG_MAX)) {
	temp = g_strdup_printf("%lu",condition->enterprise_code);
	query_aux = g_strconcat(query, " AND (pedido.codigoEmpresa = ", temp, ") ", NULL);
	g_free(temp);
	g_free(query);
	query = query_aux;
	query_aux = NULL;
    }

    if ((condition->warehouse_code!=0) && (condition->warehouse_code!=ULONG_MAX)) {
	temp = g_strdup_printf("%lu",condition->warehouse_code);
	query_aux = g_strconcat(query, " AND (pedido.codigoAlmacen = ", temp, ") ", NULL);
	g_free(temp);
	g_free(query);
	query = query_aux;
	query_aux = NULL;
    }

    if ((condition->supplier_code!=0) && (condition->supplier_code!=ULONG_MAX)) {
	temp = g_strdup_printf("%lu",condition->supplier_code);
	query_aux = g_strconcat(query, " AND (pedido.codigoProveedor = ", temp, ") ", NULL);
	g_free(temp);
	g_free(query);
	query = query_aux;
	query_aux = NULL;
    }

    temp = g_strdup(charge_delivery_payment_number);
    query_aux = g_strconcat(query, "AND (lineapedido.numeroAlbaran = '", temp, "') ", NULL);
    g_free(temp);
    g_free(query);
    query = query_aux;
    query_aux = NULL;

    if (query_aux!=NULL) {
	g_free(query);
	query = query_aux;
	query_aux = NULL;
    }

    query_aux = g_strconcat(query,groupby,orderby,NULL);
    g_free(query);
    query = query_aux;
    query_aux = NULL;

    g_print(_("\nQuery: %s\n"),query);
    correct = m_model_command_execute (&record_set, query, connection, command,
				    tuples,0, TRUE);
    g_free(query);
    g_free(orderby);
    g_free(groupby);

    list=NULL;
    if (correct) {
	if (*tuples!=0) {
	    list = (Order **) g_malloc0(sizeof(Order*)*(*tuples+1));
	    index = gda_recordset_move_first(record_set);

	    while (!gda_recordset_eof(record_set)) {

		if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;

		list[index-1] = m_order_create (record_set);
		d_order_set_order_lines (list[index-1],
				  m_order_line_get_from_order (list[index-1]));
		index = gda_recordset_move_next(record_set);
	    }
	    list[index] = NULL;
	} else {
	    list=NULL;
	}
    }

    gda_recordset_free(record_set);
    igalia_command_free(command);
    return list;

}



/**
 * m_order_insert:
 * @order: Pedido a insertar en la base de datos.
 *
 * Inserta un pedido en la base de datos.
 *
 * Returns:
 * %TRUE si se ha insertado correctamente, %FALSE en otro caso.
 */
gboolean m_order_insert (Order *order) {
    gboolean result = FALSE;

    gchar         *gchar_query_order;
    gchar         *gchar_query_order_aux;
    GdaRecordset  *record_set = NULL;
    gchar         *temp;
    gulong        *tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaConnection *connection = m_model_get_connection ();
    GdaCommand    *command = gda_command_new();
    time_t         date = d_order_get_date (order);
    gulong enterprise_code_order = ULONG_MAX;
    gulong warehouse_code_order = ULONG_MAX;

    if (order!=NULL) {
	enterprise_code_order = d_order_get_enterprise_code (order);
    }
    if (order!=NULL) {
        warehouse_code_order = d_order_get_warehouse_code (order);
    }

    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,enterprise_code_order,ULONG_MAX,warehouse_code_order);

    gchar_query_order = "";
    gchar_query_order = g_strconcat("INSERT INTO ",m_order_get_table (),
				    " VALUES ",
				    "(", NULL);

    temp = m_model_nextval_sequence ("seqPedido", connection);
    gchar_query_order_aux = g_strconcat(gchar_query_order,
					temp, ", '",
					NULL);
    g_free(gchar_query_order);
    g_free(temp);
    gchar_query_order = gchar_query_order_aux;
    gchar_query_order_aux = NULL;

    temp = g_strdup_printf("%lu",d_order_get_enterprise_code (order));
    gchar_query_order_aux = g_strconcat(gchar_query_order, temp,
					"', '", NULL);
    g_free(temp);
    g_free(gchar_query_order);
    gchar_query_order = gchar_query_order_aux;
    gchar_query_order_aux = NULL;

    temp = g_strdup_printf("%lu",d_order_get_warehouse_code (order));
    gchar_query_order_aux = g_strconcat(gchar_query_order, temp,
					"', '", NULL);
    g_free(temp);
    g_free(gchar_query_order);
    gchar_query_order = gchar_query_order_aux;
    gchar_query_order_aux = NULL;

    temp = g_strdup_printf("%lu",d_order_get_supplier_code (order));
    gchar_query_order_aux = g_strconcat(gchar_query_order, temp,
					"', '", NULL);
    g_free(temp);
    g_free(gchar_query_order);
    gchar_query_order = gchar_query_order_aux;
    gchar_query_order_aux = NULL;

    gchar_query_order_aux = g_strconcat(gchar_query_order, ctime(&date),
					"', '", NULL);
    g_free(gchar_query_order);
    gchar_query_order = gchar_query_order_aux;
    gchar_query_order_aux = NULL;

    if (d_order_get_requested (order)) {
	gchar_query_order_aux = g_strconcat(gchar_query_order,
					    "true') ", NULL);
    } else {
	gchar_query_order_aux = g_strconcat(gchar_query_order,
					    "false') ", NULL);
    }
    g_free(gchar_query_order);
    gchar_query_order = gchar_query_order_aux;
    gchar_query_order_aux = NULL;

    result = m_model_command_execute (&record_set, gchar_query_order, connection, command,tuples,0, FALSE);

    result = result && m_order_line_insert (d_order_get_order_lines (order), connection, command);

    result = m_model_execute_mutex_modification_logged (connection) && result;

    g_free(gchar_query_order);
    g_free(tuples);

    gda_recordset_free(record_set);
    igalia_command_free(command);

    return result;
}

/**
 * m_order_update: 
 * @order: Pedido que se modificar en la base de datos.
 *
 * Modifica un pedido en la base de datos. Esta funcin incluye el borrado de todas las lneas del pedido
 * (incluidas la que ya no estn en el pedido) y la insercin de las nuevas en la base de datos.
 *
 * Returns:
 * %TRUE si la operacin se ha realizado correctamente, %FALSE en otro caso.
 */
gboolean m_order_update (Order *order) {
    gboolean result = FALSE;

    gchar         *gchar_query_order;
    gchar         *gchar_query_order_aux;
    GdaRecordset  *record_set = NULL;
    gchar         *temp;
    gulong        *tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaConnection *connection = m_model_get_connection ();
    GdaCommand    *command = gda_command_new();
    OrderLine   **lines;
    gint           index = 0;
    gulong enterprise_code_order = ULONG_MAX;
    gulong warehouse_code_order = ULONG_MAX;

    if (order!=NULL) {
	enterprise_code_order = d_order_get_enterprise_code (order);
    }
    if (order!=NULL) {
        warehouse_code_order = d_order_get_warehouse_code (order);
    }

    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,enterprise_code_order,ULONG_MAX,warehouse_code_order);

    gchar_query_order = "";
    gchar_query_order = g_strconcat("UPDATE ",m_order_get_table (),
				    " SET ", NULL);

    temp = g_strdup_printf("%lu",d_order_get_supplier_code (order));
    gchar_query_order_aux = g_strconcat(gchar_query_order,
					" codigoProveedor= '",temp,
					"', ", NULL);
    g_free(temp);
    g_free(gchar_query_order);
    gchar_query_order = gchar_query_order_aux;
    gchar_query_order_aux = NULL;

    if (d_order_get_requested (order)) {
	gchar_query_order_aux = g_strconcat(gchar_query_order,
					    "solicitado = 'true' ", NULL);
    } else {
	gchar_query_order_aux = g_strconcat(gchar_query_order,
					    "solicitado = 'false' ", NULL);
    }
    g_free(gchar_query_order);
    gchar_query_order = gchar_query_order_aux;
    gchar_query_order_aux = NULL;

    gchar_query_order_aux = g_strconcat(gchar_query_order,
					" WHERE ", NULL);
    g_free(gchar_query_order);
    gchar_query_order = gchar_query_order_aux;
    gchar_query_order_aux = NULL;

    temp = g_strdup_printf("%lu",d_order_get_code (order));
    gchar_query_order_aux = g_strconcat(gchar_query_order,
					"codigoPedido = ", temp,
					" AND ", NULL);
    g_free(temp);
    g_free(gchar_query_order);
    gchar_query_order = gchar_query_order_aux;
    gchar_query_order_aux = NULL;

    temp = g_strdup_printf("%lu",d_order_get_enterprise_code (order));
    gchar_query_order_aux = g_strconcat(gchar_query_order,
					"codigoEmpresa = ", temp,
					" AND ", NULL);
    g_free(temp);
    g_free(gchar_query_order);
    gchar_query_order = gchar_query_order_aux;
    gchar_query_order_aux = NULL;

    temp = g_strdup_printf("%lu",d_order_get_warehouse_code (order));
    gchar_query_order_aux = g_strconcat(gchar_query_order,
					"codigoAlmacen = ", temp,
					" ", NULL);
    g_free(temp);
    g_free(gchar_query_order);
    gchar_query_order = gchar_query_order_aux;
    gchar_query_order_aux = NULL;

    result = m_model_command_execute (&record_set, gchar_query_order, connection, command,tuples,0, FALSE);


    lines = d_order_get_order_lines (order);

    index = 0;
    while (lines[index]!=NULL) {
	d_order_line_set_code (lines[index], d_order_get_code (order));
	index ++;
    }

    result = result && m_order_line_delete_in_transaction (order, connection, command);
    result = result && m_order_line_insert (lines, connection, command);

    result = m_model_execute_mutex_modification_logged (connection) && result;

    g_free(gchar_query_order);
    g_free(tuples);

    gda_recordset_free(record_set);
    igalia_command_free(command);

    return result;
}
