/* payment_note_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "payment_note_model.h"

#define TABLE         "Nota"
#define ATTRIBUTES    "codigoNota as codigoNotaNota, codigoReparacion as codigoReparacionNota,\
    codigoEmpresa as codigoEmpresaNota, fechaEmision as fechaEmisionNota,codigoTienda as codigoTiendaNota,\
    codigoPago as codigoPagoNota";


/**
 * m_payment_note_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_payment_note_get_table () {
    return TABLE;
}

/**
 * m_payment_note_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_payment_note_get_attributes () {
    return ATTRIBUTES;
}


/**
 * m_payment_note_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #PaymentNote con los campos rellenados con los datos de la base de datos.
 */
PaymentNote * m_payment_note_create (GdaRecordset *rs) {

    GdaField * field;
    PaymentNote * payment_note;
    payment_note = d_payment_note_create_empty();

    field = gda_recordset_field_name(rs,"codigonotanota");

    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    payment_note->emission_date = igalia_field_get_bigint_value (field);

	    payment_note->payment = m_payment_create (rs);
	    payment_note->repair_detail = m_repair_create (rs,NULL);
	    gda_field_free(field);

	    field = gda_recordset_field_name(rs, "totalNota");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp=gda_stringify_value(NULL, 0, field);
		    payment_note->total = atof(temp);
		}
		gda_field_free(field);
	    }
	    field = gda_recordset_field_name(rs,"fechaEmisionNota");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp;
		    time_t * my_time;
		    temp=gda_stringify_value(NULL, 0, field);
		    my_time = u_parse_date (temp);
		    payment_note->emission_date = (*my_time);
		    my_time = NULL;
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	} else {
	    gda_field_free(field);
	}
    }
    return payment_note;
}



/**
 * m_payment_note_insert:
 * @payment_note: Nota a insertar en la base de datos.
 *
 * Inserta una nota en la base de datos. Esta insercin incluye la insercin de un pago asociado
 * a la nota.
 *
 * Returns:
 * %ULONG_MAX si no se ha insertado correctamente, cdigo de la nota en otro caso.
 */
gulong m_payment_note_insert (PaymentNote * payment_note) {
    gulong result=0;
    gchar * query1=NULL;
    gchar * query_aux;
    gchar * query2;
    gchar * query3;
    gchar * temp_nextval=NULL;
    gchar * query4;
    gchar * query;
    gchar * values=NULL;
    gchar * values_aux;
    Payment * payment = d_payment_note_get_payment (payment_note);
    gulong shop_code=ULONG_MAX;
    gulong enterprise_code=ULONG_MAX;
    gulong payer_code=ULONG_MAX;
    gulong payment_code = ULONG_MAX;
    gdouble total = G_MINDOUBLE;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set1=NULL;
    GdaRecordset *record_set2=NULL;
    GdaRecordset *record_set3=NULL;
    GdaRecordset *record_set4=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    time_t time_aux;
    gboolean with_payment=FALSE;

    if (d_payment_note_get_repair_detail (payment_note)!=NULL) {
	shop_code= d_participant_get_code (d_shop_get_participant (d_repair_get_shop (
										  d_payment_note_get_repair_detail (payment_note)
										 )));
	enterprise_code= d_participant_get_code (d_enterprise_get_participant (d_repair_get_enterprise (
											       d_payment_note_get_repair_detail (payment_note)
											      )));
	payer_code = d_participant_get_code (d_client_get_participant (d_repair_get_client (d_payment_note_get_repair_detail (payment_note))));
    }
    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,enterprise_code,shop_code,ULONG_MAX);

    if (d_payment_note_get_repair_detail (payment_note)!=NULL) {
	if ((enterprise_code!=ULONG_MAX) && (shop_code!=ULONG_MAX)) {
	    if (payment!=NULL) {
		/* The payment has data but it can already exists or no  */
		/* If the payment already exists codigoPago != ULONG_MAX */
                
		query1 = g_strconcat(" INSERT INTO PAGO VALUES ",NULL);
		values = g_strconcat(" (",NULL);
		payment_code = d_payment_get_payment_code (payment);
		if (payment_code==ULONG_MAX) {

		    if (d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (payment)))!=ULONG_MAX) {
			gchar * temp = g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (payment))));
			values_aux = g_strconcat(values,temp,",",NULL);
			g_free(values);
			g_free(temp);
			values = values_aux;
		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);
			g_free(values);
			values = values_aux;

		    }
		    if (d_participant_get_code (d_shop_get_participant (d_payment_get_shop (payment)))!=ULONG_MAX) {
			gchar * temp = g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (d_payment_get_shop (payment))));
			values_aux = g_strconcat(values,temp,",",NULL);
			g_free(values);
			g_free(temp);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);
			g_free(values);
			values = values_aux;

		    }
                    temp_nextval = m_model_nextval_sequence ("seqPago",connection);
		    values_aux = g_strconcat(values,temp_nextval,",",NULL);
		    g_free(values);
		    values = values_aux;
                    g_free(temp_nextval);

		    if (d_payment_get_achieved_payments (payment)!=ULONG_MAX) {
			gchar * temp = g_strdup_printf("%lu",d_payment_get_achieved_payments (payment));
			values_aux = g_strconcat(values,temp,",",NULL);
			g_free(temp);
			g_free(values);
			values = values_aux;
		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);
			g_free(values);
			values = values_aux;
		    }

		    if (d_payment_get_achieved_date (payment)!=time(NULL)) {
			time_t temp = d_payment_get_achieved_date (payment);
			values_aux = g_strconcat(values,"'",ctime(&temp),"',",NULL);
			g_free(values);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);
			g_free(values);
			values = values_aux;

		    }


		    if (d_payment_get_is_hire_purchase (payment)==TRUE) {
			values_aux = g_strconcat(values,"TRUE,",NULL);
			g_free(values);
			values = values_aux;
		    } else {
			values_aux = g_strconcat(values,"FALSE,",NULL);
			g_free(values);
			values = values_aux;
		    }

		    if (d_payment_get_init_date (payment)!=time(NULL)) {
			time_t temp = d_payment_get_init_date (payment);
			values_aux = g_strconcat(values,"'",ctime(&temp),"',",NULL);
			g_free(values);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);
			g_free(values);
			values = values_aux;

		    }

		    if (d_payment_get_entry_percentage (payment)!=-1) {
			gchar * temp =g_strdup_printf("%.2f",d_payment_get_entry_percentage (payment));
			u_commas_to_points(temp);
			values_aux = g_strconcat(values,temp,",",NULL);
			g_free(temp);
			g_free(values);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);
			g_free(values);
			values = values_aux;

		    }
		    if (d_payment_get_number_expiries (payment)!=-1) {
			gchar * temp = g_strdup_printf("%d",d_payment_get_number_expiries (payment));
			values_aux = g_strconcat(values,temp,",",NULL);
			g_free(temp);
			g_free(values);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);
			g_free(values);
			values = values_aux;

		    }

		    if (d_payment_get_interval_days (payment)!=-1) {
			gchar * temp = g_strdup_printf("%d",d_payment_get_interval_days (payment));
			values_aux = g_strconcat(values,temp,",",NULL);
			g_free(temp);
			g_free(values);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);
			g_free(values);
			values = values_aux;

		    }

		    if (d_payment_get_is_direct_debit (payment)==TRUE) {
			values_aux = g_strconcat(values,"TRUE,",NULL);
			g_free(values);
			values = values_aux;
		    } else {
			values_aux = g_strconcat(values,"FALSE,",NULL);
			g_free(values);
			values = values_aux;
		    }

		    if (d_payment_get_bank_code (payment)!=-1) {
			gchar * temp =g_strdup_printf("%d",d_payment_get_bank_code (payment));
			values_aux = g_strconcat(values,temp,",",NULL);
			g_free(temp);
			g_free(values);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);
			g_free(values);
			values = values_aux;

		    }

		    if (d_payment_get_office_code (payment)!=-1) {
			gchar * temp =g_strdup_printf("%d",d_payment_get_office_code (payment));
			values_aux = g_strconcat(values,temp,",",NULL);
			g_free(temp);
			g_free(values);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);
			g_free(values);
			values = values_aux;

		    }
		    g_print("%s",d_payment_get_account_number (payment));
		    if (d_payment_get_account_number (payment)!=NULL) {
			gchar * temp =d_payment_get_account_number (payment);
			values_aux = g_strconcat(values,temp,",",NULL);
			g_free(temp);
			g_free(values);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);
			g_free(values);
			values = values_aux;

		    }


		    if (d_payment_get_is_transfer (payment)==TRUE) {
			values_aux = g_strconcat(values,"TRUE,",NULL);
			g_free(values);
			values = values_aux;
		    } else {
			values_aux = g_strconcat(values,"FALSE,",NULL);
			g_free(values);
			values = values_aux;
		    }

		    if (d_payment_get_is_cheque (payment)==TRUE) {
			values_aux = g_strconcat(values,"TRUE,",NULL);
			g_free(values);
			values = values_aux;
		    } else {
			values_aux = g_strconcat(values,"FALSE,",NULL);
			g_free(values);
			values = values_aux;
		    }


		    if (d_payment_c_util_get_cheque_number (payment)!=NULL) {
			gchar * temp = d_payment_c_util_get_cheque_number (payment);
			values_aux = g_strconcat(values,temp,",",NULL);
			g_free(temp);
			g_free(values);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);
			g_free(values);
			values = values_aux;

		    }

		    if (d_payment_get_is_credit_card (payment)==TRUE) {
			values_aux = g_strconcat(values,"TRUE,",NULL);
			g_free(values);
			values = values_aux;
		    } else {
			values_aux = g_strconcat(values,"FALSE,",NULL);
			g_free(values);
			values = values_aux;
		    }

		    if (d_payment_get_credit_card_number (payment)!=NULL) {
			gchar * temp = d_payment_get_credit_card_number (payment);
			values_aux = g_strconcat(values,temp,",",NULL);
			g_free(temp);
			g_free(values);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);;
			g_free(values);
			values = values_aux;

		    }

		    if (d_payment_get_expiry_date (payment)!=time(NULL)) {
			time_t temp = d_payment_get_expiry_date (payment);
			values_aux = g_strconcat(values,"'",ctime(&temp),"',",NULL);
			g_free(values);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);;
			g_free(values);
			values = values_aux;

		    }

		    if (d_payment_get_is_paid_cash (payment)==TRUE) {
			values_aux = g_strconcat(values,"TRUE",NULL);
			g_free(values);
			values = values_aux;
		    } else {
			values_aux = g_strconcat(values,"FALSE",NULL);
			g_free(values);
			values = values_aux;
		    }


		    query_aux=g_strconcat(query1,values,")",NULL);
		    g_free(query1);
		    query1=query_aux;
		    query_aux=NULL;
		}
		if (m_model_command_execute (&record_set1, query1, connection,command,tuples,0,FALSE)==FALSE) {
                    result=ULONG_MAX;
		}

		query4 = g_strdup("select currval('seqPago')") ;
		if (m_model_command_execute (&record_set4, query4, connection,command,tuples,0,TRUE)==FALSE) {
                    result=ULONG_MAX;
		}
		if ((record_set4!=NULL) && (result==0)) {
		    if (*tuples!=0) {
			GdaField * field;
			gda_recordset_move_first(record_set4);
			field  = gda_recordset_field_idx(record_set4,0);
			if (field!=NULL) {
			    if (!gda_field_is_null(field)) {
				gchar * numeroPago=NULL;
				gchar ** stop = (gchar **)g_malloc0(sizeof(gchar *));
				gulong codigo_pago;
				numeroPago = gda_stringify_value(NULL, 0, field);
				codigo_pago = strtoul(numeroPago,stop,10);
                                d_payment_set_payment_code (d_payment_note_get_payment (payment_note),codigo_pago);
				g_free(stop);
			    }
			    gda_field_free(field);
			}

		    } else {
			result = ULONG_MAX;
		    }
		} else {
		    result = ULONG_MAX;
		}

		with_payment=TRUE;
	    } else {
		with_payment=FALSE;
	    }


	    g_free(values);
	    values = g_strconcat(" ",NULL);
	    query2 = g_strconcat("INSERT INTO NOTA VALUES (",NULL);

            temp_nextval=m_model_nextval_sequence ("seqNota",connection);
	    values_aux = g_strconcat(values,temp_nextval,",",NULL);
	    g_free(values);
	    values = values_aux;
            g_free(temp_nextval);

	    if (d_payment_note_get_repair_detail (payment_note)!=NULL) {
		if (d_repair_get_repair_code (d_payment_note_get_repair_detail (payment_note))!=ULONG_MAX) {
                    gchar * temp = g_strdup_printf("%lu",d_repair_get_repair_code (d_payment_note_get_repair_detail (payment_note)));
		    values_aux = g_strconcat(values,temp,",",NULL);
		    g_free(values);
                    g_free(temp);
		    values = values_aux;
		} else {
		    values_aux = g_strconcat(values,"NULL",NULL);
		    g_free(values);
		    values = values_aux;
		}
	    } else {
		values_aux = g_strconcat(values,"NULL",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (payment)))!=ULONG_MAX) {
		gchar * temp = g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (payment))));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(values);
		g_free(temp);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;

	    }
	    if (d_participant_get_code (d_shop_get_participant (d_payment_get_shop (payment)))!=ULONG_MAX) {
		gchar * temp = g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (d_payment_get_shop (payment))));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(values);
		g_free(temp);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;

	    }

	    if (with_payment==TRUE) {
                gchar * temp_currval=m_model_currval_sequence ("seqPago",connection);
		values_aux = g_strconcat(values,temp_currval,",",NULL);
		g_free(values);
		values = values_aux;
                g_free(temp_currval);
	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

            total = d_payment_note_get_total (payment_note);
	    if (total!=G_MINDOUBLE) {
		gchar * temp = g_strdup_printf("%10.2f",total);
		g_strstrip(temp);
		values_aux = g_strconcat(values,"to_number('",temp,"','99999999D99'),",NULL);
		g_free(values);
                g_free(temp);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    time_aux = d_payment_note_get_emission_date (payment_note);
	    values_aux = g_strconcat(values,"'",ctime(&time_aux),"'",NULL);
	    g_free(values);
	    values = values_aux;

	    query_aux=g_strconcat(query2,values,")",NULL);
	    g_free(query2);
	    query2=query_aux;
	    g_free(values);
	    query_aux=NULL;

	    if (m_model_command_execute (&record_set2, query2, connection,command,tuples,0,FALSE)==FALSE) {
		result=ULONG_MAX;
	    }

	    query3 = g_strconcat("select currval('seqNota') from seqNota",NULL);
	    if (m_model_command_execute (&record_set3, query3, connection,command,tuples,0,TRUE)==FALSE) {
		result=ULONG_MAX;
	    }

	    if ((record_set3!=NULL) && (result==0)) {
		if (*tuples!=0) {
		    GdaField * field;
		    gda_recordset_move_first(record_set3);
		    field  = gda_recordset_field_idx(record_set3,0);
		    if (field!=NULL) {
			if (!gda_field_is_null(field)) {
                            gchar * numeroNota=NULL;
			    gchar ** stop = (gchar **)g_malloc0(sizeof(gchar *));
			    numeroNota = gda_stringify_value(NULL, 0, field);
			    result = strtoul(numeroNota,stop,10);
			    g_free(stop);
			}
			gda_field_free(field);
		    }

		} else {
		    result = ULONG_MAX;
		}
	    } else {
		result = ULONG_MAX;
	    }


	    if (query1!=NULL) {
		query = g_strconcat (query1,";\n",query2,";\n",query3,";\n",NULL);
		g_free(query1);
	    } else {
		query = g_strconcat (query2,";\n",query3,";\n",NULL);
	    }

	    g_free(query2);
	    g_free(query3);
	    g_free(query);
	    g_free(tuples);

	    if (record_set1!=NULL)
		gda_recordset_free(record_set1);
	    if (record_set2!=NULL)
		gda_recordset_free(record_set2);
	    if (record_set3!=NULL)
		gda_recordset_free(record_set3);


	}
    }
    if (m_model_execute_mutex_modification_logged (connection)!=TRUE) {
	result = ULONG_MAX;
    }

    igalia_command_free(command);

    return result;
}

/**
 * m_payment_note_find_by_condition:
 * @condition:
 *
 * Permite buscar una nota por una serie de condiciones. Los filtros que se pueden elegir
 * so los mismos que en la funcion m_invoice_find_by_condition ().
 *
 * Returns:
 * Lista de notas que cumplent la condicin de bsqueda.
 */
PaymentNote ** m_payment_note_find_by_condition (ConditionFindInvoice * condition) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar              *gchar_where=NULL;
    gchar              *gchar_where_aux=NULL;
    gchar              *gchar_from=NULL;
    gchar              *gchar_order=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set1 = NULL;
    gulong index_of_payment_notes=0;

    PaymentNote ** payment_notes=NULL;
    PaymentNote * payment_note;
    gint index;
    gulong * tuples1;

    my_command = gda_command_new();
    tuples1 = (gulong *) g_malloc0(sizeof(gulong));
    gda_command_set_connection(my_command,my_connection);

    if  (condition->type==D_PAYMENT_NOTE_CODE) {

	gchar_select = g_strconcat("SELECT ",
				   "nota.codigoNota as codigoNotaNota,",
				   "nota.codigoReparacion as codigoReparacionRepair,",
				   "nota.codigoTienda as codigoPartyParticipantShop,",
                                   "nota.codigoEmpresa as codigoPartyParticipantEnter,",
				   "nota.codigoPago as codigoPagoNota,",
				   "nota.fechaEmision as fechaEmisionPagoNota,",
				   "to_char(nota.total,'99999999D99') as totalNota,",
				   "participanteShop.codigoParty as codigoPartyParticipantShop,",
				   "participanteShop.nombre as nombreParticipantShop,",
				   "participanteCliente.codigoParty as codigoPartyParticipantClient,",
				   "participanteCliente.nick as nickParticipantClient,",
				   "participanteCliente.nombreCorto as shortNameParticipantClient,",
				   "participanteCliente.cif as cifParticipantClient,",
				   "participanteEmpresa.nombre as nombreParticipantEnter,",
				   "reparacion.codigoCompania as codigoCompaniaRepair,",
				   "reparacion.matricula as matriculaVehicle,",
				   "tienda.codigoPertenenciaGrupo as codigoPertenenciaGrupoShop,",
				   "participanteCompany.nombre as nombreParticipantCompany,",
				   "participanteCompany.nombreCorto as shortNameParticipantCompany,",
				   "participanteCompany.codigoParty as codigoPartyParticipantCompany,",
				   "participanteCompany.cif as cifParticipantCompany",
				   NULL );


	gchar_from = g_strconcat(" from reparacion left outer join participante participanteCompany on ",
                                 "(reparacion.codigoCompania=participanteCompany.codigoParty),",
				 "participante participanteCliente,",
				 "participante participanteShop,",
                                 "participante participanteEmpresa,",
				 "tienda,",
				 "nota ",
				 NULL);



	gchar_where=g_strconcat(" where ",
				" nota.codigoTienda=participanteShop.codigoParty ",
				" and nota.codigoTienda =reparacion.codigoTienda ",
				" and reparacion.codigoCliente=participanteCliente.codigoParty ",
				" and nota.codigoEmpresa =reparacion.codigoEmpresa ",
				" and nota.codigoReparacion = reparacion.codigoReparacion",
				" and tienda.codigoTienda = nota.codigoTienda ",
				" and nota.codigoEmpresa = participanteEmpresa.codigoParty ",
				NULL);

        gchar_order=g_strconcat("order by nota.codigonota",NULL);
	if ((condition->shop_code) > 0) {
	    gchar * temp = g_strdup_printf("%lu",condition->shop_code);
	    gchar_where_aux = g_strconcat(gchar_where, " and (nota.codigoTienda=",
					  temp,")",NULL);
	    g_free(gchar_where);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}

	if (((condition->client_name) != NULL) && (strlen(condition->client_name)!=0)) {
	    gchar_where_aux = g_strconcat(gchar_where, " and (participanteCliente.nick ilike '%",
					  condition->client_name,"%')",NULL);
	    g_free(gchar_where);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}

	if (((condition->client_legal_id) != NULL) && (strlen(condition->client_legal_id)!=0)) {
	    gchar_where_aux = g_strconcat(gchar_where, " and (participanteCliente.cif='",
					  condition->client_legal_id,"')",NULL);
	    g_free(gchar_where);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}

	if ((condition->initial_issue_date) != NULL) {
	    gchar * temp_date = g_strdup(ctime(condition->initial_issue_date));

	    gchar_where_aux = g_strconcat(gchar_where, " and (nota.fechaEmision>='",
					  temp_date,"')",NULL);
	    g_free(gchar_where);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
            g_free(temp_date);
	}

	if ((condition->ending_issue_date) != NULL)  {
	    gchar * temp_date = g_strdup(ctime(condition->ending_issue_date));
	    gchar_where_aux = g_strconcat(gchar_where, " and (nota.fechaEmision<='",
					  temp_date,"')",NULL);
	    g_free(gchar_where);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
            g_free(temp_date);
	}


	if ((condition->invoice_number) != 0)  {
	    gchar * temp = g_strdup_printf("%lu",condition->invoice_number);
	    gchar_where_aux = g_strconcat(gchar_where, " and (nota.codigoNota=",
					  temp,")",NULL);
	    g_free(gchar_where);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}
	if (((condition->license_number) != NULL) && (strlen(condition->license_number)!=0)) {
	    gchar_where_aux = g_strconcat(gchar_where, " and (reparacion.matricula ilike '%",
					  condition->license_number,"%')",NULL);
	    g_free(gchar_where);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}

	if ((condition->amount) != 0)  {
	    gchar * temp = g_strdup_printf("%.2f",condition->amount);
	    gchar_where_aux = g_strconcat(gchar_where, " and (nota.total=to_number('",
					  temp,"','99999999D99'))",NULL);
	    g_free(gchar_where);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}
	gchar_query = g_strconcat(gchar_select,gchar_from,gchar_where,gchar_order, NULL);

	g_print(_("\nQuery: %s\n"),gchar_query);
	m_model_command_execute (&record_set1, gchar_query, my_connection, my_command,tuples1,0,TRUE);

	if (record_set1!=NULL) {
	    if (*tuples1!=0) {
		payment_notes = (PaymentNote **) g_malloc0(sizeof(PaymentNote*)*(*tuples1+1));
		index = gda_recordset_move_first(record_set1);
		while (!gda_recordset_eof(record_set1)) {
		    if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		    payment_note = m_payment_note_create (record_set1);
		    payment_notes[index_of_payment_notes] = payment_note;
		    index = gda_recordset_move_next(record_set1);
		    index_of_payment_notes ++;
		}
	    } else {
		payment_notes=NULL;
	    }
	    gda_recordset_close(record_set1);
	    gda_recordset_free(record_set1);
	} else {
	    payment_notes=NULL;
	}


	if (payment_notes!=NULL) {
	    payment_notes[index_of_payment_notes] = NULL;
	}

	g_free(gchar_where);
	g_free(gchar_select);
	g_free(gchar_from);
	g_free(gchar_order);
	g_free(gchar_query);
	g_free(tuples1);

    }

    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    return payment_notes;

}

/**
 * m_payment_note_find_by_id:
 * @payment_note_id:
 *
 * Busca una nota en la base de datos por identificador de la nota.
 *
 * Returns:
 * Devuelve la nota si la ha encontrad o %NULL en caso contrario.
 */
PaymentNote * m_payment_note_find_by_id (PaymentNoteId * payment_note_id){
    PaymentNote *payment_note=NULL;
    GdaCommand  *my_command;
    gchar       *gchar_query=NULL;
    gchar       *gchar_select=NULL;
    gchar       *gchar_from;
    gchar       *gchar_where;
    GdaConnection *my_connection = m_model_get_connection ();
    GdaRecordset  *record_set = NULL;
    GdaField *field;
    GdaField *field2=NULL;

    gint index;
    gulong repair_code=ULONG_MAX;
    gulong payment_code=ULONG_MAX;
    gulong * tuples;
    gint error=0;

    tuples = (gulong *) g_malloc0(sizeof(gulong));

    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);
    gchar_select=g_strconcat("select codigoNota as codigoNotaNota,",
			     "codigoTienda as codigoTiendaNota,",
			     "codigoEmpresa as codigoEmpresaNota,",
			     "codigoReparacion as codigoReparacionNota,"
			     "to_char(total,'99999999D99') as totalNota,"
			     "codigoPago as codigoPagoNota,",
			     "fechaEmision as fechaEmisionNota ",
			     NULL);
    gchar_from = g_strconcat(" from nota ",NULL);
    gchar_where = g_strconcat(" where nota.codigoNota=",payment_note_id->payment_note_code,
			      " and nota.codigoEmpresa=",payment_note_id->enterprise_code,
                              " and nota.codigoTienda=",payment_note_id->shop_code,
			      NULL);


    gchar_query = g_strconcat(gchar_select,gchar_from,gchar_where,NULL);
    g_print(_("\nQuery: %s\n"),gchar_query);
    m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);

    if (record_set!=NULL) {
	if (*tuples==1) {
	    index = gda_recordset_move_first(record_set);
	    field = gda_recordset_field_name(record_set,"codigoReparacionNota");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    repair_code=igalia_field_get_bigint_value (field);
		} else {
                    repair_code=ULONG_MAX;
		}
		gda_field_free(field);
	    }
	    field2 = gda_recordset_field_name(record_set,"codigoPagoNota");
	    if (field2!=NULL) {
		if (!gda_field_is_null(field2)) {
		    payment_code=igalia_field_get_bigint_value (field2);
		}
		gda_field_free(field2);
	    }
	    payment_note = m_payment_note_create (record_set);
	} else {
            error =1;
	}
	gda_recordset_close(record_set);
	gda_recordset_free(record_set);
        record_set=NULL;
    }
    g_free(gchar_query);
    g_free(gchar_select);
    g_free(gchar_where);
    g_free(gchar_from);

    
    if (error!=1) {
	PaymentId * payment_id = (PaymentId *) g_malloc0(sizeof(PaymentId));
	Repair_line ** repair_lines;
	RepairId * repair_id;
	repair_id = (RepairId *) g_malloc0(sizeof(RepairId));

	payment_id->payment_code=g_strdup_printf("%lu",payment_code);
        payment_id->enterprise_code=g_strdup(payment_note_id->enterprise_code);
        payment_id->shop_code=g_strdup(payment_note_id->shop_code);
	d_payment_note_set_payment (payment_note,m_payment_find_by_id (payment_id));

	repair_id->enterprise_code = g_strdup(payment_note_id->enterprise_code);
	repair_id->shop_code = g_strdup(payment_note_id->shop_code);
	repair_id->repair_code = g_strdup_printf("%lu",repair_code);

	repair_lines =  m_repair_line_find_by_id (repair_id);

	d_payment_note_set_repair_detail (payment_note,m_repair_find_by_id (repair_id,repair_lines));

    } else {
	payment_note =NULL;
    }

    g_free(tuples);
    igalia_command_free(my_command);

    return payment_note;
}




/**
 * m_payment_note_clear_array:
 * @payment_notes: Lista de notas que se desea liberar.
 *
 * Libera la memoria utilizada por una lista de notas.
 *
 */
void m_payment_note_clear_array (PaymentNote ** payment_notes) {
    if (payment_notes!=NULL) {
	gint index = 0;
	while (payment_notes[index]!=NULL) {
	    d_payment_note_clear (payment_notes[index]);
	    index++;
	}
    }
    g_free(payment_notes);
    payment_notes=NULL;
}

/**
 * m_payment_note_delete:
 * @payment_note: Nota a borrar.
 *
 * Borra una nota en la base de datos.
 *
 * Returns:
 * %NULL si se ha borrado correctamente, un mensaje de error en otro caso.
 */
gchar *m_payment_note_delete (PaymentNote * payment_note) {
    gchar * query;
    GdaConnection *connection = m_model_get_connection ();
    GdaCommand *command  = gda_command_new();
    GdaRecordset *record_set=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    gulong payment_note_code = d_payment_note_get_note_code (payment_note);
    gulong enterprise_code = d_participant_get_code (d_enterprise_get_participant (d_repair_get_enterprise (d_payment_note_get_repair_detail (payment_note))));
    gulong shop_code = d_participant_get_code (d_shop_get_participant (d_repair_get_shop (d_payment_note_get_repair_detail (payment_note))));
    gulong payment_code = d_payment_get_payment_code (d_payment_note_get_payment (payment_note));
    gchar * payment_note_code_char = NULL; 
    gchar * payment_code_char = NULL;
    gchar * shop_code_char = NULL;
    gchar * enterprise_code_char = NULL;
    gchar * result = NULL;
    gboolean no_error=TRUE;

    gda_command_set_connection(command,connection);

    payment_note_code_char = g_strdup_printf("%lu",payment_note_code);
    enterprise_code_char = g_strdup_printf("%lu",enterprise_code);
    shop_code_char = g_strdup_printf("%lu",shop_code);
    payment_code_char = g_strdup_printf("%lu",payment_code);

    m_model_connection_begin_mutex_transaction_logged (connection,enterprise_code,shop_code,ULONG_MAX);

    query = g_strconcat(" delete from pago ",
			" WHERE CODIGOPAGO=",payment_code_char,
			" AND CODIGOEMPRESA=",enterprise_code_char,
			" AND CODIGOTIENDA=",shop_code_char,NULL);

    no_error = m_model_command_execute (&record_set, query, connection,command,tuples,0,TRUE);

    g_print(_("\nDelete: %s\n"),query);

    if (no_error!=FALSE) {
	if (record_set==NULL) {
	    result = g_strdup(_("Error deleting the payment note."));
	}
    } else {
	result = g_strdup(_("Error deleting the payment note."));
    }

    gda_recordset_free(record_set);

    m_model_execute_mutex_modification_logged (connection);

    if (query!=NULL)
	g_free(query);
    query=NULL;
    g_free(tuples);
    if (payment_note_code_char!=NULL)
	g_free(payment_note_code_char);
    if (enterprise_code_char!=NULL)
	g_free(enterprise_code_char);
    if (shop_code_char!=NULL)
	g_free(shop_code_char);
    if (payment_code_char!=NULL)
	g_free(payment_code_char);

    return result;
}




