/* piece_history_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "piece_history_model.h"
#include "delivery_payment_movement_model.h"
#include "repair_movement_model.h"
#include "exchange_piece_model.h"
#include "regularization_model.h"

static gint recordset_to_piece_history_array(GdaRecordset *rs,PieceHistory **history_array);

/**
 * m_piece_history_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #PieceHistory con los campos rellenados con los datos de la base de datos.
 */
PieceHistory  *m_piece_history_create (GdaRecordset *rs){
    PieceHistory * result=d_piece_history_create_empty ();
    GdaField * field = NULL;
    

    field = gda_recordset_field_name(rs,"tipoMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result->movement_type = gda_field_get_integer_value(field);
	}
	gda_field_free(field);
    }
    switch (result->movement_type) {
    case D_PIECE_HISTORY_REGULARIZATION_MOVEMENT:{
        result -> movement = (gpointer) m_regularization_create (rs);
	break;
    }
    case D_PIECE_HISTORY_INTERCHANGE_MOVEMENT:{
        result -> movement = (gpointer) m_exchange_create (rs);
	break;
    }
    case D_MOVEMENT_REPAIR:{
	result -> movement = (gpointer) m_repair_movement_create (rs);
	break;
    }
    case D_MOVEMENT_DELIVERY_PAYMENT:{
	result -> movement = (gpointer) m_delivery_payment_movement_create (rs);
        break;
    } default: {

	result->movement_type=-1;
    }
    }




    return result;

}



/**
 * :model_find_history_by_condition
 * @condition: Filtro de bsqueda. Permite buscar movimientos por cdigo de empresa,
 * cdigo de almacn, por producto y por perodo de fechas.
 * @tuples: Nmero de piezas que cumplen la condicin.
 *
 * Permite buscar los movimientos de piezas producidos en un almacn.
 *
 * Returns:
 * Array con la lista de movimientos de piezas que cumplen la condicin de bsqueda.
 */
PieceHistory ** m_piece_history_find_by_condition (ConditionFindHistory *condition, gulong * tuples) {
    GdaCommand    *command;
    gchar       *regularization_select=NULL;
    gchar       *interchange_select=NULL;
    gchar       *select_reparacion=NULL;
    gchar       *delivery_payment_select=NULL;
    gchar       *pending_select=NULL;
    gchar * gchar_select;
    gchar * gchar_union;
    gchar * gchar_query=NULL;
    gchar * gchar_order = "order by fechaMovimiento";
    gchar * gchar_in = "TRUE as entradaMovimiento";
    gchar * gchar_out = "FALSE as entradaMovimiento";
    gchar * gchar_from_interchange = " from intercambio,almacen";
    gchar * gchar_where_in = "and almacen.codigoAlmacen=intercambio.codigoAlmacen ";
    gchar * gchar_where_out = "and almacen.codigoAlmacen=intercambio.codigoAlmacenDestino ";
    gchar * movement_type_reg = g_strdup_printf("%d",D_PIECE_HISTORY_REGULARIZATION_MOVEMENT);
    gchar * movement_type_ex =g_strdup_printf("%d",D_PIECE_HISTORY_INTERCHANGE_MOVEMENT);
    gchar * movement_type_rm =g_strdup_printf("%d",D_MOVEMENT_REPAIR);
    gchar * movement_type_dpm =g_strdup_printf("%d",D_MOVEMENT_DELIVERY_PAYMENT);

    gchar * gchar_where_in_interchange = NULL;
    gchar * gchar_where_out_interchange = NULL;
    gchar * gchar_where = NULL;
    gchar * gchar_where_repair = NULL;
    gchar * gchar_where_pending = NULL;
    GdaConnection *connection = m_model_get_connection ();

    GdaRecordset  *record_set = NULL;
    PieceHistory  **history_array;
    gboolean       correct = FALSE;
    gchar * enterprise_code_char=g_strdup_printf("%lu",condition->enterprise_code);
    gchar * warehouse_code_char=g_strdup_printf("%lu",condition->warehouse_code);
    gchar * product_code_char=g_strdup_printf("%lu",condition->product_code);
    gchar * initial_date= g_strdup(ctime(&condition->initial_date));
    gchar * ending_date= g_strdup(ctime(&condition->ending_date));

    command = gda_command_new();

    gda_command_set_connection(command,connection);

    gchar_union = " union ";
    gchar_select = " select * from ";
    interchange_select=g_strconcat(" select intercambio.codigoEmpresa as codigoEmpresaMovimiento,",
				   "intercambio.codigoAlmacen as codigoAlmacenMovimiento,",
				   "codigoMovimiento as codigoMovimiento,",
				   "fecha as fechaMovimiento,",
				   "unidades as unidadesMovimiento,",
				   "codigoProducto as codigoProductoMovimiento,",
				   "codigoEmpresaDestino as codigoEmpresaDestinoMovimiento,",
				   "codigoALmacenDestino as codigoAlmacenDestinoMovimiento,",
				   "CAST(NULL as BIGINT) as codigoProveedorMovimiento,",
				   "CAST(NULL as VARCHAR(40)) as numeroAlbaranMovimiento,",
				   "CAST(NULL as BIGINT) as numeroLineaPedidoMovimiento,",
				   "CAST(NULL as BIGINT) as codigoPedidoMovimiento,",
				   "CAST(NULL as BIGINT) as codigoReparacionMovimiento,",
				   "CAST(NULL as BIGINT) as codigoEMpresaRepaMovimiento,",
				   "CAST(NULL as BIGINT) as codigoTiendaMovimiento,",
				   "CAST(NULL as BIGINT) as numeroLineaMovimiento,",
				   "CAST(NULL as VARCHAR(12)) as matriculaMovimiento,",
				   movement_type_ex," as tipoMovimiento,",
                                   "nick as nombreAlmacenDestinoMovimiento, ",
				      NULL);
    delivery_payment_select=g_strconcat(" select codigoEmpresa as codigoEmpresaMovimiento,",
			       "codigoAlmacen as codigoAlmacenMovimiento,",
			       "codigoMovimiento as codigoMovimiento,",
			       "fecha as fechaMovimiento,",
			       "unidades as unidadesMovimiento,",
			       "codigoProducto as codigoProductoMovimiento,",
			       "NULL as codigoEmpresaDestinoMovimiento,",
			       "NULL as codigoAlmacenDestinoMovimiento,",
			       "codigoProveedor as codigoProveedorMovimiento,",
			       "numeroAlbaran as numeroAlbaranMovimiento,",
			       "numeroLineaPedido as numeroLineaPedidoMovimiento,",
			       "codigoPedido as codigoPedidoMovimiento,",
			       "NULL as codigoReparacionMovimiento,",
			       "NULL as codigoEMpresaRepaMovimiento,",
			       "NULL as codigoTiendaMovimiento,",
			       "NULL as numeroLineaMovimiento, ",
			       "NULL as matriculaMovimiento,",
			       movement_type_dpm," as tipoMovimiento,",
			       "NULL as entradaMovimiento, ",
			       "NULL as nombreAlmacenDestinoMovimiento ",
			       "from movimientoPorAlbaran ",NULL);

    pending_select=g_strconcat(" select lineaPEdido.codigoEmpresa as codigoEmpresaMovimiento,",
			       "lineaPEdido.codigoAlmacen as codigoAlmacenMovimiento,",
			       "NULL as codigoMovimiento,",
			       "pedido.fecha as fechaMovimiento,",
			       "lineaPEdido.unidades as unidadesMovimiento,",
			       "lineaPEdido.codigoProducto as codigoProductoMovimiento,",
			       "NULL as codigoEmpresaDestinoMovimiento,",
			       "NULL as codigoAlmacenDestinoMovimiento,",
			       "pedido.codigoProveedor as codigoProveedorMovimiento,",
			       "lineaPEdido.numeroAlbaran as numeroAlbaranMovimiento,",
			       "lineaPEdido.numeroLinea as numeroLineaPedidoMovimiento,",
			       "lineaPEdido.codigoPedido as codigoPedidoMovimiento,",
			       "NULL as codigoReparacionMovimiento,",
			       "NULL as codigoEMpresaRepaMovimiento,",
			       "NULL as codigoTiendaMovimiento,",
			       "NULL as numeroLineaMovimiento, ",
			       "NULL as matriculaMovimiento,",
			       movement_type_dpm," as tipoMovimiento,",
			       "NULL as entradaMovimiento, ",
			       "NULL as nombreAlmacenDestinoMovimiento ",
				 "from lineaPedido left outer join pedido on lineaPedido.codigoPedido=pedido.codigoPedido ",
				 " and lineaPedido.codigoAlmacen=pedido.codigoAlmacen and lineaPedido.codigoEmpresa=pedido.codigoEmpresa ",NULL);

    regularization_select=g_strconcat(" select codigoEmpresa as codigoEmpresaMovimiento,",
				      "codigoAlmacen as codigoAlmacenMovimiento,",
				      "codigoMovimiento as codigoMovimiento,",
				      "fecha as fechaMovimiento,",
				      "unidades as unidadesMovimiento,",
				      "codigoProducto as codigoProductoMovimiento,",
				      "NULL as codigoEmpresaDestinoMovimiento,",
				      "NULL as codigoAlmacenDestinoMovimiento,",
				      "NULL as codigoProveedorMovimiento,",
				      "NULL as numeroAlbaranMovimiento,",
				      "NULL as numeroLineaPedidoMovimiento,",
				      "NULL as codigoPedidoMovimiento,",
				      "NULL as"," codigoReparacionMovimiento,",
				      "NULL as codigoEMpresaRepaMovimiento,",
				      "NULL as codigoTiendaMovimiento,",
				      "NULL as numeroLineaMovimiento, ",
				      "NULL as matriculaMovimiento,",
				      movement_type_reg," as tipoMovimiento,",
				      "NULL as entradaMovimiento, ",
				      "NULL as nombreAlmacenDestinoMovimiento ",
				      " from regularizacion",NULL);

    select_reparacion=g_strconcat(" select movimientoPorReparacion.codigoEmpresa as codigoEmpresaMovimiento,",
				  "movimientoPorReparacion.codigoAlmacen as codigoAlmacenMovimiento,",
				  "codigoMovimiento as codigoMovimiento,",
				  "fecha as fechaMovimiento,",
				  "unidades as unidadesMovimiento,",
				  "codigoProducto as codigoProductoMovimiento,",
				  "NULL as codigoEmpresaDestinoMovimiento,",
				  "NULL as codigoAlmacenDestinoMovimiento,",
				  "NULL as codigoProveedorMovimiento,",
				  "NULL as numeroAlbaranMovimiento,",
				  "NULL as numeroLineaPedidoMovimiento,",
				  "NULL as codigoPedidoMovimiento,",
				  "movimientoPorReparacion.codigoReparacion as codigoReparacionMovimiento,",
				  "codigoEmpresaReparacion as codigoEMpresaRepaMovimiento,",
				  "movimientoPorReparacion.codigoTienda as codigoTiendaMovimiento,",
				  "numeroLinea as numeroLineaMovimiento, ",
				  "matricula as matriculaMovimiento,",
				  movement_type_rm," as tipoMovimiento,",
				  "NULL as entradaMovimiento, ",
				  "NULL as nombreAlmacenDestinoMovimiento ",
				  "from movimientoPorReparacion left outer join reparacion on "
				  " (movimientoPorReparacion.codigoReparacion=reparacion.codigoReparacion) "
				  " and (movimientoPorReparacion.codigoEmpresaReparacion=reparacion.codigoEmpresa)"
				  " and (movimientoPorReparacion.codigoTienda=reparacion.codigoTienda) ",
				  NULL);
    u_delete_intros (initial_date);
    u_delete_intros (ending_date);
    gchar_where=g_strconcat(" where codigoAlmacen=",warehouse_code_char,
			    " and codigoEmpresa=",enterprise_code_char,
			    " and codigoProducto=",product_code_char,
			    " and fecha>='",initial_date,
			    " ' and fecha<='",ending_date,"' ",NULL);
    gchar_where_pending=g_strconcat(" where numeroAlbaran is null and lineaPEdido.codigoAlmacen=",warehouse_code_char,
			    " and lineaPEdido.codigoEmpresa=",enterprise_code_char,
			    " and codigoProducto=",product_code_char,
			    " and fecha>='",initial_date,
			    " ' and fecha<='",ending_date,"' ",NULL);

    gchar_where_repair=g_strconcat(" where movimientoPorReparacion.codigoAlmacen=",warehouse_code_char,
			    " and movimientoPorReparacion.codigoEmpresa=",enterprise_code_char,
			    " and codigoProducto=",product_code_char,
			    " and fecha>='",initial_date,
			    " ' and fecha<='",ending_date,"' ",NULL);
    gchar_where_out_interchange=g_strconcat(" where intercambio.codigoAlmacen=",warehouse_code_char,
			    " and intercambio.codigoEmpresa=",enterprise_code_char,
			    " and codigoProducto=",product_code_char,
			    " and fecha>='",initial_date,
			    " ' and fecha<='",ending_date,"' ",NULL);

    gchar_where_in_interchange=g_strconcat(" where codigoAlmacenDestino=",warehouse_code_char,
					" and codigoEmpresaDestino=",enterprise_code_char,
					" and codigoProducto=",product_code_char,
					" and fecha>='",initial_date,
					" ' and fecha<='",ending_date,"' ",NULL);


    gchar_query = g_strconcat(gchar_select,"((",
			      interchange_select,gchar_in,gchar_from_interchange,gchar_where_in_interchange,gchar_where_in,")",gchar_union,"(",
			      regularization_select,gchar_where,")",gchar_union,"(",
			      interchange_select,gchar_out,gchar_from_interchange,gchar_where_out_interchange,gchar_where_out,")",gchar_union,"(",
			      select_reparacion,gchar_where_repair,")",gchar_union,"(",
			      pending_select,gchar_where_pending,")",gchar_union,"(",
			      delivery_payment_select,gchar_where,")",
			      ") as movimientos ",gchar_order,NULL);

    correct = m_model_command_execute (&record_set, gchar_query, connection, command,tuples,0, TRUE);
    g_print(_("\nHISTORY: %s\n"),gchar_query);

    history_array=NULL;

    if (correct) {
	if (*tuples!=0) {
	    history_array = (PieceHistory **) g_malloc0 ((sizeof(PieceHistory *))*(*tuples +1));
	    recordset_to_piece_history_array(record_set, history_array);
	         
	}
    }



    if (record_set!=NULL) gda_recordset_free(record_set);
    igalia_command_free(command);

    g_free(enterprise_code_char);
    g_free(warehouse_code_char);
    g_free(product_code_char);
    g_free(initial_date);
    g_free(ending_date);
    g_free(interchange_select);
    g_free(regularization_select);
    g_free(delivery_payment_select);
    g_free(select_reparacion);
    g_free(gchar_where_out_interchange);
    g_free(gchar_where_in_interchange);
    g_free(gchar_where);
    g_free(gchar_where_repair);
    g_free(gchar_query);
    g_free(movement_type_dpm);
    g_free(movement_type_rm);
    g_free(movement_type_ex);
    g_free(movement_type_reg);
    return history_array;

}



/**
 * recordset_to_piece_history_array:
 * @rs: Record set del que se desea obtener la lista de piezas.
 * @history_array: Lista de movimientos de piezas en un almacn.
 * 
 * Obtiene una lista de movimientos de piezas en un almacn.
 *
 * Returns:
 * Tamao de la lista de movimientos.
 */
static gint recordset_to_piece_history_array(GdaRecordset *rs,
				        PieceHistory** history_array) {
    gint index;
    index = gda_recordset_move_first(rs);

    while (!gda_recordset_eof(rs)) {

	if (index==GDA_RECORDSET_INVALID_POSITION) return -1;

	history_array[index-1] = m_piece_history_create (rs);

	index = gda_recordset_move_next(rs);
    }
    history_array[index] = NULL;

    return index;
}

/**
 * m_piece_history_clear_array:
 * @piece_warehouse_history: Array de piezas en stock que se desea liberar.
 *
 * Libera la memoria utilizada por un array de piezas en stock.
 *
 * Returns: 0
 */

gint m_piece_history_clear_array (PieceHistory ** piece_warehouse_history) {
    gulong index = 0;

    if (piece_warehouse_history!=NULL) {
	while(piece_warehouse_history[index]!=NULL) {
	    d_piece_history_clear (piece_warehouse_history[index]);
            piece_warehouse_history[index] = NULL;
	    index++;
	}
    }
    g_free(piece_warehouse_history);
    piece_warehouse_history=NULL;
    return 0;
}



