/* piece_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "piece_model.h"
#include "enterprise_model.h"
#include "shop_model.h"
#include "warehouse_model.h"
#include "parameter_manager.h"
#include "product_model.h"

#define TABLE      "Pieza"
#define ATTRIBUTES "eurocode as eurocodepiece, \
    unidadesPorPaquete as unidadesporPaquetePiece ,codigoMedida as codigoMedidaPiece,largo as largopiece, ancho as anchopiece, \
   preciocoste as preciocostepiece, codigoProveedor as codigoProveedorPiece"

/**
 * m_piece_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_piece_get_table () {
    return TABLE;
}

/**
 * m_piece_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_piece_get_attributes () {
    return ATTRIBUTES;
}


/**
 * m_piece_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Piece con los campos rellenados con los datos de la base de datos.
 */
Piece * m_piece_create (GdaRecordset *rs) {
    gchar  *temp;
    GdaField * field = NULL;
    Piece * piece = d_piece_create_empty();
    Product product;

    product = m_product_create (rs);
    d_product_set_is_piece (&product, TRUE);
    d_piece_set_product (piece, product);

    if(d_product_get_product_code (&product)!=-1) {

	field = gda_recordset_field_name(rs,"eurocodepiece");
	if (field!=NULL) {
	    if (!gda_field_is_null(field)) {
		temp=gda_stringify_value(NULL, 0, field);
		piece->euroCode = g_strdup(temp);
                g_strstrip(piece->euroCode);
		g_free(temp);
		gda_field_free(field);

		field = gda_recordset_field_name(rs,"largopiece");
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
			piece->length = gda_field_get_smallint_value(field);
		    }
		    gda_field_free(field);
		}

		field = gda_recordset_field_name(rs,"unidadesporpaquetepiece");
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
			piece->units_per_packet = igalia_field_get_double_value (field);
		    }
		    gda_field_free(field);
		}

		field = gda_recordset_field_name(rs,"codigoMedidapiece");
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
			piece->meassure_code = igalia_field_get_bigint_value (field);
		    }
		    gda_field_free(field);
		}

		field = gda_recordset_field_name(rs,"codigoProveedorpiece");
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
			piece->supplier_code = igalia_field_get_bigint_value (field);
		    }
		    gda_field_free(field);
		}

		field = gda_recordset_field_name(rs,"anchopiece");
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
			piece->width = gda_field_get_smallint_value(field);
		    }
		    gda_field_free(field);
		}

		field = gda_recordset_field_name(rs,"preciocostepiece");
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
			temp=gda_stringify_value(NULL, 0, field);
			u_points_to_commas (temp);
			piece->cost_price = atof(temp);
			g_free(temp);
		    }
		    gda_field_free(field);
		}

	    } else {
		gda_field_free(field);
	    }
	}
    }

    return piece;
}

/**
 * m_piece_insert: 
 * @piece: Pieza a insertar.
 *
 * Inserta una pieza en la base de datos.
 *
 * Returns:
 * %TRUE si se ha insertado correctamente, %FALSE en otro caso.
 */
gboolean m_piece_insert (Piece *piece) {
    gboolean result = FALSE;

    if (d_product_get_is_piece ((Product *)piece)) {
	gchar         *gchar_query_piece;
	gchar         *gchar_query_piece_aux;
	GdaRecordset  *record_set = NULL;
	gchar         *temp;
	gulong        *tuples = (gulong *) g_malloc0(sizeof(gulong));
	GdaConnection *connection = m_model_get_connection ();
	GdaCommand    *command = gda_command_new();
	gulong logged_shop_code;
	gulong logged_enterprise_code;
	gulong logged_warehouse_code;
	gchar * temp_name;

	temp_name = u_parameter_manager_get_parameter ("empresa");
	logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
	g_free(temp_name);
	temp_name = u_parameter_manager_get_parameter ("tienda");
	logged_shop_code = m_shop_get_code_from_name (temp_name);
	g_free(temp_name);
	temp_name = u_parameter_manager_get_parameter ("almacen");
	logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

	g_free(temp_name);

	gda_command_set_connection(command,connection);
	m_model_connection_begin_mutex_transaction_logged (connection,logged_enterprise_code,logged_shop_code,logged_warehouse_code);

	result = m_product_insert_in_transaction ((Product *)piece, connection, command);

	gchar_query_piece = "";
	gchar_query_piece = g_strconcat("INSERT INTO ",m_piece_get_table (),
					" VALUES ",
					"('", NULL);

	temp = d_piece_get_euroCode (piece);
	gchar_query_piece_aux = g_strconcat(gchar_query_piece, temp,
					    "', ", NULL);
        g_free(temp);
	g_free(gchar_query_piece);
	gchar_query_piece = gchar_query_piece_aux;
	gchar_query_piece_aux = NULL;

	temp = m_model_currval_sequence ("seqProducto", connection);
	gchar_query_piece_aux = g_strconcat(gchar_query_piece, temp,
					    ", '",
					    NULL);
        g_free(temp);
	g_free(gchar_query_piece);
	gchar_query_piece = gchar_query_piece_aux;
	gchar_query_piece_aux = NULL;

	temp = g_strdup_printf("%d",d_piece_get_length (piece));
	gchar_query_piece_aux = g_strconcat(gchar_query_piece, temp,
					  "', '", NULL);
	g_free(temp);
	g_free(gchar_query_piece);
	gchar_query_piece = gchar_query_piece_aux;
        gchar_query_piece_aux = NULL;

        temp = g_strdup_printf("%d",d_piece_get_width (piece));
	gchar_query_piece_aux = g_strconcat(gchar_query_piece, temp,
					  "', '", NULL);
	g_free(temp);
	g_free(gchar_query_piece);
	gchar_query_piece = gchar_query_piece_aux;
        gchar_query_piece_aux = NULL;

        temp = g_strdup_printf("%.2f",d_piece_get_cost_price (piece)*1.0);
        u_commas_to_points (temp);
	gchar_query_piece_aux = g_strconcat(gchar_query_piece, temp,
					  "', ", NULL);
        g_free(temp);
	g_free(gchar_query_piece);
	gchar_query_piece = gchar_query_piece_aux;
        gchar_query_piece_aux = NULL;

	if (d_piece_get_units_per_packet (piece)!=G_MINDOUBLE) {
	    temp = g_strdup_printf("'%.2f'",d_piece_get_units_per_packet (piece));
            u_commas_to_points (temp);
	} else {
            temp = g_strdup("NULL");
	}
	gchar_query_piece_aux = g_strconcat(gchar_query_piece, temp,
					  ", ", NULL);
	g_free(temp);
	g_free(gchar_query_piece);
	gchar_query_piece = gchar_query_piece_aux;
        gchar_query_piece_aux = NULL;

	if ((d_piece_get_meassure_code (piece)!=0) &&
	   (d_piece_get_meassure_code (piece)!=ULONG_MAX)) {
	    temp = g_strdup_printf("'%lu'",d_piece_get_meassure_code (piece));
	} else {
            temp = g_strdup("NULL");
	}
	gchar_query_piece_aux = g_strconcat(gchar_query_piece, temp,
					  ") ", NULL);
	g_free(temp);
	g_free(gchar_query_piece);
	gchar_query_piece = gchar_query_piece_aux;
        gchar_query_piece_aux = NULL;

	g_print(gchar_query_piece);
	result = result && m_model_command_execute (&record_set, gchar_query_piece, connection,command,tuples,0,FALSE);

	result = m_model_execute_mutex_modification_logged (connection) && result;

	g_free(gchar_query_piece);
	g_free(tuples);

	gda_recordset_free(record_set);
	igalia_command_free(command);
    }

    return result;
}

/**
 * m_piece_update: 
 * @piece: Pieza que se modificar en la base de datos.
 *
 * Modifica una pieza en la base de datos.
 *
 * Returns:
 * %TRUE si se ha modificado correctamente, %FALSE en otro caso.
 */
gboolean m_piece_update (Piece *piece) {
    gboolean result = FALSE;

    if (d_product_get_is_piece ((Product *)piece)) {
	gchar         *gchar_query_piece;
	gchar         *gchar_query_piece_aux;
	GdaRecordset  *record_set = NULL;
	gchar         *temp;
	gulong        *tuples = (gulong *) g_malloc0(sizeof(gulong));
	GdaConnection *connection = m_model_get_connection ();
	GdaCommand    *command = gda_command_new();
	gulong logged_shop_code;
	gulong logged_enterprise_code;
	gulong logged_warehouse_code;
	gchar * temp_name;

	temp_name = u_parameter_manager_get_parameter ("empresa");
	logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
	g_free(temp_name);
	temp_name = u_parameter_manager_get_parameter ("tienda");
	logged_shop_code = m_shop_get_code_from_name (temp_name);
	g_free(temp_name);
	temp_name = u_parameter_manager_get_parameter ("almacen");
	logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

	g_free(temp_name);

	gda_command_set_connection(command,connection);
	m_model_connection_begin_mutex_transaction_logged (connection,logged_enterprise_code,logged_shop_code,logged_warehouse_code);

	result = m_product_update_in_transaction ((Product *)piece, connection, command,FALSE);

	gchar_query_piece = "";
	gchar_query_piece = g_strconcat("UPDATE ",m_piece_get_table (),
					" SET ", NULL);

        temp = d_piece_get_euroCode (piece);
	gchar_query_piece_aux = g_strconcat(gchar_query_piece,
					      " eurocode = '",
                                              temp,
					      "', ", NULL);
        g_free(temp);
	g_free(gchar_query_piece);
	gchar_query_piece = gchar_query_piece_aux;
        gchar_query_piece_aux = NULL;

        temp = g_strdup_printf("%d",d_piece_get_length (piece));
	gchar_query_piece_aux = g_strconcat(gchar_query_piece,
					    " largo = '", temp,
					  "', ", NULL);
	g_free(temp);
	g_free(gchar_query_piece);
	gchar_query_piece = gchar_query_piece_aux;
        gchar_query_piece_aux = NULL;

        temp = g_strdup_printf("%d",d_piece_get_width (piece));
	gchar_query_piece_aux = g_strconcat(gchar_query_piece,
					    " ancho = '",temp,
					    "', ", NULL);
	g_free(temp);
	g_free(gchar_query_piece);
	gchar_query_piece = gchar_query_piece_aux;
        gchar_query_piece_aux = NULL;

        temp = g_strdup_printf("%.2f",d_piece_get_cost_price (piece)*1.0);
        u_commas_to_points (temp);
	gchar_query_piece_aux = g_strconcat(gchar_query_piece,
					    "preciocoste = '", temp,
					    "'  ", NULL);
        g_free(temp);
	g_free(gchar_query_piece);
	gchar_query_piece = gchar_query_piece_aux;
        gchar_query_piece_aux = NULL;

	if ((d_piece_get_meassure_code (piece)!=0) &&
	    (d_piece_get_meassure_code (piece)!=ULONG_MAX)) {
	    temp = g_strdup_printf("'%lu'",d_piece_get_meassure_code (piece));
	} else {
            temp = g_strdup("NULL");
	}
	gchar_query_piece_aux = g_strconcat(gchar_query_piece,
					    ", codigoMedida = ",temp,
					    " ", NULL);
	g_free(temp);
	g_free(gchar_query_piece);
	gchar_query_piece = gchar_query_piece_aux;
	gchar_query_piece_aux = NULL;


	if (d_piece_get_units_per_packet (piece)!=G_MINDOUBLE) {
	    temp = g_strdup_printf("'%.2f'",d_piece_get_units_per_packet (piece));
	    u_commas_to_points (temp);
	} else {
	    temp = g_strdup("NULL");
	}
	gchar_query_piece_aux = g_strconcat(gchar_query_piece,
					    ", unidadesPorPaquete = ",temp,
					    " ", NULL);
	g_free(temp);
	g_free(gchar_query_piece);
	gchar_query_piece = gchar_query_piece_aux;
	gchar_query_piece_aux = NULL;


	temp = g_strdup_printf("%lu",d_product_get_product_code ((Product *)piece));
	gchar_query_piece_aux = g_strconcat(gchar_query_piece,
					      " WHERE codigoProducto = '",temp,
					      "' ", NULL);
        g_free(temp);
	g_free(gchar_query_piece);
	gchar_query_piece = gchar_query_piece_aux;
	gchar_query_piece_aux = NULL;

        g_print(gchar_query_piece);
	result = result && m_model_command_execute (&record_set, gchar_query_piece, connection,command,tuples,0,FALSE);

	result = m_model_execute_mutex_modification_logged (connection) && result;

	g_free(gchar_query_piece);
	g_free(tuples);

	gda_recordset_free(record_set);
	igalia_command_free(command);
    }

    return result;
}

/**
 * m_piece_delete:
 * @piece: Pieza que se borrar de la base de datos.
 *
 * Borra una pieza de la base de datos.
 *
 * Returns:
 * %TRUE si se ha borrado correctamente, %FALSE en otro caso.
 */
gboolean m_piece_delete (Piece *piece) {
  gboolean result = FALSE;
  gulong logged_shop_code;
  gulong logged_enterprise_code;
  gulong logged_warehouse_code;
  gchar * temp_name;

  temp_name = u_parameter_manager_get_parameter ("empresa");
  logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
  g_free(temp_name);
  temp_name = u_parameter_manager_get_parameter ("tienda");
  logged_shop_code = m_shop_get_code_from_name (temp_name);
  g_free(temp_name);
  temp_name = u_parameter_manager_get_parameter ("almacen");
  logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

  g_free(temp_name);
  
    if (d_product_get_is_piece ((Product *)piece)) {
	GdaConnection *connection = m_model_get_connection ();
	GdaCommand    *command = gda_command_new();

	gda_command_set_connection(command,connection);
	m_model_connection_begin_mutex_transaction_logged (connection,logged_enterprise_code,logged_shop_code,logged_warehouse_code);

	result = m_product_delete_in_transaction ((Product *)piece, connection, command);

	result = m_model_execute_mutex_modification_logged (connection) && result;

	igalia_command_free(command);
    }

    return result;
}

/**
 * :
 * @product_code: Cdigo de la pieza para la que se buscar el proveedor asignado.
 *
 * Busca el proveedor asignado a la pieza.
 *
 * Returns:
 * Cdigo del proveedor asociado.
 */
gulong m_piece_find_supplier_assigned (gint product_code) {
    GdaCommand         * my_command;
    GdaConnection      * my_connection = m_model_get_connection ();
    GdaRecordset       * record_set = NULL;
    gulong result = ULONG_MAX;
    gchar * gchar_query;
    gchar * aux;
    gulong * tuples;
    gint * column;

    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);
    aux = g_strdup_printf("%d",product_code);

    gchar_query = g_strconcat("SELECT ",
			      " codigoProveedor as codigoProveedor",
			      " FROM pieza WHERE ",
			      "(pieza.CODIGOproducto=",aux,")",NULL);



    g_free(aux);

    g_print(gchar_query);
    tuples = (gulong *) g_malloc0(sizeof(gulong));
    m_model_command_execute (&record_set, gchar_query, my_connection,my_command,tuples,0, TRUE);
    g_free(gchar_query);


    column = (gint *) g_malloc0(sizeof(gint));
    (*column)=0;
    if (record_set!=NULL) {
	if (*tuples!=0) {
            GdaField * field = NULL;
	    gda_recordset_move_first(record_set);
	    field = gda_recordset_field_name(record_set,"codigoProveedor");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    result = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }
	}
    }
 
    igalia_command_free(my_command);
    gda_recordset_free(record_set);

    g_print(_("\nCommand and recordset freeing completed\n"));
    g_free(tuples);
    g_free(column);
    return result;

}
