 /* received_book_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "received_book_model.h"

/**
 * m_received_book_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos. El objeto #ReceivedBook
 * representa una tupla del libro de facturas recibidas que se encuadra dentro del Libro de Iva.
 * Las tuplas representan la agrupacin de las lneas de las facturas segn el tipo de Iva.
 *
 * Returns:
 * Un objeto #ReceivedBook con los campos rellenados con los datos de la base de datos.
 */
ReceivedBook * m_received_book_create (GdaRecordset *rs) {

    GdaField * field;
    ReceivedBook * received_book;

    received_book = d_received_book_create_empty();



    field = gda_recordset_field_name(rs,"numeroLibro" );
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
            received_book->number = igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoProveedorLibro" );
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
            received_book->supplier_code = igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }


    field = gda_recordset_field_name(rs,"numeroFacturaLibro" );
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp_number=gda_stringify_value(NULL, 0, field);
	    received_book->invoice_number = g_strdup(temp_number);
	    g_free(temp_number);
	}
	gda_field_free(field);
    }


    field = gda_recordset_field_name(rs,"fechaEmisionLibro");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp;
	    time_t * my_time;
	    temp=gda_stringify_value(NULL, 0, field);
	    my_time = u_parse_date (temp);
	    received_book->issue_date = (*my_time);
	    my_time = NULL;
	    g_free(temp);
	}
	gda_field_free(field);
    }


    field = gda_recordset_field_name(rs, "razonSocialLibro");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp=gda_stringify_value(NULL, 0, field);
	    received_book->trade_name = g_strdup(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }



    field = gda_recordset_field_name(rs, "tipoIvaLibro");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp=gda_stringify_value(NULL, 0, field);
	    received_book->vat_type = atof(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs, "totalLibro");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp=gda_stringify_value(NULL, 0, field);
	    received_book->total = atof(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }
    field = gda_recordset_field_name(rs, "subtotalLibro");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp=gda_stringify_value(NULL, 0, field);
	    received_book->line_subtotal = atof(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }

    return received_book;
}


/**
 * m_received_books_find_by_condition:
 * @condition: Condicin de bsqueda, permite buscar por perodo de tiempo y por cdigo de empresa.
 * @tuples: Nmero de tuplas que cumplen la condicin.
 *
 * Busca las facturas recibidas de una empresa desglosadas por tipos de iva de las lneas de la factura, segn exige
 * el libro de iva.
 *
 * Returns:
 * Array con las factura recibidas si se han encotrado, %NULL en otro caso.
 */
ReceivedBook **  m_received_books_find_by_condition (ConditionAccountingPeriod *condition,gulong * tuples) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar              *gchar_where_enterprise=NULL;
    gchar              *gchar_where=NULL;
    gchar              *gchar_from=NULL;
    gchar              *gchar_groupby=NULL;
    gchar              *gchar_order=NULL;
    gchar              *gchar_where_aux=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    gchar * enterprise_code = FALSE;
    ReceivedBook ** received_books;
    ReceivedBook * received_book;
    gint index;

    my_command = gda_command_new();


    gda_command_set_connection(my_command,my_connection);

    if ((condition->enterprise_code!=ULONG_MAX) && (condition->enterprise_code!=0))
        enterprise_code = g_strdup_printf("%lu",condition->enterprise_code);

    if (((enterprise_code) != NULL) && ((enterprise_code)!=0)) {
	gchar_where_enterprise = g_strconcat(" and (facturaCargo.codigoEmpresa=",
					  enterprise_code,")",NULL);
    } else {
	gchar_where_enterprise = g_strdup(" ");
    }

    gchar_select=g_strconcat("SELECT facturas.numeroFactura as numeroFacturaLibro, ",
			     "Facturas.fechaEmision as fechaEmisionLibro, ",
			     "facturas.nickProveedor as razonSocialLibro, ",
			     "facturas.codigoProveedor as codigoProveedorLibro, ",
			     "facturas.totalFactura as totalLibro, ",
			     "to_char(LineaFacturaCargo.tipoIva,'99999999D99') as tipoIvaLibro, ",
			     "to_char(SUM(LineaFacturaCargo.total),'99999999D99') as subtotalLibro ",NULL);


    gchar_from= g_strconcat(" FROM LineaFacturaCargo,(SELECT numeroFactura,codigoFactura,fecha as fechaEmision,",
			    "nick as nickProveedor,codigoProveedor,total as totalFactura,facturacargo.codigoEmpresa as codigoEmpresa from ",
			    "facturacargo,participante  where facturacargo.codigoProveedor = participante.codigoPArty ",
			    gchar_where_enterprise,") as facturas "
			    ,NULL);

    gchar_where=g_strconcat(" WHERE ",
			    " LineaFacturaCargo.numeroFactura=facturas.codigoFactura and ",
			    " LineaFacturaCargo.codigoProveedor=facturas.codigoProveedor ",
			    NULL);

    if (((enterprise_code) != NULL) && (strlen(enterprise_code)!=0)) {
	gchar_where_aux = g_strconcat(gchar_where, " and (facturas.codigoEmpresa=",
				      enterprise_code,")",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }
    if ((condition->initial_date) != NULL) {
	gchar_where_aux = g_strconcat(gchar_where, " and (facturas.fechaEmision>='",
				      condition->initial_date,"')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }
    if ((condition->ending_date) != NULL) {
	gchar_where_aux = g_strconcat(gchar_where, " and (facturas.fechaEmision<='",
				      condition->ending_date,"')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }


    gchar_order = g_strconcat(" order by FECHAEMISIONLIBRO,numeroFACTURALIBRO  ",NULL);
                                       
    gchar_groupby= g_strconcat(" group by  numeroFacturaLibro,"
			       " FECHAEMISIONLIBRO, razonsocialLIBRO, codigoProveedorLibro,"
			       " totalLibro, "
			       " tipoIvaLibro ",NULL);
    gchar_query = g_strconcat(gchar_select,gchar_from, gchar_where, gchar_groupby,gchar_order,NULL);

    g_print(_("\nQuery: %s\n"),gchar_query);

    m_model_command_execute (&record_set, gchar_query, my_connection,
			  my_command,tuples,0, TRUE);


    if (record_set!=NULL) {
	if (*tuples!=0) {

	    received_books = (ReceivedBook **) g_malloc0(sizeof(ReceivedBook*)*(*tuples+1));
	    index = gda_recordset_move_first(record_set);
	    while (!gda_recordset_eof(record_set)) {
		if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		received_book = m_received_book_create (record_set);
		received_books[index-1] = received_book;
		    index = gda_recordset_move_next(record_set);
	    }
	    received_books[*tuples]=NULL;
	} else {
		received_books=NULL;
	}
	gda_recordset_close(record_set);
	gda_recordset_free(record_set);
    } else {
	received_books=NULL;
    }

    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    g_free(gchar_where);
    g_free(gchar_where_enterprise);
    g_free(gchar_select);
    g_free(gchar_query);
    g_free(gchar_order);
    g_free(gchar_groupby);
    return received_books;
    
}

/**
 * m_received_books_group_by_vat:
 * @condition: Condicin de bsqueda, permite buscar por perodo de tiempo y por cdigo de empresa.
 * @tuples: Nmero de tuplas que cumplen la condicin.
 *
 * Busca las facturas recibidas de una empresa agrupadas por tipos de iva de todas las lneas de las facturas, segn exige
 * el libro de iva.
 *
 * Returns:
 * Array con las factura recibidas si se han encotrado, %NULL en otro caso.
 */
ReceivedBook **  m_received_books_group_by_vat (ConditionAccountingPeriod *condition,gulong * tuples) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar              *gchar_where_enterprise=NULL;
    gchar              *gchar_where=NULL;
    gchar              *gchar_from=NULL;
    gchar              *gchar_groupby=NULL;
    gchar              *gchar_order=NULL;
    gchar              *gchar_where_aux=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    gchar * enterprise_code = FALSE;
    ReceivedBook ** received_books;
    ReceivedBook * received_book;
    gint index;

    my_command = gda_command_new();


    gda_command_set_connection(my_command,my_connection);

    if ((condition->enterprise_code!=ULONG_MAX) && (condition->enterprise_code!=0))
        enterprise_code = g_strdup_printf("%lu",condition->enterprise_code);

    if (((enterprise_code) != NULL) && ((enterprise_code)!=0)) {
	gchar_where_enterprise = g_strconcat(" and (facturaCargo.codigoEmpresa=",
					  enterprise_code,")",NULL);
    } else {
	gchar_where_enterprise = g_strdup(" ");
    }

    gchar_select=g_strconcat("SELECT to_char(tipoIvaL,'99999999D99') as tipoIvaLibro, "
			     " to_char(SUM(subtotalL),'99999999D99') as subtotalLibro, ",
                             " to_char(SUM(subtotalL*(1+(tipoIvaL/100))),'99999999D99') as totalLibro "
			     " FROM (SELECT facturas.numeroFactura as numeroFacturaL, ",
			     "Facturas.fechaEmision as fechaEmisionL, ",
			     "facturas.nickProveedor as razonSocialL, ",
			     "facturas.codigoProveedor as codigoProveedorL, ",
			     "facturas.totalFactura as totalL, ",
			     " LineaFacturaCargo.tipoIva as tipoIvaL, ",
			     " SUM(LineaFacturaCargo.total) as subtotalL ",NULL);


    gchar_from= g_strconcat(" FROM LineaFacturaCargo,(SELECT numeroFactura,codigoFactura,fecha as fechaEmision,",
			    "nick as nickProveedor,codigoProveedor,total as totalFactura,facturacargo.codigoEmpresa as codigoEmpresa from ",
			    "facturacargo,participante  where facturacargo.codigoProveedor = participante.codigoPArty ",
			    gchar_where_enterprise,") as facturas "
			    ,NULL);

    gchar_where=g_strconcat(" WHERE ",
			    " LineaFacturaCargo.numeroFactura=facturas.codigoFactura and ",
			    " LineaFacturaCargo.codigoProveedor=facturas.codigoProveedor ",
			    NULL);

    if (((enterprise_code) != NULL) && (strlen(enterprise_code)!=0)) {
	gchar_where_aux = g_strconcat(gchar_where, " and (facturas.codigoEmpresa=",
				      enterprise_code,")",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }
    if ((condition->initial_date) != NULL) {
	gchar_where_aux = g_strconcat(gchar_where, " and (facturas.fechaEmision>='",
				      condition->initial_date,"')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }
    if ((condition->ending_date) != NULL) {
	gchar_where_aux = g_strconcat(gchar_where, " and (facturas.fechaEmision<='",
				      condition->ending_date,"')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }


    gchar_order = g_strconcat(" order by FECHAEMISIONL,numeroFACTURAL) as Libro group by tipoIvaL  ",NULL);
                                       
    gchar_groupby= g_strconcat(" group by  numeroFacturaL,"
			       " FECHAEMISIONL, razonsocialL, codigoProveedorL,"
			       " totalL, "
			       " tipoIvaL ",NULL);
    gchar_query = g_strconcat(gchar_select,gchar_from, gchar_where, gchar_groupby,gchar_order,NULL);

    g_print(_("\nQuery: %s\n"),gchar_query);

    m_model_command_execute (&record_set, gchar_query, my_connection,
			  my_command,tuples,0, TRUE);


    if (record_set!=NULL) {
	if (*tuples!=0) {

	    received_books = (ReceivedBook **) g_malloc0(sizeof(ReceivedBook*)*(*tuples+1));
	    index = gda_recordset_move_first(record_set);
	    while (!gda_recordset_eof(record_set)) {
		if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		received_book = m_received_book_create (record_set);
                d_received_book_set_vat_summary (received_book,TRUE);
		received_books[index-1] = received_book;
		    index = gda_recordset_move_next(record_set);
	    }
	    received_books[*tuples]=NULL;
	} else {
		received_books=NULL;
	}
	gda_recordset_close(record_set);
	gda_recordset_free(record_set);
    } else {
	received_books=NULL;
    }

    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    g_free(gchar_where);
    g_free(gchar_where_enterprise);
    g_free(gchar_select);
    g_free(gchar_query);
    g_free(gchar_order);
    g_free(gchar_groupby);
    return received_books;
    
}


/**
 * m_received_books_clear_array:
 * @received_books: Lista de facturas recibidas a liberar.
 *
 * Libera la memoria utilizada por un array de objetos #ReceivedBook.
 *
 */
void m_received_books_clear_array (ReceivedBook ** received_books) {
    if (received_books!=NULL) {
	gint index = 0;
	while (received_books[index]!=NULL) {
	    d_received_book_clear (received_books[index]);
	    index++;
	}
    }
    g_free(received_books);

}

/**
 * m_received_books_set_receipt:
 * @received_books: Lista de facturas recibidas.
 * @accounting_period: Perdo de contabilidad en el que se actuar.
 * 
 * Establece el orden de recepcin de las facturas recibidas, cuando se desee cerrar un perodo de contabilidad.
 * Aunque las facturas tienen un orden de recepcin, se podran recebir facturas emitidas con anterioridad a otras
 * que ya estn en la BD, para ello, se comprueba el order de emisin de las mismas por ltima vez antes de
 * cerrar la contabilidad para un perodo e impedir as que ms tarde se puedan introducir facturas con fecha
 * de emisin anterior.
 *
 * Returns:
 * %TRUE si se ha realizado correctamente, %FALSE en otro caso.
 * 
 */
gboolean m_received_books_set_receipt (ReceivedBook ** received_books,AccountingPeriod * accounting_period) {
    gboolean result=0;
    gchar * query=NULL;
    gchar * received_number=NULL;
    gchar * supplier_code;
    gchar * invoice_number;
    gchar * enterprise=NULL;
    gulong enterprise_code = ULONG_MAX;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    gulong logged_shop_code;
    gulong logged_warehouse_code;
    gboolean can_insert=TRUE;
    gchar * temp_name;
    gulong index=0;
    temp_name = u_parameter_manager_get_parameter ("tienda");
    logged_shop_code = m_shop_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("almacen");
    logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

    g_free(temp_name);

    enterprise_code=d_accounting_get_enterprise_code (accounting_period);

    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,enterprise_code,logged_shop_code,logged_warehouse_code);

    enterprise = g_strdup_printf("%lu",enterprise_code);

    if (m_accounting_period_exists (accounting_period)==FALSE) {
        can_insert=FALSE;
    }
    if ((can_insert!=FALSE) && (m_accounting_period_is_closed (accounting_period)==TRUE)) {
        can_insert=FALSE;
    }
    if ((can_insert!=FALSE) && (m_received_books_exists_charge_invoice (received_books,accounting_period)==TRUE)) {
        can_insert=FALSE;
    }

    if ((can_insert!=FALSE) && (received_books!=NULL)) {
	while (received_books[index]!=NULL) {
	    if ((d_received_book_get_total_summary (received_books[index])==FALSE) && (d_received_book_get_vat_summary (received_books[index])==FALSE)) {
		invoice_number = d_received_book_get_invoice_number (received_books[index]);
		supplier_code = g_strdup_printf("%lu",d_received_book_get_supplier_code (received_books[index]));
		received_number = g_strdup_printf("%lu",d_received_book_get_number (received_books[index]));


		query = g_strconcat(" update facturacargo set numerorecepcion = ",received_number,
				    " where (numeroFactura='",invoice_number,"'",
				    " and codigoProveedor=",supplier_code,
				    " and codigoEmpresa=",enterprise,")",
				    NULL);
		g_print(_("\nUpdate: %s\n"),query);
		if (m_model_command_execute (&record_set, query, connection, command,tuples,0, FALSE)==FALSE) {
		    result = FALSE;
		} else {
		    result = TRUE;
		}

		g_free(received_number);
		g_free(invoice_number);
		g_free(supplier_code);
		g_free(query);
	    }
	    index++;
	}
    }
    if (record_set!=NULL)
	gda_recordset_free(record_set);

    if (result!=FALSE)
	result = m_accounting_period_close (d_accounting_get_accounting_code (accounting_period),enterprise_code,connection,command);

    if (m_model_execute_mutex_modification_logged (connection)!=TRUE) {
	result = FALSE;
    }


    g_free(tuples);
    g_free(enterprise);




    igalia_command_free(command);

    return result;
}

/**
 * m_received_books_exists_charge_invoice:
 * @received_books: Lista de facturas recibidas a comprobar.
 * @accounting_period: Perodo de contabilidad para el que semprobar la condicin.
 *
 * Comprueba si en el array de facturas recibidas existe alguna que ya pertenezca a un perodo de contabilidad
 * cerrado y con un nmero de rececin en la BD, para impedir que se sobreescriba.
 *
 * Returns:
 * %TRUE si existe alguna que cumpla la condicin, %FALSE en otro caso.
 */
gboolean m_received_books_exists_charge_invoice (ReceivedBook ** received_books,AccountingPeriod * accounting_period) {
    gboolean result=0;
    gchar * query=NULL;
    gchar * received_number=NULL;
    gchar * supplier_code;
    gchar * invoice_number;
    gchar * enterprise=NULL;
    gulong enterprise_code = ULONG_MAX;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    gulong index=0;


    enterprise_code=d_accounting_get_enterprise_code (accounting_period);

    gda_command_set_connection(command,connection);

    enterprise = g_strdup_printf("%lu",enterprise_code);

    if (received_books!=NULL) {
	while (received_books[index]!=NULL) {
	    invoice_number = NULL;
	    supplier_code = g_strdup_printf("%lu",d_received_book_get_supplier_code (received_books[index]));
	    received_number = NULL;

	    if (d_received_book_get_supplier_code (received_books[index])!=ULONG_MAX) {
		invoice_number = d_received_book_get_invoice_number (received_books[index]);
                received_number = g_strdup_printf("%lu",d_received_book_get_number (received_books[index]));
		query = g_strconcat(" select numeroRecepcion from facturaCargo "
				    " where (numeroFactura='",invoice_number,"'",
				    " and codigoProveedor=",supplier_code,
				    " and codigoEmpresa=",enterprise,")",
				    NULL);
		g_print(_("Query: %s\n"),query);
		if (m_model_command_execute (&record_set, query, connection, command,tuples,0, TRUE)==FALSE) {
		    result = TRUE;
		} else {
		    GdaField * field;
		    gda_recordset_move_first(record_set);
		    field = gda_recordset_field_idx(record_set,0);
		    if (!gda_field_is_null(field)) {
			result=TRUE;
		    }
		}
		g_free(query);
	    }

            if (received_number!=NULL)
		g_free(received_number);
            if (invoice_number!=NULL)
		g_free(invoice_number);
            g_free(supplier_code);

	    

            index++;
	}
    }
    if (record_set!=NULL)
	gda_recordset_free(record_set);




    g_free(tuples);
    g_free(enterprise);


    igalia_command_free(command);

    return result;
}

