/* repair_movement_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "repair_movement_model.h"
#include "find_pieces_model.h"
#include "stock_model.h"
#include "piece.h"
#include "stock.h"

#define M_FIRST_ERROR_MESSAGE N_("Stock update: There's no enough stock.")
#define M_WARNING_MESSAGE N_("Piece stock %s is bellow the minimum.")

#define TABLE      "MovimientoPorAlbaran"


/**
 * m_repair_movement_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_repair_movement_get_table () {
    return TABLE;
}

/**
 * m_repair_movement_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #RepairMovement con los campos rellenados con los datos de la base de datos.
 */
RepairMovement  *m_repair_movement_create (GdaRecordset *rs){
    RepairMovement * result=d_repair_movement_create_empty ();
    gchar  *temp;
    GdaField * field = NULL;
    

    field = gda_recordset_field_name(rs,"codigoempresaMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result->enterprise_code = igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoalmacenMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result->warehouse_code = igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result->movement_code = igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"unidadesMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result->quantity = u_round_to_decimal (igalia_field_get_double_value (field),3);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"fechaMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    time_t * my_time;
	    temp=gda_stringify_value(NULL, 0, field);
	    my_time = u_parse_date (temp);
	    result->date = (*my_time);
	    g_free(my_time);
	    my_time = NULL;
	    g_free(temp);
	}
	gda_field_free(field);
    }


    field = gda_recordset_field_name(rs,"codigoReparacionMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result-> repair_code= igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoEmpresaRepaMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result-> repair_enterprise_code=    igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"numeroLineaMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result-> line_number=     igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoTiendaMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result-> shop_code=igalia_field_get_bigint_value (field)  ;
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoProductoMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result-> product_code=igalia_field_get_bigint_value (field)  ;
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs, "matriculaMovimiento");
    if (field!=NULL){
	if (!gda_field_is_null(field)) {
	    temp = gda_stringify_value(NULL, 0, field);
	    result->license_number = g_strdup(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }


    return result;

}


/**
 * m_repair_movement_insert:
 * @repair_line: Lnea de la reparacin para la que se insertar un movimiento.
 * @repair_code: Cdigo de la reparacin a la que pertenece la lnea de reparacin.
 * @enterprise_code: Cdigo de la empresa en la que se realiz la reparacin.
 * @shop_code: Cdigo de tienda en la que se realiz la reparacin.
 * @update: Indica si se modifica o se inserta el movimiento.
 * @command: Comando en el que se ejecuta una sentencia sql.
 * @connection: Conexin a la base de datos.
 * @exception: Objeto que indica el tipo de error que se produce.
 *
 * Inserta un movimiento por reparacin en la base de datos. Este movimiento indica un movmiento de stock
 * debido a la insercin o modificacin de una lnea de reparacin.
 *
 * Returns:
 * %TRUE si se ha realizado la insercin, %FALSE en otro caso.
 */
gboolean m_repair_movement_insert (Repair_line *repair_line, gchar *repair_code, gchar *enterprise_code, gchar *shop_code, gboolean update, GdaCommand * command, GdaConnection *connection, Exception *exception) {
 
  gboolean result = FALSE;

  if ((repair_line != NULL)&&(command!=NULL)) {
    gchar * query = NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set = NULL;
    gulong line_number_gulong = d_repair_line_get_line_number (repair_line);
    gchar *line_number = NULL;
    gdouble quantity_gulong = u_round_to_decimal (d_repair_line_get_quantity (repair_line),3);
    gdouble previous_quantity_gulong = u_round_to_decimal (d_repair_line_get_earlier_quantity (repair_line),3);
    gchar *quantity = NULL;
    Stock *stock = d_repair_line_get_stock (repair_line);
    Piece *piece = (Piece *) d_repair_line_get_product (repair_line);
    gulong enterprise_movement_code_gulong = d_stock_get_enterprise_code (stock);
    gchar *enterprise_movement_code = NULL;
    gulong warehouse_code_gulong = d_stock_get_warehouse_code (stock);
    gchar *warehouse_code = NULL;
//    gdouble previous_quantity_stock_gulong = u_round_to_decimal (d_stock_get_previous_quantity (stock),3);
    gdouble quantity_stock_gulong = u_round_to_decimal (d_stock_get_quantity (stock),3);
    gchar *quantity_stock = NULL;
    gdouble minimun_quantity_stock_gulong = u_round_to_decimal (d_stock_get_minimum_stock (stock),3);
    Product product = d_piece_get_product (piece);
    gulong product_code = d_product_get_product_code (&product);
    gchar * product_code_char=NULL;
    time_t now_time = time(NULL);
    gchar *now = g_strconcat("'",ctime(&now_time),"'",NULL);

    if (previous_quantity_gulong == G_MINDOUBLE) previous_quantity_gulong = 0.0;

    if (enterprise_movement_code_gulong!=ULONG_MAX)
      enterprise_movement_code = g_strdup_printf("%lu",enterprise_movement_code_gulong);
    else enterprise_movement_code = g_strdup("NULL");
    
    if (warehouse_code_gulong!=ULONG_MAX)
      warehouse_code = g_strdup_printf("%lu",warehouse_code_gulong);
    else warehouse_code = g_strdup("NULL");
           
    if (line_number_gulong!=ULONG_MAX)
      line_number = g_strdup_printf("%lu",line_number_gulong);
    else line_number = g_strdup("NULL");

    if (quantity_gulong!=G_MINDOUBLE)
      quantity = g_strdup_printf("%.3f",quantity_gulong);
    else quantity = g_strdup("NULL");
    u_commas_to_points (quantity);

    if (product_code!=ULONG_MAX)
	product_code_char = g_strdup_printf("%lu",product_code);
    else
	product_code_char = g_strdup("NULL");

    if (update) {
      query = g_strconcat("UPDATE movimientoporreparacion SET ",
			  " unidades = ",quantity,
			  " WHERE codigoreparacion = ",repair_code,
			  " AND codigoempresareparacion = ",enterprise_code,
			  " AND codigotienda = ",shop_code,
			  " AND numerolinea = ",line_number,
			  NULL);
      
      if (quantity_stock_gulong + previous_quantity_gulong >= quantity_gulong)
	{
	  d_stock_set_quantity (stock, quantity_stock_gulong + previous_quantity_gulong - quantity_gulong);
	  
	  if ((quantity_stock_gulong + previous_quantity_gulong - quantity_gulong)<=minimun_quantity_stock_gulong) {
	    gchar * desc = g_strstrip(d_piece_get_euroCode (piece));
	    gchar * warn_message = g_strdup_printf(_(M_WARNING_MESSAGE), desc);
 
	    d_exception_set_message (exception, warn_message);

	    g_free(desc);
	    g_free(warn_message);
	  }
	}
      else  {
	d_exception_set_code (exception, 1);
	d_exception_set_message (exception, _(M_FIRST_ERROR_MESSAGE));
	g_free(query);
	query = g_strdup("INSERT INTO movimientoporreparacion VALUES ()");
      }

    } else {
      gchar *temp_mov = m_model_nextval_sequence ("seqmovimientoalmacen",connection);
      gchar *temp_lin = m_model_currval_sequence ("seqlineareparacion",connection);

      query = g_strconcat("INSERT INTO movimientoporreparacion VALUES (",
			  enterprise_movement_code,",",
			  warehouse_code,",",
			  temp_mov,",",
			  now,",",
			  quantity,",",
			  repair_code,",",
			  enterprise_code,",",
			  shop_code,",",
			  temp_lin,",",
                          product_code_char,
			  ")",
			  NULL);
      
      g_free(temp_mov);
      g_free(temp_lin);

      if (quantity_stock_gulong >= quantity_gulong){

	d_stock_set_quantity (stock, quantity_stock_gulong - quantity_gulong);

	if (( quantity_stock_gulong - quantity_gulong)<=minimun_quantity_stock_gulong) {
	  gchar * desc = g_strstrip(d_piece_get_euroCode (piece));
	  gchar * warn_message = g_strdup_printf(_(M_WARNING_MESSAGE), desc);
	  
	  d_exception_set_message (exception, warn_message);
	  
	  g_free(desc);
	  g_free(warn_message);
	}	
      }
      else {
	d_exception_set_code (exception, 1);
	d_exception_set_message (exception, _(M_FIRST_ERROR_MESSAGE));
	g_free(query);
	query = g_strdup("INSERT INTO movimientoporreparacion VALUES ();");
      }
    }
    
    g_print(_("\nInsert: %s\n"),query);
    
    result = m_model_command_execute (&record_set, query, connection,
				   command,tuples,0,FALSE);

    d_stock_set_piece (stock, d_piece_duplicate ((Piece *)d_repair_line_get_product (repair_line)));

    result = result && m_stock_insert (stock, connection, command);
    
    if (record_set != NULL) gda_recordset_free(record_set);
        
    g_free(query);
    g_free(line_number);
    g_free(quantity);
    g_free(tuples);
    g_free(enterprise_movement_code);
    g_free(warehouse_code);
    g_free(product_code_char);
    g_free(quantity_stock);
    g_free(now);

  }
  
  return result;
}
