/* shop_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#define TABLE         "Tienda"
#define IDENTIFICATOR "codigoTienda"
#define ATTRIBUTES    "codigoEmpresa as codigoempresashop, \
                       codigoreparador as codigoreparadorshop, \
                       codigoPertenenciaGrupo as codigoPertenenciaGruposhop"

#include "shop_model.h"
#include "participant_model.h"
#include "util.h"



static Shop ** shops_list=NULL;
static gint length_of_list=0;

/**
 * m_shop_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_shop_get_table () {
    return TABLE;
}

/**
 * m_shop_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_shop_get_attributes () {
    return ATTRIBUTES;
}

/**
 * m_shop_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Shop con los campos rellenados con los datos de la base de datos.
 */
Shop *m_shop_create (GdaRecordset *rs) {
  gchar  *temp;
  GdaField * field;
  Shop * shop = d_shop_create_empty();

  shop->participant = m_participant_create (rs,"shop");

  if (d_participant_get_code (d_shop_get_participant (shop)) != ULONG_MAX) {
    field = gda_recordset_field_name(rs,"codigoempresashop");    
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    shop->enterprise_code = igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoPertenenciaGruposhop");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    temp=gda_stringify_value(NULL, 0, field);
	    shop->group_code = g_strdup(temp);
            g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoreparadorshop");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    temp=gda_stringify_value(NULL, 0, field);
	    shop->repairer_code = g_strdup(temp);
            g_free(temp);
	}
	gda_field_free(field);
    }
    
  } 

  return shop;
}


/**
 * m_shop_find_by_shop_code:
 * @shop_code: Cdigo de la tienda que se desea buscar.
 *
 * Busca una tienda que se identifique por el cdigo @shop_code
 *
 * Returns:
 * Tienda que cumple la condicin o %NULL en caso de error o no existencia de la misma.
 */
Shop * m_shop_find_by_shop_code (gint shop_code) {
    GdaCommand         * my_command;
    GdaConnection      * my_connection = m_model_get_connection ();
    GdaRecordset       * record_set = NULL;
    Shop * shop;
    gchar * gchar_query=NULL;
    gchar * aux=NULL;
    gchar * aux2=NULL;
    gulong * tuples;
    gint * column;

    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);
    aux = g_strdup_printf("%d",shop_code);

    aux2 = u_table_put_name (m_shop_get_attributes (),m_shop_get_table ());
     gchar_query = g_strconcat("SELECT ",
			       aux2,",",
			       " Participante.codigoParty as codigopartyparticipantshop, ",
			       " Participante.nombre as nombreparticipantshop, ",
			       " Participante.cif as cifparticipantshop, ",
			       " Participante.nick as nickparticipantshop, ",
			       " Participante.telefono as telefonoparticipantshop, ",
			       " Participante.telefono2 as telefono2participantshop, ",
			       " Participante.fax as faxparticipantshop, ",
			       " Participante.email as emailparticipantshop, ",
			       " Participante.calle as calleparticipantshop, ",
			       " Participante.numero as numeroparticipantshop, ",
			       " Participante.piso as pisoparticipantshop, ",
			       " Participante.CP as cpparticipantshop, ",
			       " Participante.municipio as municipioparticipantshop ",
			       " FROM TIENDA,PARTICIPANTE WHERE ",
			      "(PARTICIPANTE.CODIGOPARTY=TIENDA.CODIGOTIENDA) ",
			      "AND (TIENDA.CODIGOTIENDA=",aux,")",NULL);



    g_free(aux);
    g_free(aux2);

    tuples = (gulong *) g_malloc0(sizeof(gulong));
    m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);


    column = (gint *) g_malloc0(sizeof(gint));
    (*column)=0;
    if (record_set!=NULL) {
	if (*tuples!=0) {
	    gda_recordset_move_first(record_set);
	    shop = m_shop_create (record_set);
	} else {
            shop=NULL;
	}
        gda_recordset_close(record_set);
	gda_recordset_free(record_set);
    } else {
        shop=NULL;
    }

    igalia_command_free(my_command);
    g_free(gchar_query);

    g_free(tuples);
    g_free(column);
    return shop;
}

/**
 * m_shop_find_all:
 *
 * Busca todas las tiendas de la empresa. Si es la primera vez que se buscan las tienda se realiza
 * la bsqueda en la base de datos, en otro caso, devuelve la lista que est actualmente en memoria.
 *
 * Returns:
 * Array con todas las tiendas de la empresa.
 */
Shop ** m_shop_find_all () {
    GdaCommand         * my_command;
    GdaConnection      * my_connection = m_model_get_connection ();
    GdaRecordset       * record_set = NULL;
    Shop * shop;
    gchar * gchar_query;
    gchar *aux;
    gulong * tuples;
    gint * column;
    gint index;


    if (shops_list==NULL) {
	my_command = gda_command_new();
	gda_command_set_connection(my_command,my_connection);

        aux = u_table_put_name (m_shop_get_attributes (),m_shop_get_table ());
	gchar_query = g_strconcat("SELECT ",
				  aux,",",
				  " Participante.codigoParty as codigopartyparticipantshop, ",
				  " Participante.nombre as nombreparticipantshop, ",
				  " Participante.cif as cifparticipantshop, ",
				  " Participante.nick as nickparticipantshop, ",
				  " Participante.telefono as telefonoparticipantshop, ",
				  " Participante.telefono2 as telefono2participantshop, ",
				  " Participante.fax as faxparticipantshop, ",
				  " Participante.email as emailparticipantshop, ",
				  " Participante.calle as calleparticipantshop, ",
				  " Participante.numero as numeroparticipantshop, ",
				  " Participante.piso as pisoparticipantshop, ",
				  " Participante.CP as cpparticipantshop, ",
				  " Participante.municipio as municipioparticipantshop ",
				  " FROM ",m_shop_get_table (),",",m_participant_get_table (),
                                  " WHERE TIENDA.CODIGOTIENDA=PARTICIPANTE.CODIGOPARTY"
                                  ,NULL
				 );

        g_free(aux);
	tuples = (gulong *) g_malloc0(sizeof(gulong));
	m_model_command_execute (&record_set, gchar_query, my_connection,my_command,tuples,0, TRUE);
	g_free(gchar_query);


        length_of_list = *tuples;
	column = (gint *) g_malloc0(sizeof(gint));

	if (record_set!=NULL) {
	    if (*tuples!=0) {
		shops_list = (Shop **) g_malloc0(sizeof(Shop*)*(*tuples+1));
		index = gda_recordset_move_first(record_set);
		while (!gda_recordset_eof(record_set)) {
		    if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		    (*column)=0;
		    shop = m_shop_create (record_set);
		    shops_list[index-1] = shop;
		    index = gda_recordset_move_next(record_set);
		}
		shops_list[length_of_list]=NULL;

	    } else {
		shops_list=NULL;
	    }
	    gda_recordset_close(record_set);
	    gda_recordset_free(record_set);
	} else {
	    shops_list=NULL;
	}

	igalia_command_free(my_command);

	g_free(tuples);
	g_free(column);


    }
    return shops_list;
}

/**
 * m_shop_find_by_enterprise_code:
 * @enterprise_code: Cdigo de la empresa para laque se busca las tiendas.
 *
 * Busca las tiendas pertenecientes a una empresa del grupo. Si el cdigo de la empresa es 0
 * devuelve todas las tiendas.
 *
 * Returns:
 * Lista con las tiendas que pertenecen a una empresa del grupo. %NULL en caso de error o cdigo de
 * empresa incorrecto.
 */
Shop ** m_shop_find_by_enterprise_code (gint enterprise_code) {
    GdaCommand         * my_command;
    GdaConnection      * my_connection = m_model_get_connection ();
    GdaRecordset       * record_set = NULL;
    Shop * shop;
    Shop ** shops;
    gchar * gchar_query;
    gulong * tuples;
    gint * column;
    gint index;
    gchar * aux;
    gchar * aux2;


    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);
    aux = g_strdup_printf("%d",enterprise_code);
    aux2 = u_table_put_name (m_shop_get_attributes (),m_shop_get_table ());
    if (enterprise_code>0) {
	gchar_query = g_strconcat("SELECT ",
				  aux2,",",
				  " Participante.codigoParty as codigopartyparticipantshop, ",
				  " Participante.nombre as nombreparticipantshop, ",
				  " Participante.cif as cifparticipantshop, ",
				  " Participante.nick as nickparticipantshop, ",
				  " Participante.telefono as telefonoparticipantshop, ",
				  " Participante.telefono2 as telefono2participantshop, ",
				  " Participante.fax as faxparticipantshop, ",
				  " Participante.email as emailparticipantshop, ",
				  " Participante.calle as calleparticipantshop, ",
				  " Participante.numero as numeroparticipantshop, ",
				  " Participante.piso as pisoparticipantshop, ",
				  " Participante.CP as cpparticipantshop, ",
				  " Participante.municipio as municipioparticipantshop ",
				  " FROM ",m_shop_get_table (),",",m_participant_get_table (),
				  " WHERE (TIENDA.CODIGOTIENDA=PARTICIPANTE.CODIGOPARTY) "
				  " AND (TIENDA.CODIGOEMPRESA=",aux,")",
				  NULL
				 );
    } else {
	gchar_query = g_strconcat("SELECT ",
				  aux2,",",
				  " Participante.codigoParty as codigopartyparticipantshop, ",
				  " Participante.nombre as nombreparticipantshop, ",
				  " Participante.cif as cifparticipantshop, ",
				  " Participante.nick as nickparticipantshop, ",
				  " Participante.telefono as telefonoparticipantshop, ",
				  " Participante.telefono2 as telefono2participantshop, ",
				  " Participante.fax as faxparticipantshop, ",
				  " Participante.email as emailparticipantshop, ",
				  " Participante.calle as calleparticipantshop, ",
				  " Participante.numero as numeroparticipantshop, ",
				  " Participante.piso as pisoparticipantshop, ",
				  " Participante.CP as cpparticipantshop, ",
				  " Participante.municipio as municipioparticipantshop, ",
				  " Participante.nombrecorto as shortNAmeparticipantshop ",
				  " FROM ",m_shop_get_table (),",",m_participant_get_table (),
				  " WHERE (TIENDA.CODIGOTIENDA=PARTICIPANTE.CODIGOPARTY) ",
				  NULL
				 );
    }
    g_free(aux2);

    tuples = (gulong *) g_malloc0(sizeof(gulong));
    m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0,TRUE);
    g_free(gchar_query);

    column = (gint *) g_malloc0(sizeof(gint));

    if (record_set!=NULL) {
	if (*tuples!=0) {
	    shops = (Shop **) g_malloc0(sizeof(Shop*)*(*tuples+1));
	    index = gda_recordset_move_first(record_set);
	    while (!gda_recordset_eof(record_set)) {
		if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;

		shop = m_shop_create (record_set);
		shops[index-1] = shop;
		index = gda_recordset_move_next(record_set);
	    }
	    shops[*tuples]=NULL;

	} else {
	    shops=NULL;
	}
	gda_recordset_close(record_set);
	gda_recordset_free(record_set);
    } else {
	shops=NULL;
    }

    igalia_command_free(my_command);

    g_free(tuples);
    g_free(column);



    return shops;
}

/**
 * m_shop_get_code_from_name:
 * @name: Nombre de la tienda
 *
 * Devuelve el cdigo de la tienda a partir de nombre.
 *
 * Returns:
 * Cdigo de la tienda si el nombre coincide con el de una tienda del grupo, %ULONG_MAX en otro caso.
 */
gulong m_shop_get_code_from_name (gchar *name) {
    gulong code  = ULONG_MAX;
    gint   index = 0;
    gchar  *temp = NULL;

    if (shops_list==NULL) {
	m_shop_find_all ();
    }

    while(shops_list[index]!=NULL) {
        temp = d_participant_get_nick (d_shop_get_participant (shops_list[index]));
	if (!strcmp(name, temp)) {
	    code = d_participant_get_code (d_shop_get_participant (shops_list[index]));
	    g_free(temp);
            break;
	}
        g_free(temp);
        index++;
    }

    return code;
}

/**
 * m_shop_get_code_from_letter:
 * @letter: Letra de la tienda
 *
 * Devuelve el cdigo de la tienda a partir de la letra.
 *
 * Returns:
 * Cdigo de la tienda si la letra coincide con el de una tienda del grupo, %ULONG_MAX en otro caso.
 */
gulong m_shop_get_code_from_letter (gchar *name) {
    gulong code  = ULONG_MAX;
    gint   index = 0;
    gchar  *temp = NULL;

    if (shops_list==NULL) {
	m_shop_find_all ();
    }

    while(shops_list[index]!=NULL) {
	temp =d_shop_get_group_code (shops_list[index]);
	if (!strcmp(name, temp)) {
	    code = d_participant_get_code (d_shop_get_participant (shops_list[index]));
	    g_free(temp);
	    break;
	}
        g_free(temp);
        index++;
    }

    return code;
}

/**
 * m_shop_get_name_from_code:
 * @code: Cdigo de la tienda.
 *
 * Devuelve el nombre de la tienda que se identifica por @code.
 *
 * Returns:
 * Nombre de la tienda si el cdigo de tienda es correcto, %NULL en otro caso.
 */
gchar *m_shop_get_name_from_code (gulong code) {
    gchar *name = NULL;
    gint   index=0;

    if (shops_list==NULL) {
	m_shop_find_all ();
    }

    while(shops_list[index]!=NULL) {
	if (code == d_participant_get_code (d_shop_get_participant (shops_list[index]))) {
            name = d_participant_get_nick (d_shop_get_participant (shops_list[index]));
	}
        index++;
    }

    return name;
}


/**
 * m_shop_clear_array:
 * @shops: Array de tiendas que se desea liberar.
 *
 * Libera la memoria utilizada por el array de tiendas.
 *
 */
void m_shop_clear_array (Shop ** shops) {
    if (shops!=NULL) {
	gint index = 0;
	while (shops[index]!=NULL) {
	    d_shop_clear (shops[index]);
	    index++;
	}
    }
    g_free(shops);
    shops  = NULL;
}

/**
 * m_shop_clear_cached_array:
 *
 * Libera la memoria utilizada por la variable global @shops_list
 *
 */
void m_shop_clear_cached_array () {
    m_shop_clear_array (shops_list);
}


