/* supplier_price_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "product_model.h"
#include "piece_model.h"
#include "supplier_price_model.h"
#include "supplier_price.h"
#include "enterprise_model.h"
#include "participant_model.h"
#include "shop_model.h"
#include "warehouse_model.h"
#include "parameter_manager.h"

#define TABLE      "Pieza"
#define ATTRIBUTES "codigoProducto as codigoProductosupplierprice, \
    codigoProveedor as codigoProveedorSupplierPrice, precioCoste as precioCosteSupplierPrice"

/**
 * m_supplier_price_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_supplier_price_get_table () {
    return TABLE;
}

/**
 * m_supplier_price_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_supplier_price_get_attributes () {
    return ATTRIBUTES;
}


/**
 * m_supplier_price_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #SupplierPrice con los campos rellenados con los datos de la base de datos.
 */
SupplierPrice * m_supplier_price_create (GdaRecordset *rs) {
    GdaField * field = NULL;
    SupplierPrice * supplier_price = d_supplier_price_create_empty ();

    field = gda_recordset_field_name(rs,"codigoProductoProduct");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {

	    supplier_price->piece = m_piece_create (rs);
	    supplier_price->participant = m_participant_create (rs,"supplier");

	    gda_field_free(field);

	    field = gda_recordset_field_name(rs,"precioCostesupplierprice");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp;
		    temp=gda_stringify_value(NULL, 0, field);
		    u_points_to_commas (temp);
		    supplier_price->cost_price = atof(temp);
		    g_free(temp);
		}
		gda_field_free(field);
		}

	} else {
	    gda_field_free(field);
	}
    }


    return supplier_price;
}


/**
 * m_supplier_price_insert: 
 * @supplier_price: Pieza a insertar.
 *
 * Inserta el precio de coste de un producto para un proveedor en la base de datos.
 *
 * Returns:
 * %TRUE si se ha insertado correctamente, %FALSE en otro caso.
 */
gboolean m_supplier_price_insert (SupplierPrice *supplier_price) {
    gboolean result=TRUE;
    gchar * query=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set=NULL;
    GdaCommand * command = gda_command_new();
    GdaConnection * connection = m_model_get_connection ();
    Product product = d_piece_get_product (d_supplier_price_get_piece (supplier_price));
    gulong product_code = d_product_get_product_code (&product);
    gulong supplier_code = d_participant_get_code (d_supplier_price_get_participant (supplier_price));
    gdouble cost_price = u_round_to_decimal (d_supplier_price_get_cost_price (supplier_price),2);

    gchar * product_code_gchar=NULL;
    gchar * supplier_code_gchar=NULL;
    gchar * cost_price_gchar=NULL;

    gulong logged_shop_code;
    gulong logged_enterprise_code;
    gulong logged_warehouse_code;
    gchar * temp_name;

    temp_name = u_parameter_manager_get_parameter ("empresa");
    logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("tienda");
    logged_shop_code = m_shop_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("almacen");
    logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

    g_free(temp_name);


    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,
						    logged_enterprise_code,
						    logged_shop_code,
						    logged_warehouse_code);
    if (product_code!=ULONG_MAX) {
	if (supplier_code!=ULONG_MAX) {

	    product_code_gchar = g_strdup_printf("%lu",product_code);
	    supplier_code_gchar = g_strdup_printf("%lu",supplier_code);
	    cost_price_gchar = g_strdup_printf("%.2f",cost_price);
	    u_commas_to_points (cost_price_gchar);
	    query = g_strconcat(" SELECT * FROM PRECIOSCOSTE ",
				"WHERE CODIGOPRODUCTO=",product_code_gchar," and ",
				" CODIGOPROVEEDOR=",supplier_code_gchar,
				NULL);

	    result = m_model_command_execute (&record_set, query, connection,
					   command,tuples,0, TRUE);

	    g_print(_("\nQuery: %s\n"),query);
	    g_free(query);

	    if ((*tuples) == 0) {
		query = g_strconcat(" INSERT INTO PRECIOSCOSTE VALUES (",
				    product_code_gchar,",",
				    supplier_code_gchar,",",
				    cost_price_gchar,")",
				    NULL);
	    } else {
		query = g_strconcat(" UPDATE PRECIOSCOSTE SET PRECIOCOSTE=",cost_price_gchar,
				    " WHERE CODIGOPROVEEDOR=",supplier_code_gchar,
				    " AND CODIGOPRODUCTO=",product_code_gchar,
				   NULL);

	    }
	    g_print(_("\nInsert: %s\n"),query);
	    result = result && m_model_command_execute (&record_set, query, connection,
						     command,tuples,0, FALSE);


	}
    }
    result = m_model_execute_mutex_modification_logged (connection) && result;

    if (query!=NULL) g_free(query);
    if (tuples!=NULL) g_free(tuples);
    if (product_code_gchar!=NULL)
        g_free(product_code_gchar);
    if (supplier_code_gchar!=NULL)
        g_free(supplier_code_gchar);
    if (cost_price_gchar!=NULL)
        g_free(cost_price_gchar);

    if (record_set!=NULL)
	gda_recordset_free(record_set);

    igalia_command_free(command);

    return result;

}


/**
 * m_supplier_price_delete:
 * @supplier_price: Pieza que se borrar de la base de datos.
 *
 * Borra una pieza de la base de datos.
 *
 * Returns:
 * %TRUE si se ha borrado correctamente, %FALSE en otro caso.
 */
gboolean m_supplier_price_delete (SupplierPrice *supplier_price) {
  gboolean result = FALSE;
  gulong logged_shop_code;
  gulong logged_enterprise_code;
  gulong logged_warehouse_code;
  gchar * temp_name;
//  gulong product_code = get_codigo_producto_piece(d_supplier_price_get_piece (supplier_price));
  Product product = d_piece_get_product (d_supplier_price_get_piece (supplier_price));
  gulong product_code = d_product_get_product_code (&product);

  gulong supplier_code = d_participant_get_code (d_supplier_price_get_participant (supplier_price));
  gulong * tuples;
  GdaRecordset *record_set=NULL;
  gchar * product_code_gchar=NULL;
  gchar * supplier_code_gchar=NULL;
  gchar * query = NULL;
  temp_name = u_parameter_manager_get_parameter ("empresa");
  logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
  g_free(temp_name);
  temp_name = u_parameter_manager_get_parameter ("tienda");
  logged_shop_code = m_shop_get_code_from_name (temp_name);
  g_free(temp_name);
  temp_name = u_parameter_manager_get_parameter ("almacen");
  logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

  g_free(temp_name);
  
    if (d_product_get_is_piece ((Product *)supplier_price)) {
	GdaConnection *connection = m_model_get_connection ();
	GdaCommand    *command = gda_command_new();

	gda_command_set_connection(command,connection);
	m_model_connection_begin_mutex_transaction_logged (connection,logged_enterprise_code,logged_shop_code,logged_warehouse_code);

	product_code_gchar = g_strdup_printf("%lu",product_code);
	supplier_code_gchar = g_strdup_printf("%lu",supplier_code);
	
	query = g_strconcat(" DELETE FROM PRECIOSCOSTE ",
			    " WHERE CODIGOPROVEEDOR=",supplier_code_gchar,
			    " AND CODIGOPRODUCTO=",product_code_gchar
			   );

	result = result && m_model_command_execute (&record_set, query, connection,
						 command,tuples,0, FALSE);


	result = m_model_execute_mutex_modification_logged (connection) && result;
        g_free(product_code_gchar);
        g_free(supplier_code_gchar);
	igalia_command_free(command);
    }

    return result;
}

SupplierPrice ** model_find_prices_by_product(gulong product_code) {

    gint          index = 0;
    gulong        *tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaCommand    *my_command = NULL;
    gchar         *query;
    GdaConnection *my_connection = m_model_get_connection ();
    GdaRecordset  *record_set = NULL;
    SupplierPrice **supplier_prices = NULL;
    gboolean       correct = FALSE;
    gchar * product_code_gchar = g_strdup_printf("%lu",product_code);
    my_command = gda_command_new();

    gda_command_set_connection(my_command,my_connection);


    query = g_strconcat("SELECT participante.codigoParty as codigopartyparticipantsupplier, "
		     " nick as nickparticipantsupplier, "
		     " telefono as telefonoparticipantsupplier, "
		     " telefono2 as telefono2participantsupplier, "
		     " fax as faxparticipantsupplier, "
		     " email as emailparticipantsupplier, "
		     " calle as calleparticipantsupplier, "
		     " numero as numeroparticipantsupplier, "
		     " piso as pisoparticipantsupplier, "
		     " CP as cpparticipantsupplier, "
		     " municipio as municipioparticipantsupplier, "
		     " participante.nombre as nombreparticipantsupplier, "
		     " cif as cifparticipantsupplier, "
		     " provincia as provinciaparticipantsupplier, ",
		     " preciosCoste.codigoProducto as codigoProductoProduct, ",
		     " eurocode as eurocodePiece, ",
		     " to_char(precioscoste.precioCoste,'99999999D99') as precioCosteSupplierPrice ",
		     " from precioscoste, participante, pieza ",
		     " where precioscoste.codigoProveedor = participante.codigoParty and "
		     " precioscoste.codigoProducto = pieza.codigoProducto and "
			" precioscoste.codigoProducto = ",product_code_gchar,
                        " order by precioscoste.precioCoste ",
		     NULL);


    g_print(_("\nPrices: %s\n"),query);
    correct = m_model_command_execute (&record_set, query, my_connection,my_command, tuples, 0,TRUE);

    if (record_set!=NULL) {
	supplier_prices = (SupplierPrice **) g_malloc0 (sizeof(SupplierPrice *)*(*tuples+1));
        index=0;
	if (*tuples!=0) {
	    gda_recordset_move_first(record_set);
	    while (!gda_recordset_eof(record_set)) {
		supplier_prices[index] = m_supplier_price_create (record_set);
		gda_recordset_move_next(record_set);
		index++;
	    }
	}
	supplier_prices[index]=NULL;

    } else {
        supplier_prices =NULL;
    }

    gda_recordset_free(record_set);
    igalia_command_free(my_command);
    g_free(query);
    return supplier_prices;
}

/**
 * m_supplier_price_clear_array:
 * @supplier_prices: Array de precios que se desea liberar.
 *
 * Libera la memoria utilizada por un array de piezas en stock.
 *
 * Returns: 0
 */

gint m_supplier_price_clear_array (SupplierPrice ** supplier_prices) {
    gulong index = 0;

    if (supplier_prices!=NULL) {
	while(supplier_prices[index]!=NULL) {
	    d_supplier_price_clear (supplier_prices[index]);
            supplier_prices[index] = NULL;
	    index++;
	}
    }
    g_free(supplier_prices);
    supplier_prices=NULL;
    return 0;
}

/**
 * m_supplier_price_clear_array:
 * @supplier_price: Proveedor y precio de coste que se asignarn al producto.
 *
 * Asigna un precio de coste y un proveedor  a una pieza.
 *
 * Returns: %TRUE si se ha asingnado correctamente.
 */
gboolean m_supplier_price_assign (SupplierPrice *supplier_price) {
    gboolean result=TRUE;
    gchar * query=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set=NULL;
    GdaCommand * command = gda_command_new();
    GdaConnection * connection = m_model_get_connection ();
    Product product = d_piece_get_product (d_supplier_price_get_piece (supplier_price));
    gulong product_code = d_product_get_product_code (&product);
    gulong supplier_code = d_participant_get_code (d_supplier_price_get_participant (supplier_price));
    gdouble cost_price = u_round_to_decimal (d_supplier_price_get_cost_price (supplier_price),2);

    gchar * product_code_gchar=NULL;
    gchar * supplier_code_gchar=NULL;
    gchar * cost_price_gchar=NULL;

    gulong logged_shop_code;
    gulong logged_enterprise_code;
    gulong logged_warehouse_code;
    gchar * temp_name;

    temp_name = u_parameter_manager_get_parameter ("empresa");
    logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("tienda");
    logged_shop_code = m_shop_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("almacen");
    logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

    g_free(temp_name);


    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,
						    logged_enterprise_code,
						    logged_shop_code,
						    logged_warehouse_code);
    if (product_code!=ULONG_MAX) {
	if (supplier_code!=ULONG_MAX) {

	    product_code_gchar = g_strdup_printf("%lu",product_code);
	    supplier_code_gchar = g_strdup_printf("%lu",supplier_code);
	    cost_price_gchar = g_strdup_printf("%.2f",cost_price);
	    u_commas_to_points (cost_price_gchar);

	    query = g_strconcat(" UPDATE PIEZA SET PRECIOCOSTE=",cost_price_gchar,
				" , CODIGOPROVEEDOR=",supplier_code_gchar,
				" WHERE CODIGOPRODUCTO=",product_code_gchar,
				NULL);


	    g_print(_("\nUpdate: %s\n"),query);

	    result = result && m_model_command_execute (&record_set, query, connection,
						     command,tuples,0, FALSE);


	}
    }
    result = m_model_execute_mutex_modification_logged (connection) && result;

    if (query!=NULL) g_free(query);
    if (tuples!=NULL) g_free(tuples);
    if (product_code_gchar!=NULL)
        g_free(product_code_gchar);
    if (supplier_code_gchar!=NULL)
        g_free(supplier_code_gchar);
    if (cost_price_gchar!=NULL)
        g_free(cost_price_gchar);

    if (record_set!=NULL)
	gda_recordset_free(record_set);

    igalia_command_free(command);

    return result;


}


