
/* centrallog2todos.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "log2todo.h"

#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>

#include "parse_file.h"
#include "centrallog2todos.h"

#include "parameter_manager.h"
#include "warehouse_model.h"
#include "shop_model.h"
#include "enterprise_model.h"

#define R_LOG_EXTENSION   ".log"
#define R_BLOCK_SIZE   4*1024

#define R_ENTERPRISE_CODE_POSITION 0
#define R_SHOP_CODE_POSITION       1
#define R_WAREHOUSE_CODE_POSITION  2


static gchar   **get_path_from_codes(gulong enterprise_code, gulong shop_code,
				    gulong warehouse_code, gchar **file_names);
static gint     distributed_central_queries(gchar **shop_query_file_names,
					    const gchar *in_file_name);
static void     parse_central_transaction(FILE *in_file, gchar **shop_query_file_names);
static void     parse_shop_transaction(gchar *buffer, FILE *in_file, gchar **shop_query_file_names);
static gulong   get_code(gchar *buffer, gint partial_position);
static gint     is_central(const gchar *query);
static gint     is_unknown(const gchar *query);
static gboolean has_extension(const gchar *file_name, const gchar *extension);

static gchar **central_mark = NULL;
static gchar  *unknown_mark = NULL;

// returns TRUE if the file_name ends with the extension
gboolean has_extension(const gchar *file_name, const gchar *extension) {
    gboolean result = FALSE;
    gchar *temp;

    temp = strstr(file_name, extension);
    if (temp!=NULL) {
	result = (temp+strlen(extension)==file_name+strlen(file_name));
    }

    return result;
}

// takes the following code from the given position
// the n-tuple that conforms the big-code is
// (partial_code,..., partial_code)
static gulong get_code(gchar *buffer, gint partial_position) {
    gchar *temp1 = buffer;
    gchar *temp2 = buffer;
    gchar *gchar_code = (gchar *) g_malloc0(sizeof(gchar) * strlen(buffer));
    gulong code = ULONG_MAX;
    gint  position = 0;

    temp1 = strstr(buffer, "(");
    temp2 = temp1;

    while (position <= partial_position) {
	temp1 = temp2+1;
	temp2 = strstr(temp1, ",");
	position++;
    }

    if(temp2==NULL) {
	temp2 = strstr(buffer, "(");
	temp2 = strstr(temp2, ")");
    }

    memcpy(gchar_code, temp1, temp2-temp1);

    code = strtoul(gchar_code, NULL, 10);

    g_free(gchar_code);
    gchar_code=NULL;
    return code;
}

// returns TRUE if the query's identificator shows that
// it have been performed in the central DB
static gint is_central(const gchar *query) {
    gboolean result = FALSE;
    gint     index;
    gchar * central_enterprise = u_parameter_manager_get_parameter ("empresa_central");
    gchar * central_shop = u_parameter_manager_get_parameter ("tienda_central");
    gchar * central_warehouse = u_parameter_manager_get_parameter ("almacen_central");


    if (central_mark==NULL) {
	central_mark = (gchar **) g_malloc(sizeof(gchar *)*4);
//	central_mark[0] = g_strdup_printf("(%lu,%lu,%lu)",
//					  m_enterprise_get_code_from_name (R_ENTERPRISE_CENTRAL_NICK),
//					  m_shop_get_code_from_name (R_SHOP_CENTRAL_NICK),
//					  m_warehouse_get_code_from_name (R_WAREHOUSE_CENTRAL_NICK)
//					 );
	central_mark[0] = g_strdup_printf("(%lu,%lu,%lu)",
					  m_enterprise_get_code_from_name (central_enterprise),
					  m_shop_get_code_from_name (central_shop),
					  m_warehouse_get_code_from_name (central_warehouse)
					 );

	/* No se tienen en cuenta estos casos ya que cuando no hay tienda pero s hay almacn
	 de la central, se deben dispersar a las tiendas pertenecientes a la empresa R_ENTERPRISE_CENTRAL_NICK*/
	/*	central_mark[1] = g_strdup_printf("(%lu,%lu,%lu)",
					  m_enterprise_get_code_from_name (R_ENTERPRISE_CENTRAL_NICK),
					  ULONG_MAX,
					  m_warehouse_get_code_from_name (R_WAREHOUSE_CENTRAL_NICK)
					 );
					 */
//	central_mark[1] = g_strdup_printf("(%lu,%lu,%lu)",
//					  m_enterprise_get_code_from_name (R_ENTERPRISE_CENTRAL_NICK),
//					  m_shop_get_code_from_name (R_SHOP_CENTRAL_NICK),
//					  ULONG_MAX
//					 );
	central_mark[1] = g_strdup_printf("(%lu,%lu,%lu)",
					  m_enterprise_get_code_from_name (central_enterprise),
					  m_shop_get_code_from_name (central_shop),
					  ULONG_MAX
					 );
	central_mark[2] = g_strdup_printf("(%lu,%lu,%lu)",
					  m_enterprise_get_code_from_name (central_enterprise),
					  ULONG_MAX,
					  ULONG_MAX
					 );
	g_print(_("\nFirst mark:%s\n"),central_mark[0]);
        g_print(_("\nSecond mark:%s\n"),central_mark[1]);
	g_print(_("\nThird mark:%s\n"),central_mark[2]);

	central_mark[3] = NULL;
    }

    index = 0;

    while(!result && central_mark[index]!=NULL) {
	result = result || (strstr(query, central_mark[index])!=NULL);
	index++;
    }

    return result;
}

// If we can't "imagine" where this query have been performed
static gint is_unknown(const gchar *query) {

    if (unknown_mark == NULL) {
	unknown_mark= g_strdup_printf("(%lu,%lu,%lu)",
				      ULONG_MAX,
				      ULONG_MAX,
				      ULONG_MAX
				     );
    }
    return (strstr(query, unknown_mark)!=NULL);
}



// Returns the file_name that corresponds according the given codes.
// Only works ok if the assertion shop=warehouse is true, that is
// a warehouse is a set of pieces in a shop, so shop and warehouse
// are the same thing (but have distint codes)
//
static gchar **get_path_from_codes(gulong enterprise_code, gulong shop_code,
				  gulong warehouse_code, gchar **file_names) {
    gint   index = 0;
    gint index_of_result=0;
    gchar *enterprise_name = m_enterprise_get_name_from_code (enterprise_code);
    gchar *shop_name       = m_shop_get_name_from_code (shop_code);
    gchar **result = (gchar **) g_malloc0(sizeof(gchar *)*10);
    gchar *temp            = file_names[index];

    while(file_names[index]!=NULL) {
	if (enterprise_code!=ULONG_MAX) {
	    temp = strstr(file_names[index], enterprise_name);
	    if (temp!=NULL) {
		if(shop_code!=ULONG_MAX) {
		    temp = strstr(temp, shop_name);
		    if (temp!=NULL) {
			/*Fichero X,Y para la tienda (X,Y,Z)*/
			result[0] = file_names[index];
                        result[1] = NULL;
			g_free(enterprise_name);
			enterprise_name=NULL;
			g_free(shop_name);
			shop_name=NULL;
                        return result;
		    } else {
		    }

		} else {
                    /*Ficheros (X,Y1),(X,Y2),(X,Y3) para la tienda (X,ULONG_MAX,Z)*/
		    result[index_of_result]=file_names[index];
                    index_of_result++;
		}
	    } else {
		// The enterprise_name is not in the file_name
		// so we do nothing
	    }
	} else {
	    // enterprise_code == ULONG_MAX so there is something wrong
	    // in the query;  we do nothing
	}
	index++;
    }

    g_free(enterprise_name);
    enterprise_name=NULL;
    g_free(shop_name);
    shop_name=NULL;
    return result;
}

// Parse a transaction performed in central. we only are interested in
// spreadable queries, which will be write in all todo shop files
static void parse_central_transaction(FILE *in_file,
				      gchar **shop_query_file_names) {
    gint      old_position = 0;
    gint      new_position = 0;
    gint      write_position = 0;
    gchar    *buffer = (gchar *) g_malloc(sizeof(gchar)*R_LIMIT_SIZE);
    gchar    *buffer_to_print = NULL;
    gchar    *buffer_to_print_aux = NULL;
    gboolean  correct_execute = TRUE;
    gint      index = 0;
    FILE      *out_file;

    new_position = r_read_line (&buffer, old_position, in_file);

    while((new_position!=old_position)
	  && !strstr(buffer,R_END_TRANSACTION)
	  && (buffer!=NULL)
	  && correct_execute) {

	while((new_position!=old_position)
	      && !r_is_query (buffer)
	      && !r_is_value (buffer)
	      && !strstr(buffer,R_END_TRANSACTION)
	      && (buffer!=NULL)) {
	    new_position = r_read_line (&buffer, old_position, in_file);
	}

	if (!strstr(buffer,R_END_TRANSACTION) &&
	    !strstr(buffer,R_BEGIN_TRANSACTION)) {
	    if (new_position!=old_position) {
		if (r_is_query (buffer)) {
		    // Fix me!!! Ok, we insert this "if" because there are queries
		    // build with this form and they cause a glib error.
		    // When this queries do not exist remove this if.
		    if (!strstr(buffer, "INSERT INTO FALSE")) {
			if(r_is_spreadable (buffer)) {
			    buffer_to_print_aux = buffer_to_print;
			    buffer_to_print = (gchar *)
				realloc(buffer_to_print_aux,
					sizeof(gchar)*(write_position+new_position));
			    memcpy(buffer_to_print+write_position,buffer,
				   (new_position-old_position));
			    write_position += (new_position-old_position)-1;
			}
			// In a not spreadable query from the central we do nothing
		    }
		} else {
		    if (r_is_value (buffer)) {
			if(!strstr(buffer,R_CORRECT)) {
			    // The execution within de transaction was
			    // not performed
			    correct_execute = FALSE;
			    break;
			}
		    }
		}
		new_position = r_read_line (&buffer, old_position, in_file);
	    }
	}
    }

    if (correct_execute && (buffer_to_print!=NULL)
	&& strstr(buffer,R_CORRECT)) { //Transaction is OK
	index = 0;
	while(shop_query_file_names[index]!=NULL) {
	    // The queries are spreadables so we write them
	    // in all shop TODO files
	    out_file = fopen(shop_query_file_names[index], "a");
	    if(out_file==NULL) {
		perror("replication/centrallog2todos.c/parse_central_transaction:fopen");
		printf(_("\tTrying open: %s\n"), shop_query_file_names[index]);
                m_model_destroy_connection ();
		exit(-1);
	    }
	    fprintf(out_file, "%s", buffer_to_print);
	    fclose(out_file);
	    index++;
	}
    }

    if (buffer!=NULL) {
	g_free(buffer);
	buffer=NULL;
    }

    if (buffer_to_print!=NULL) {
	g_free(buffer_to_print);
	buffer_to_print=NULL;
    }
}


// In a transaction of a shop performed in central,
// the transaction can have two query types: spreadable queries and not
// spreadable queries. Spreadable queries have to be written in all todo
// shop files (the shop do not know the change). Not spreadable queries
// only be written in the suitable todo shop file.

static void parse_shop_transaction(gchar *query, FILE *in_file,
				   gchar **shop_query_file_names) {
    gint      old_position = 0;
    gint      new_position = 0;
    gint      write_position = 0;
    gint      write_position_spreadable = 0;
    gchar    *buffer = (gchar *) g_malloc(sizeof(gchar)*R_LIMIT_SIZE);
    gchar    *buffer_to_print = NULL;
    gchar    *buffer_to_print_aux = NULL;
    gchar    *buffer_to_print_spreadable = NULL;
    gchar    *buffer_to_print_spreadable_aux = NULL;
    gboolean  correct_execute = TRUE;
    gint      index = 0;
    FILE      *out_file;
    gchar     **path = NULL;

    // Get the path according transaction codes.
    // Shop where would be performed the transaction
    path = get_path_from_codes(get_code(query, R_ENTERPRISE_CODE_POSITION),
			       get_code(query, R_SHOP_CODE_POSITION),
			       get_code(query, R_WAREHOUSE_CODE_POSITION),
			       shop_query_file_names);

    new_position = r_read_line (&buffer, old_position, in_file);

    while((new_position!=old_position)
	  && !strstr(buffer,R_END_TRANSACTION)
	  && (buffer!=NULL)
	  && correct_execute) {

	while((new_position!=old_position)
	      && !r_is_query (buffer)
	      && !r_is_value (buffer)
	      && !strstr(buffer,R_END_TRANSACTION)
	      && (buffer!=NULL)) {
	    new_position = r_read_line (&buffer, old_position, in_file);
	}

	if (new_position!=old_position) {
	    if (r_is_query (buffer)) {
		// Fix me!!! Ok, we insert this "if" because there are queries
		// build with this form and they cause a glib error.
		// When this queries do not exist remove this if.
		if (!strstr(buffer, "INSERT INTO FALSE")) {
		    if(r_is_spreadable (buffer)) {
			buffer_to_print_spreadable_aux = buffer_to_print_spreadable;
			buffer_to_print_spreadable = (gchar *)
			    realloc(buffer_to_print_spreadable_aux,
				    sizeof(gchar)*(write_position_spreadable+new_position));
			memcpy(buffer_to_print_spreadable+write_position_spreadable,buffer,
			       (new_position-old_position));
			write_position_spreadable += (new_position-old_position)-1;
			buffer_to_print_spreadable_aux = NULL;
		    } else {
			buffer_to_print_aux = buffer_to_print;
			buffer_to_print = (gchar *) realloc(buffer_to_print_aux,
							    sizeof(gchar)*(write_position+new_position));
			memcpy(buffer_to_print+write_position,buffer,
			       (new_position-old_position));
			write_position += (new_position-old_position)-1;
			buffer_to_print_aux = NULL;
		    }
		}
	    } else {
		if (r_is_value (buffer)) {
		    if (!strstr(buffer,R_END_TRANSACTION) &&
			!strstr(buffer,R_BEGIN_TRANSACTION)) {
			if(!strstr(buffer,R_CORRECT)) { //The execution within de transaction was not performed
			    correct_execute = FALSE;
			    break;
			}
		    }
		}
	    }
	    new_position = r_read_line (&buffer, old_position, in_file);
	}
    }

    if (correct_execute && strstr(buffer,R_CORRECT)) { //Transaction is OK
	if(buffer_to_print_spreadable!=NULL) {
	    index = 0;
	    while(shop_query_file_names[index]!=NULL) {
		// The queries are spreadables so we write them in all shop TODO files
		out_file = fopen(shop_query_file_names[index], "a");
		if(in_file==NULL) {
		    perror("replication/centrallog2todos.c/parse_shop_transaction:fopen");
		    printf(_("\tTrying open: %s\n"), shop_query_file_names[index]);
                    m_model_destroy_connection ();
		    exit(-1);
		}
		fprintf(out_file, "%s", buffer_to_print_spreadable);
		fclose(out_file);
		index++;
	    }
	}

	// If there are not spreadable queries in the transaction,
	// write them in the suitable file.
	// NOW!!!, the suitable files are the files belong to their enterprise,
	// so there is a bucle while in which we compare the paths.

	if (buffer_to_print!=NULL) {
            gint index_of_path = 0;
	    while (path[index_of_path]!=NULL) {
		out_file = fopen(path[index_of_path], "a");

		if(out_file==NULL) {
		    perror("replication/centrallog2todos.c/parse_shop_transaction:fopen");
		    printf(_("\tTrying open: %s\n"), path[index_of_path]);
                    m_model_destroy_connection ();
		    exit(-1);
		}

		fprintf(out_file, "%s", buffer_to_print);
		fclose(out_file);
                index_of_path++;
	    }
	}
    }

    if (buffer!=NULL) {
	g_free(buffer);
	buffer=NULL;
    }

    if (buffer_to_print!=NULL) {
	g_free(buffer_to_print);
	buffer_to_print=NULL;
    }
    if (buffer_to_print_spreadable!=NULL) {
	g_free(buffer_to_print_spreadable);
	buffer_to_print_spreadable=NULL;
    }
}

// We work here with central log file transaction by transaction.
// They can be shop transactions or central transaction.
// We work with them independly
static gint distributed_central_queries(gchar **shop_query_file_names,
					const gchar *in_file_name) {
    FILE   *in_file  = NULL;
    gint    old_position = 0;
    gint    new_position = 0;
    gchar  *buffer = (gchar *) g_malloc(sizeof(gchar)*R_LIMIT_SIZE);
    gchar  *buffer_to_print;


    in_file  = fopen(in_file_name, "r");

    if(in_file==NULL) {
	perror("replication/centrallog2todos.c/distributed_central_queries:fopen");
	printf(_("\tTrying open: %s\n"), in_file_name);
        m_model_destroy_connection ();
	exit(-1);
    }

    if (r_parse_file (in_file_name)!=0) {
	perror("replication/centrallog2todos.c/distributed_central_queries:parse_file");
	printf(_("\tIncorrect file parsing: %s\n"), in_file_name);
        m_model_destroy_connection ();
	exit(-1);
    }

    do {
	new_position = r_read_line (&buffer, old_position, in_file);

	while((new_position!=old_position)
	      && !r_is_query (buffer)
	      && !strstr(buffer, R_BEGIN_TRANSACTION)
	      && (buffer!=NULL)) {

	    new_position = r_read_line (&buffer, old_position, in_file);
	}

	if ((buffer!=NULL) && (new_position!=old_position)) {
	    if (strstr(buffer, R_BEGIN_TRANSACTION)) {
		if (is_central(buffer) || is_unknown(buffer)) {
		    parse_central_transaction(in_file, shop_query_file_names);
		} else {
		    parse_shop_transaction(buffer, in_file, shop_query_file_names);
		}
	    } else {
		if (new_position!=old_position) {
		    buffer_to_print = g_strdup(buffer);
		    new_position = r_read_line (&buffer, old_position, in_file);
		    while((new_position!=old_position)
			  && (!r_is_value (buffer))
			  && (buffer!=NULL)) {
			new_position = r_read_line (&buffer, old_position, in_file);
		    }

		    if((new_position!=old_position)
		       && strstr(buffer, R_CORRECT)
		       && buffer!=NULL) {
			// A correct query executed in the central.
		    }
		    g_free(buffer_to_print);
		    buffer_to_print=NULL;
		}
	    }
	}
    } while(new_position!=old_position);

    g_free(buffer);
    buffer=NULL;
    fclose(in_file);

    return 0;
}

/**
 * r_central_log_to_todos:
 * @shop_directory_name: Cadena que contiene el path al directorio en el que se encuentran los
 * ficheros de las tiendas.
 * @log_central_file_name: Nombre completo del fichero de log de la central.
 *
 * Funcin que convierte el ficheros de log de la central en los todos de las tiendas.
 * Para ello, primero obtiene los nombres de los ficheros ".todo" que debe rellenar (estos ficheros no
 * deben existir previamente). Distribuye las setencias sql ejecutadas correctamente dentro de las transacciones
 * entre los ficheros ".todo" . Las sentencias sobre tablas dispersables se meten en todos los ficheros ".todo" y
 * las sentencias sobre tablas no dispersables por las bases de datos, se mandan al ".todo" de la tienda adecuada.
 *
 * Returns:
 * 0 si se ha acabado correctamente.
 */
gint r_central_log_to_todos (const gchar *shop_directory_name,
		      const gchar *log_central_file_name) {
    DIR           *shop_directory = NULL;
    struct dirent *dir_entry     = NULL;
    struct stat   *statbuf;
    gchar         *shop_file_name  = NULL;
    gchar        **file_name_query_array = (gchar **) g_malloc(sizeof(gchar *)*2);
    gchar        **aux   = NULL;
    gchar         *temp  = NULL;
    gint           files = 0;
    gint           index = 0;

    if ((shop_directory = opendir(shop_directory_name))==NULL) {
	perror("replication/generate_todos.c/r_central_log_to_todos:opendir");
	g_print(_("Trying open: %s"), shop_directory_name);
        m_model_destroy_connection ();
	exit(-1);
    }

    // Gets all shop todo file_names
    dir_entry = readdir(shop_directory);
    while (dir_entry!=NULL) {

	statbuf      = (struct stat *) g_malloc(sizeof(struct stat));

	if (shop_directory_name[strlen(shop_directory_name)-1]=='/') {
	    temp = g_strconcat(shop_directory_name, dir_entry->d_name, NULL);
	} else {
	    temp = g_strconcat(shop_directory_name, "/", dir_entry->d_name, NULL);
	}

	stat(temp, statbuf);
	g_free(temp);
	temp = NULL;

	if (S_ISDIR(statbuf->st_mode)) {
	    dir_entry = readdir(shop_directory);
	    g_free(statbuf);
	    statbuf = NULL;
	    continue;
	}

	if(!has_extension(dir_entry->d_name, R_TODO_EXTENSION)) {
	    dir_entry = readdir(shop_directory);
	    g_free(statbuf);
	    statbuf = NULL;
	    continue;
	}

	g_free(statbuf);
	statbuf = NULL;
	files++;
	aux = file_name_query_array;
	file_name_query_array = (gchar **)realloc(aux, sizeof(gchar *)*(files+1));

	if (shop_directory_name[strlen(shop_directory_name)-1]=='/') {
	    shop_file_name = g_strdup_printf("%s%s", shop_directory_name, dir_entry->d_name);
	} else {
	    shop_file_name = g_strdup_printf("%s/%s", shop_directory_name, dir_entry->d_name);
	}

	file_name_query_array[files-1] = shop_file_name;

	dir_entry = readdir(shop_directory);
    }
    file_name_query_array[files] = NULL;

    // Here we already have all shop todo file_names

    ////////////////////////////////////////////////////////////
    distributed_central_queries(file_name_query_array,
				log_central_file_name);
    ////////////////////////////////////////////////////////////

    // Liberations
    files = 0;
    while(file_name_query_array[files]!=NULL) {
	g_free(file_name_query_array[files]);
	file_name_query_array[files] = NULL;
	files++;
    }
    g_free(file_name_query_array);
    file_name_query_array=NULL;

    if (closedir(shop_directory)==-1) {
	perror("replication/generate_todos.c/centrallogtodo:closedir");
        m_model_destroy_connection ();
	exit(-1);
    }

    index = 0;
    if(central_mark!=NULL) {
	while(central_mark[index]!=NULL) {
	    g_free(central_mark[index]);
	    central_mark[index] = NULL;
	    index++;
	}
	g_free(central_mark);
	central_mark=NULL;
    }

    return 0;
}

