/* shoplogs2todos.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "shoplogs2todos.h"

#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>

#include "parse_file.h"

#include "log2todo.h"

static void distribute_querys(gchar **file_name_query_array,
                              gchar **file_name_todo_array,
			      const gchar *todo_central_file_name);

// Here we distributed all query_files cross the todo_files
// we write all queries, in the todo central file whereas
// we write only spreadable queries in todo shop files
static void distribute_querys(gchar **file_name_query_array,
                              gchar **file_name_todo_array,
			      const gchar *todo_central_file_name) {
    gint   query_file = 0;
    gint   todo_file = 0;
    gchar *temp;

    // Save all changes into the central TODO file
    while(file_name_query_array[query_file]!=NULL) {
	r_copy (file_name_query_array[query_file], todo_central_file_name);
	query_file++;
    }

    query_file = 0;

    // Save changes from de query files into the suitable TODO file.
    // Only spreadable queries.
    // i.e. for query_files A.query, B.query and
    // for todo_files A.todo, B.todo and C.todo
    // A.query file is writing into B.todo and C.todo files and, finally,
    // B.query file is writing into A.todo and C.todo files 

    while(file_name_query_array[query_file]!=NULL) {
	temp = g_strdup(strstr(file_name_todo_array[todo_file],
			       file_name_query_array[query_file]));

	if(temp!=NULL) {
	    todo_file++;
            g_free(temp);
	    if (file_name_todo_array[todo_file]==NULL) {
		query_file++;
		if (file_name_query_array[query_file]!=NULL) {
		    todo_file = 0;
		}
	    }
	    continue;
	}
	r_copy_spreadable (file_name_query_array[query_file], file_name_todo_array[todo_file]);

	todo_file++;
	if (file_name_todo_array[todo_file]==NULL) {
	    query_file++;
	    if (file_name_query_array[query_file]!=NULL) {
		todo_file = 0;
	    }
	}
    }
}

// Get all log files, convert it in query files and these
// will have been convert in the suitable todo files.

/**
 * r_shop_logs_to_todos:
 * @log_shop_directory_name: Directorio que contiene los ficheros de log de las tiendas.
 * @todo_shop_directory_name: Directorio que contendr los "todo"'s de las tiendas.
 * @todo_central_file_name: Path completo que ser el todo de la central.
 *
 * Parsea todos los log's de las tiendas y distribuye las sentencias sql a los ficheros "todo" que le corresponden.
 *
 * Returns:
 * 1 si se ha finalizado correctamente.
 */
gint r_shop_logs_to_todos (const gchar *log_shop_directory_name,
		    const gchar *todo_shop_directory_name,
		    const gchar *todo_central_file_name) {

    DIR           *log_shop_directory = NULL;
    DIR           *todo_shop_directory = NULL;
    struct dirent *direntry     = NULL;
    struct stat   *statbuf;
    gint     files = 0;
    gchar  **aux   = NULL;
    gchar   *temp  = NULL;
    gchar   *in_file_name  = NULL;
    gchar  **file_name_query_array = (gchar **) g_malloc(sizeof(gchar *)*2);
    gchar  **file_name_todo_array = (gchar **) g_malloc(sizeof(gchar *)*2);

    // Let's get all todo files...
    if ((todo_shop_directory = opendir(todo_shop_directory_name))==NULL) {
	perror("replication/shoplogs2todos/shoplogs2todos.c:opendir");
        m_model_destroy_connection ();
	exit(-1);
    }

    direntry = readdir(todo_shop_directory);
    while (direntry!=NULL) {

	statbuf      = (struct stat *) g_malloc(sizeof(struct stat));
	if (todo_shop_directory_name[strlen(todo_shop_directory_name)-1]=='/') {
	    temp = g_strconcat(todo_shop_directory_name, direntry->d_name, NULL);
	} else {
	    temp = g_strconcat(todo_shop_directory_name, "/", direntry->d_name, NULL);
	}
	stat(temp, statbuf);
	g_free(temp);

	if (S_ISDIR(statbuf->st_mode)) {
	    direntry = readdir(todo_shop_directory);
	    g_free(statbuf);
	    continue;
	}

	g_free(statbuf);
	files++;
	aux = file_name_todo_array;
	file_name_todo_array = (gchar **)realloc(aux, sizeof(gchar *)*(files+1));

	if (todo_shop_directory_name[strlen(todo_shop_directory_name)-1]=='/') {
	    temp = g_strconcat(todo_shop_directory_name, direntry->d_name, NULL);
	} else {
	    temp = g_strconcat(todo_shop_directory_name, "/", direntry->d_name, NULL);
	}

	file_name_todo_array[files-1] = g_strdup(temp);
        g_free(temp);

	direntry = readdir(todo_shop_directory);
    }

    file_name_todo_array[files] = NULL;
    if (closedir(todo_shop_directory)==-1) {
	perror("replication/shoplogs2todos/shoplogs2todos.c:closedir");
        m_model_destroy_connection ();
	exit(-1);
    }


    // We already have all todo files. Let's get query files...

    files=0;
    file_name_query_array[files] = NULL;
    if ((log_shop_directory = opendir(log_shop_directory_name))==NULL) {
	perror("replication/shoplogs2todos/shoplogs2todos.c:opendir");
        m_model_destroy_connection ();
	exit(-1);
    }

    direntry = readdir(log_shop_directory);
    while (direntry!=NULL) {

	statbuf      = (struct stat *) g_malloc(sizeof(struct stat));
	if (log_shop_directory_name[strlen(log_shop_directory_name)-1]=='/') {
	    temp = g_strconcat(log_shop_directory_name, direntry->d_name, NULL);
	} else {
	    temp = g_strconcat(log_shop_directory_name, "/", direntry->d_name, NULL);
	}
	stat(temp, statbuf);
	g_free(temp);

	if (S_ISDIR(statbuf->st_mode)) {
	    direntry = readdir(log_shop_directory);
	    g_free(statbuf);
	    continue;
	}

	g_free(statbuf);
	files++;
	aux = file_name_query_array;
	file_name_query_array = (gchar **)realloc(aux, sizeof(gchar *)*(files+1));

	if (log_shop_directory_name[strlen(log_shop_directory_name)-1]=='/') {
	    in_file_name = g_strdup_printf("%s%s", log_shop_directory_name, direntry->d_name);
	} else {
	    in_file_name = g_strdup_printf("%s/%s", log_shop_directory_name, direntry->d_name);
	}

	if (todo_shop_directory_name[strlen(todo_shop_directory_name)-1]=='/') {
	    temp = g_strconcat(todo_shop_directory_name, direntry->d_name, NULL);
	} else {
	    temp = g_strconcat(todo_shop_directory_name, "/", direntry->d_name, NULL);
	}

	file_name_query_array[files-1] = g_strconcat(temp, R_QUERY_EXTENSION, NULL);
        g_free(temp);

	// Transforms all log files in todo files
	if (r_parse_file (in_file_name)!=0) {
	    perror("replication/shoplogs2todos.c/r_shop_logs_to_todos:parse_file");
	    printf(_("\tIncorrect file parsing: %s\n"), in_file_name);
            m_model_destroy_connection ();
	    exit(-1);
	}
        
	r_log_to_todo (in_file_name, file_name_query_array[files-1]);

	direntry = readdir(log_shop_directory);
	g_free(in_file_name);
    }
    file_name_query_array[files] = NULL;
    if (closedir(log_shop_directory)==-1) {
	perror("replication/shoplogs2todos/shoplogs2todos.c:closedir");
        m_model_destroy_connection ();
	exit(-1);
    }

    // Here we distributed all query_files cross the todo_files
    distribute_querys(file_name_query_array, file_name_todo_array, todo_central_file_name);

    // Liberations
    files = 0;
    while(file_name_query_array[files]!=NULL) {
	g_free(file_name_query_array[files]);
	file_name_query_array[files] = NULL;
	files++;
    }
    g_free(file_name_query_array);

    return 1;

}
